@@ -30,6 +30,12 @@ int main(void)
         gnutls_srp_client_credentials_t srp_cred;
         gnutls_certificate_credentials_t cert_cred;
 
+        if (gnutls_check_version("3.1.4") == NULL) {
+                fprintf(stderr, "GnuTLS 3.1.4 or later is required for this example\n");
+                exit(1);
+        }
+
+        /* for backwards compatibility with gnutls < 3.3.0 */
         gnutls_global_init();
 
         gnutls_srp_allocate_client_credentials(&srp_cred);
@@ -51,7 +57,8 @@ int main(void)
         /* Set the priorities.
          */
         gnutls_priority_set_direct(session,
-                                   "NORMAL:+SRP:+SRP-RSA:+SRP-DSS", NULL);
+                                   "NORMAL:+SRP:+SRP-RSA:+SRP-DSS",
+                                   NULL);
 
         /* put the SRP credentials to the current session
          */
