@@ -41,11 +41,13 @@ struct gnutls_privkey_st {
 			gnutls_privkey_sign_func sign_func;
 			gnutls_privkey_decrypt_func decrypt_func;
 			gnutls_privkey_deinit_func deinit_func;
+			gnutls_privkey_info_func info_func;
 			void *userdata;
 		} ext;
 	} key;
 
 	unsigned int flags;
+	gnutls_sign_algorithm_t preferred_sign_algo;
 	struct pin_info_st pin;
 };
 
@@ -85,7 +87,6 @@ int _gnutls_pubkey_compatible_with_sig(gnutls_session_t,
 				       gnutls_pubkey_t pubkey,
 				       const version_entry_st * ver,
 				       gnutls_sign_algorithm_t sign);
-int _gnutls_pubkey_is_over_rsa_512(gnutls_pubkey_t pubkey);
 int
 _gnutls_pubkey_get_mpis(gnutls_pubkey_t key, gnutls_pk_params_st * params);
 
@@ -108,4 +109,10 @@ const mac_entry_st *_gnutls_dsa_q_to_hash(gnutls_pk_algorithm_t algo,
 					  const gnutls_pk_params_st *
 					  params, unsigned int *hash_len);
 
+int
+_gnutls_privkey_get_mpis(gnutls_privkey_t key, gnutls_pk_params_st * params);
+
+gnutls_sign_algorithm_t
+_gnutls_privkey_get_preferred_sign_algo(gnutls_privkey_t key);
+
 #endif
