@@ -28,7 +28,6 @@
 
 #ifdef ENABLE_CRYPTODEV
 
-#include <assert.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <crypto/cryptodev.h>
@@ -91,8 +90,6 @@ cryptodev_cipher_setkey(void *_ctx, const void *key, size_t keysize)
 {
 	struct cryptodev_ctx *ctx = _ctx;
 
-	CHECK_AES_KEYSIZE(keysize);
-
 	ctx->sess.keylen = keysize;
 	ctx->sess.key = (void *) key;
 
@@ -214,7 +211,7 @@ static int register_crypto(int cfd)
 				  gnutls_cipher_get_name(i));
 		ret =
 		    gnutls_crypto_single_cipher_register(i, 90,
-							 &cipher_struct);
+							 &cipher_struct, 0);
 		if (ret < 0) {
 			gnutls_assert();
 			return ret;
@@ -299,15 +296,12 @@ static const int gnutls_mac_map[] = {
 
 static int
 cryptodev_mac_fast(gnutls_mac_algorithm_t algo,
-		   const void *nonce, size_t nonce_size,
 		   const void *key, size_t key_size, const void *text,
 		   size_t text_size, void *digest)
 {
 	struct cryptodev_ctx ctx;
 	int ret;
 
-	assert(nonce_size == 0);
-
 	memset(&ctx, 0, sizeof(ctx));
 	ctx.cfd = _gnutls_cryptodev_fd;
 	ctx.sess.mac = gnutls_mac_map[algo];
@@ -436,7 +430,7 @@ static int register_mac_digest(int cfd)
 		ioctl(cfd, CIOCFSESSION, &sess.ses);
 
 		ret =
-		    gnutls_crypto_single_mac_register(i, 90, &mac_struct);
+		    gnutls_crypto_single_mac_register(i, 90, &mac_struct, 0);
 		if (ret < 0) {
 			gnutls_assert();
 			return ret;
@@ -473,7 +467,7 @@ static int register_mac_digest(int cfd)
 				  gnutls_mac_get_name(i));
 		ret =
 		    gnutls_crypto_single_digest_register(i, 90,
-							 &digest_struct);
+							 &digest_struct, 0);
 		if (ret < 0) {
 			gnutls_assert();
 			return ret;
