@@ -47,14 +47,14 @@ static const gnutls_sec_params_entry sec_params[] = {
 	{"Legacy", GNUTLS_SEC_PARAM_LEGACY, 96, 1024, 1024, 192, 192},
 	{"Medium", GNUTLS_SEC_PARAM_MEDIUM, 112, 2048, 2048, 224, 224},
 	{"High", GNUTLS_SEC_PARAM_HIGH, 128, 3072, 3072, 256, 256},
-	{"Ultra", GNUTLS_SEC_PARAM_ULTRA, 256, 15360, 15360, 512, 512},
 #else
 	{"Low", GNUTLS_SEC_PARAM_LOW, 80, 1024, 1024, 160, 160}, /* ENISA-LEGACY */
 	{"Legacy", GNUTLS_SEC_PARAM_LEGACY, 96, 1776, 2048, 192, 192},
 	{"Medium", GNUTLS_SEC_PARAM_MEDIUM, 112, 2048, 2048, 256, 224},
 	{"High", GNUTLS_SEC_PARAM_HIGH, 128, 3072, 3072, 256, 256},
-	{"Ultra", GNUTLS_SEC_PARAM_ULTRA, 256, 15424, 3072, 512, 512},
 #endif
+	{"Ultra", GNUTLS_SEC_PARAM_ULTRA, 192, 8192, 8192, 384, 384},
+	{"Future", GNUTLS_SEC_PARAM_FUTURE, 256, 15360, 15360, 512, 512},
 	{NULL, 0, 0, 0, 0, 0}
 };
 
@@ -131,10 +131,9 @@ unsigned int _gnutls_pk_bits_to_subgroup_bits(unsigned int pk_bits)
 	unsigned int ret = 0;
 
 	GNUTLS_SEC_PARAM_LOOP(
-		if (p->pk_bits >= pk_bits) {
-			ret = p->subgroup_bits; 
+		ret = p->subgroup_bits; 
+		if (p->pk_bits >= pk_bits)
 			break;
-		}
 	);
 	return ret;
 }
