@@ -17,13 +17,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
 #include <config.h>
-#include "gnutls_int.h"
-#include "errors.h"
+#include <gnutls_int.h>
+#include <gnutls_errors.h>
 
 #include <sys/socket.h>
 #include <errno.h>
@@ -35,16 +35,15 @@ unsigned int _gnutls_forkid = 0;
 
 #ifndef _WIN32
 
-#ifdef HAVE_ATFORK
+# ifdef HAVE_ATFORK
 static void fork_handler(void)
 {
 	_gnutls_forkid++;
 }
-#endif
+# endif
 
-#if defined(HAVE___REGISTER_ATFORK)
-extern int __register_atfork(void (*)(void), void (*)(void), void (*)(void),
-			     void *);
+# if defined(HAVE___REGISTER_ATFORK)
+extern int __register_atfork(void (*)(void), void(*)(void), void (*)(void), void *);
 extern void *__dso_handle;
 
 int _gnutls_register_fork_handler(void)
@@ -54,7 +53,7 @@ int _gnutls_register_fork_handler(void)
 	return 0;
 }
 
-#else
+# else
 
 unsigned int _gnutls_get_forkid(void)
 {
@@ -63,7 +62,7 @@ unsigned int _gnutls_get_forkid(void)
 
 int _gnutls_detect_fork(unsigned int forkid)
 {
-	if ((unsigned int)getpid() == forkid)
+	if (getpid() == forkid)
 		return 0;
 	return 1;
 }
@@ -75,6 +74,6 @@ int _gnutls_register_fork_handler(void)
 	return 0;
 }
 
-#endif
+# endif
 
 #endif /* !_WIN32 */
