@@ -92,16 +92,16 @@ static int calc_ecdh_key(gnutls_session_t session,
 	gnutls_pk_params_st pub;
 	int ret;
 
-	memset(&pub, 0, sizeof(pub));
+	gnutls_pk_params_init(&pub);
 	pub.params[ECC_X] = session->key.ecdh_x;
 	pub.params[ECC_Y] = session->key.ecdh_y;
 	pub.flags = curve;
 
-	if (psk_key == NULL)
+	if (psk_key == NULL) {
 		ret =
 		    _gnutls_pk_derive(GNUTLS_PK_EC, &session->key.key,
 				      &session->key.ecdh_params, &pub);
-	else {
+	} else {
 		gnutls_datum_t tmp_dh_key;
 
 		ret =
@@ -115,7 +115,7 @@ static int calc_ecdh_key(gnutls_session_t session,
 		ret =
 		    _gnutls_set_psk_session_key(session, psk_key,
 						&tmp_dh_key);
-		_gnutls_free_datum(&tmp_dh_key);
+		_gnutls_free_temp_key_datum(&tmp_dh_key);
 	}
 
 	if (ret < 0) {
@@ -154,15 +154,21 @@ int _gnutls_proc_ecdh_common_client_kx(gnutls_session_t session,
 	    _gnutls_ecc_ansi_x963_import(&data[i], point_size,
 					 &session->key.ecdh_x,
 					 &session->key.ecdh_y);
-	if (ret < 0)
-		return gnutls_assert_val(ret);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
 
 	/* generate pre-shared key */
 	ret = calc_ecdh_key(session, psk_key, curve);
-	if (ret < 0)
-		return gnutls_assert_val(ret);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
 
-	return 0;
+cleanup:
+      	gnutls_pk_params_clear(&session->key.ecdh_params);
+	return ret;
 }
 
 static int
@@ -172,7 +178,7 @@ proc_ecdhe_client_kx(gnutls_session_t session,
 	gnutls_certificate_credentials_t cred;
 
 	cred = (gnutls_certificate_credentials_t)
-	    _gnutls_get_cred(session, GNUTLS_CRD_CERTIFICATE, NULL);
+	    _gnutls_get_cred(session, GNUTLS_CRD_CERTIFICATE);
 	if (cred == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
@@ -202,8 +208,8 @@ _gnutls_gen_ecdh_common_client_kx_int(gnutls_session_t session,
 
 	/* generate temporal key */
 	ret =
-	    _gnutls_pk_generate(GNUTLS_PK_EC, curve,
-				&session->key.ecdh_params);
+	    _gnutls_pk_generate_keys(GNUTLS_PK_EC, curve,
+				     &session->key.ecdh_params);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
@@ -213,23 +219,32 @@ _gnutls_gen_ecdh_common_client_kx_int(gnutls_session_t session,
 					 params[ECC_X] /* x */ ,
 					 session->key.ecdh_params.
 					 params[ECC_Y] /* y */ , &out);
-	if (ret < 0)
-		return gnutls_assert_val(ret);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
 
 	ret =
 	    _gnutls_buffer_append_data_prefix(data, 8, out.data, out.size);
 
 	_gnutls_free_datum(&out);
 
-	if (ret < 0)
-		return gnutls_assert_val(ret);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
 
 	/* generate pre-shared key */
 	ret = calc_ecdh_key(session, psk_key, curve);
-	if (ret < 0)
-		return gnutls_assert_val(ret);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
 
-	return data->length;
+	ret = data->length;
+cleanup:
+      	gnutls_pk_params_clear(&session->key.ecdh_params);
+	return ret;
 }
 
 static int
@@ -259,6 +274,11 @@ _gnutls_proc_ecdh_common_server_kx(gnutls_session_t session,
 	gnutls_ecc_curve_t curve;
 	ssize_t data_size = _data_size;
 
+	/* just in case we are resuming a session */
+	gnutls_pk_params_release(&session->key.ecdh_params);
+
+	gnutls_pk_params_init(&session->key.ecdh_params);
+
 	i = 0;
 	DECR_LEN(data_size, 1);
 	if (data[i++] != 3)
@@ -304,6 +324,11 @@ int _gnutls_ecdh_common_print_server_kx(gnutls_session_t session,
 	if (curve == GNUTLS_ECC_CURVE_INVALID)
 		return gnutls_assert_val(GNUTLS_E_ECC_NO_SUPPORTED_CURVES);
 
+	/* just in case we are resuming a session */
+	gnutls_pk_params_release(&session->key.ecdh_params);
+
+	gnutls_pk_params_init(&session->key.ecdh_params);
+
 	/* curve type */
 	p = 3;
 
@@ -320,8 +345,8 @@ int _gnutls_ecdh_common_print_server_kx(gnutls_session_t session,
 
 	/* generate temporal key */
 	ret =
-	    _gnutls_pk_generate(GNUTLS_PK_EC, curve,
-				&session->key.ecdh_params);
+	    _gnutls_pk_generate_keys(GNUTLS_PK_EC, curve,
+				     &session->key.ecdh_params);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
@@ -352,7 +377,7 @@ gen_ecdhe_server_kx(gnutls_session_t session, gnutls_buffer_st * data)
 	gnutls_certificate_credentials_t cred;
 
 	cred = (gnutls_certificate_credentials_t)
-	    _gnutls_get_cred(session, GNUTLS_CRD_CERTIFICATE, NULL);
+	    _gnutls_get_cred(session, GNUTLS_CRD_CERTIFICATE);
 	if (cred == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
@@ -360,7 +385,7 @@ gen_ecdhe_server_kx(gnutls_session_t session, gnutls_buffer_st * data)
 
 	if ((ret = _gnutls_auth_info_set(session, GNUTLS_CRD_CERTIFICATE,
 					 sizeof(cert_auth_info_st),
-					 0)) < 0) {
+					 1)) < 0) {
 		gnutls_assert();
 		return ret;
 	}
