@@ -35,7 +35,10 @@
 #include <gnutls_datum.h>
 
 
-int _gnutls_proc_psk_client_kx(gnutls_session_t, uint8_t *, size_t);
+static int _gnutls_proc_psk_client_kx(gnutls_session_t, uint8_t *, size_t);
+static int
+_gnutls_proc_psk_server_kx(gnutls_session_t session, uint8_t * data,
+			   size_t _data_size);
 
 
 const mod_auth_st psk_auth_struct = {
@@ -104,7 +107,7 @@ _gnutls_set_psk_session_key(gnutls_session_t session,
 	ret = 0;
 
       error:
-	_gnutls_free_datum(&pwd_psk);
+	_gnutls_free_temp_key_datum(&pwd_psk);
 	return ret;
 }
 
@@ -158,18 +161,25 @@ _gnutls_gen_psk_client_kx(gnutls_session_t session,
 			  gnutls_buffer_st * data)
 {
 	int ret, free;
-	gnutls_datum_t username;
+	gnutls_datum_t username = {NULL, 0};
 	gnutls_datum_t key;
 	gnutls_psk_client_credentials_t cred;
+	psk_auth_info_t info;
 
 	cred = (gnutls_psk_client_credentials_t)
-	    _gnutls_get_cred(session, GNUTLS_CRD_PSK, NULL);
+	    _gnutls_get_cred(session, GNUTLS_CRD_PSK);
 
 	if (cred == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
+	info = _gnutls_get_auth_info(session, GNUTLS_CRD_PSK);
+	if (info == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
+
 	ret = _gnutls_find_psk_key(session, cred, &username, &key, &free);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
@@ -187,10 +197,20 @@ _gnutls_gen_psk_client_kx(gnutls_session_t session,
 		gnutls_assert();
 	}
 
+	if (username.size > sizeof(info->username)-1) {
+		gnutls_assert();
+		ret = GNUTLS_E_ILLEGAL_SRP_USERNAME;
+		goto cleanup;
+	}
+
+	memcpy(info->username, username.data, username.size);
+	info->username[username.size] = 0;
+
+
       cleanup:
 	if (free) {
 		gnutls_free(username.data);
-		gnutls_free(key.data);
+		_gnutls_free_temp_key_datum(&key);
 	}
 
 	return ret;
@@ -199,7 +219,7 @@ _gnutls_gen_psk_client_kx(gnutls_session_t session,
 
 /* just read the username from the client key exchange.
  */
-int
+static int
 _gnutls_proc_psk_client_kx(gnutls_session_t session, uint8_t * data,
 			   size_t _data_size)
 {
@@ -210,7 +230,7 @@ _gnutls_proc_psk_client_kx(gnutls_session_t session, uint8_t * data,
 	psk_auth_info_t info;
 
 	cred = (gnutls_psk_server_credentials_t)
-	    _gnutls_get_cred(session, GNUTLS_CRD_PSK, NULL);
+	    _gnutls_get_cred(session, GNUTLS_CRD_PSK);
 
 	if (cred == NULL) {
 		gnutls_assert();
@@ -234,7 +254,11 @@ _gnutls_proc_psk_client_kx(gnutls_session_t session, uint8_t * data,
 
 	/* copy the username to the auth info structures
 	 */
-	info = _gnutls_get_auth_info(session);
+	info = _gnutls_get_auth_info(session, GNUTLS_CRD_PSK);
+	if (info == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
 
 	if (username.size > MAX_USERNAME_SIZE) {
 		gnutls_assert();
@@ -258,7 +282,7 @@ _gnutls_proc_psk_client_kx(gnutls_session_t session, uint8_t * data,
 	ret = 0;
 
       error:
-	_gnutls_free_datum(&psk_key);
+	_gnutls_free_key_datum(&psk_key);
 
 	return ret;
 }
@@ -283,7 +307,7 @@ _gnutls_gen_psk_server_kx(gnutls_session_t session,
 	gnutls_datum_t hint;
 
 	cred = (gnutls_psk_server_credentials_t)
-	    _gnutls_get_cred(session, GNUTLS_CRD_PSK, NULL);
+	    _gnutls_get_cred(session, GNUTLS_CRD_PSK);
 
 	if (cred == NULL) {
 		gnutls_assert();
@@ -306,7 +330,7 @@ _gnutls_gen_psk_server_kx(gnutls_session_t session,
 
 /* just read the hint from the server key exchange.
  */
-int
+static int
 _gnutls_proc_psk_server_kx(gnutls_session_t session, uint8_t * data,
 			   size_t _data_size)
 {
@@ -317,7 +341,7 @@ _gnutls_proc_psk_server_kx(gnutls_session_t session, uint8_t * data,
 	psk_auth_info_t info;
 
 	cred = (gnutls_psk_client_credentials_t)
-	    _gnutls_get_cred(session, GNUTLS_CRD_PSK, NULL);
+	    _gnutls_get_cred(session, GNUTLS_CRD_PSK);
 
 	if (cred == NULL) {
 		gnutls_assert();
@@ -340,9 +364,13 @@ _gnutls_proc_psk_server_kx(gnutls_session_t session, uint8_t * data,
 
 	/* copy the hint to the auth info structures
 	 */
-	info = _gnutls_get_auth_info(session);
+	info = _gnutls_get_auth_info(session, GNUTLS_CRD_PSK);
+	if (info == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
 
-	if (hint.size > MAX_USERNAME_SIZE) {
+	if (hint.size > sizeof(info->hint)-1) {
 		gnutls_assert();
 		return GNUTLS_E_ILLEGAL_SRP_USERNAME;
 	}
@@ -350,15 +378,8 @@ _gnutls_proc_psk_server_kx(gnutls_session_t session, uint8_t * data,
 	memcpy(info->hint, hint.data, hint.size);
 	info->hint[hint.size] = 0;
 
-	ret = _gnutls_set_psk_session_key(session, &cred->key, NULL);
-	if (ret < 0) {
-		gnutls_assert();
-		goto error;
-	}
-
 	ret = 0;
 
-      error:
 	return ret;
 }
 
