@@ -23,16 +23,17 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
-#ifndef GNUTLS_LIB_AUTH_RSA_COMMON_H
-#define GNUTLS_LIB_AUTH_RSA_COMMON_H
+#ifndef AUTH_RSA_COMMON
+#define AUTH_RSA_COMMON
 
-#include "abstract_int.h"
+#include <abstract_int.h>
 
-int _gnutls_get_public_rsa_params(gnutls_session_t session,
-				  gnutls_pk_params_st *params);
+int
+_gnutls_get_public_rsa_params(gnutls_session_t session,
+			      gnutls_pk_params_st * params);
 
-#endif /* GNUTLS_LIB_AUTH_RSA_COMMON_H */
+#endif
