@@ -16,13 +16,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
 #include "gnutls_int.h"
-#include "errors.h"
-#include "auth/cert.h"
+#include "gnutls_errors.h"
+#include <auth/cert.h>
 #include <gnutls/gnutls.h>
 
 /* This file implements the client certificate auto verification functionality.
@@ -31,28 +31,26 @@
 /* The actual verification callback. */
 static int auto_verify_cb(gnutls_session_t session)
 {
-	unsigned int status;
-	int ret;
+        unsigned int status;
+        int ret;
 
-	if (session->internals.vc_elements == 0) {
-		ret = gnutls_certificate_verify_peers2(session, &status);
+        if (session->internals.vc_elements == 0) {
+        	ret = gnutls_certificate_verify_peers2(session, &status);
 	} else {
-		ret = gnutls_certificate_verify_peers(
-			session, session->internals.vc_data,
-			session->internals.vc_elements, &status);
-	}
-	if (ret < 0) {
-		return gnutls_assert_val(GNUTLS_E_CERTIFICATE_ERROR);
-	}
+	        ret = gnutls_certificate_verify_peers(session, session->internals.vc_data,
+						      session->internals.vc_elements, &status);
+        }
+        if (ret < 0) {
+                return gnutls_assert_val(GNUTLS_E_CERTIFICATE_ERROR);
+        }
 
-	session->internals.vc_status = status;
+        session->internals.vc_status = status;
 
-	if (status != 0) /* Certificate is not trusted */
-		return gnutls_assert_val(
-			GNUTLS_E_CERTIFICATE_VERIFICATION_ERROR);
+        if (status != 0)        /* Certificate is not trusted */
+                return gnutls_assert_val(GNUTLS_E_CERTIFICATE_VERIFICATION_ERROR);
 
-	/* notify gnutls to continue handshake normally */
-	return 0;
+        /* notify gnutls to continue handshake normally */
+        return 0;
 }
 
 /**
@@ -72,20 +70,17 @@ static int auto_verify_cb(gnutls_session_t session)
  * will be performed. For a more advanced verification function check
  * gnutls_session_set_verify_cert2().
  *
- * If @flags is provided which contain a profile, this function should be
- * called after any session priority setting functions.
- *
  * The gnutls_session_set_verify_cert() function is intended to be used by TLS
  * clients to verify the server's certificate.
  *
  * Since: 3.4.6
  **/
 void gnutls_session_set_verify_cert(gnutls_session_t session,
-				    const char *hostname, unsigned flags)
+				     const char *hostname, unsigned flags)
 {
 	if (hostname) {
 		session->internals.vc_sdata.type = GNUTLS_DT_DNS_HOSTNAME;
-		session->internals.vc_sdata.data = (void *)hostname;
+		session->internals.vc_sdata.data = (void*)hostname;
 		session->internals.vc_sdata.size = 0;
 		session->internals.vc_elements = 1;
 		session->internals.vc_data = &session->internals.vc_sdata;
@@ -93,9 +88,8 @@ void gnutls_session_set_verify_cert(gnutls_session_t session,
 		session->internals.vc_elements = 0;
 	}
 
-	if (flags) {
-		ADD_PROFILE_VFLAGS(session, flags);
-	}
+	if (flags)
+		session->internals.additional_verify_flags |= flags;
 
 	gnutls_session_set_verify_function(session, auto_verify_cb);
 }
@@ -116,14 +110,12 @@ void gnutls_session_set_verify_cert(gnutls_session_t session,
  * and once set must remain valid for the lifetime of the session. More precisely
  * they should be available during any subsequent handshakes.
  *
- * If @flags is provided which contain a profile, this function should be
- * called after any session priority setting functions.
- *
  * Since: 3.4.6
  **/
 void gnutls_session_set_verify_cert2(gnutls_session_t session,
-				     gnutls_typed_vdata_st *data,
-				     unsigned elements, unsigned flags)
+				     gnutls_typed_vdata_st * data,
+				     unsigned elements,
+				     unsigned flags)
 {
 	session->internals.vc_data = data;
 	session->internals.vc_elements = elements;
