@@ -67,7 +67,7 @@ _gnutls_alpn_recv_params(gnutls_session_t session,
 	if (ret < 0)
 		return 0;
 
-	priv = epriv.ptr;
+	priv = epriv;
 
 	DECR_LENGTH_RET(data_size, 2, 0);
 	len = _gnutls_read_uint16(p);
@@ -92,7 +92,8 @@ _gnutls_alpn_recv_params(gnutls_session_t session,
 					    priv->protocols[i];
 					priv->selected_protocol_size =
 					    priv->protocol_size[i];
-					break;
+
+					return 0;
 				}
 			p += len1;
 		}
@@ -136,7 +137,7 @@ _gnutls_alpn_send_params(gnutls_session_t session,
 	if (ret < 0)
 		return 0;
 
-	priv = epriv.ptr;
+	priv = epriv;
 
 	if (priv->size == 0)
 		return 0;
@@ -196,17 +197,20 @@ _gnutls_alpn_send_params(gnutls_session_t session,
 
 /**
  * gnutls_alpn_get_selected_protocol:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @protocol: will hold the protocol name
  *
  * This function allows you to get the negotiated protocol name. The
  * returned protocol should be treated as opaque, constant value and
  * only valid during the session life.
  *
+ * The selected protocol is the first supported by the list sent
+ * by the client.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
  *   otherwise a negative error code is returned.
  *
- * Since 3.1.11
+ * Since 3.2.0
  **/
 int
 gnutls_alpn_get_selected_protocol(gnutls_session_t session,
@@ -224,7 +228,7 @@ gnutls_alpn_get_selected_protocol(gnutls_session_t session,
 		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
 	}
 
-	priv = epriv.ptr;
+	priv = epriv;
 
 	if (priv->selected_protocol_size == 0)
 		return
@@ -239,13 +243,13 @@ gnutls_alpn_get_selected_protocol(gnutls_session_t session,
 
 /**
  * gnutls_alpn_set_protocols:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @protocols: is the protocol names to add.
  * @protocols_size: the number of protocols to add.
  * @flags: zero or %GNUTLS_ALPN_*
  *
  * This function is to be used by both clients and servers, to declare
- * the supported ALPN protocols, which are used during peer negotiation.
+ * the supported ALPN protocols, which are used during negotiation with peer.
  *
  * If %GNUTLS_ALPN_MAND is specified the connection will be aborted
  * if no matching ALPN protocol is found.
@@ -253,7 +257,7 @@ gnutls_alpn_get_selected_protocol(gnutls_session_t session,
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
  *   otherwise a negative error code is returned.
  *
- * Since 3.1.11
+ * Since 3.2.0
  **/
 int
 gnutls_alpn_set_protocols(gnutls_session_t session,
@@ -274,11 +278,11 @@ gnutls_alpn_set_protocols(gnutls_session_t session,
 			gnutls_assert();
 			return GNUTLS_E_MEMORY_ERROR;
 		}
-		epriv.ptr = priv;
+		epriv = priv;
 		_gnutls_ext_set_session_data(session,
 					     GNUTLS_EXTENSION_ALPN, epriv);
 	} else
-		priv = epriv.ptr;
+		priv = epriv;
 
 	if (protocols_size > MAX_ALPN_PROTOCOLS)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
@@ -300,13 +304,13 @@ gnutls_alpn_set_protocols(gnutls_session_t session,
 
 static void _gnutls_alpn_deinit_data(extension_priv_data_t priv)
 {
-	gnutls_free(priv.ptr);
+	gnutls_free(priv);
 }
 
 static int
 _gnutls_alpn_pack(extension_priv_data_t epriv, gnutls_buffer_st * ps)
 {
-	alpn_ext_st *priv = epriv.ptr;
+	alpn_ext_st *priv = epriv;
 	int ret;
 
 	BUFFER_APPEND_PFX4(ps, priv->selected_protocol,
@@ -334,7 +338,7 @@ _gnutls_alpn_unpack(gnutls_buffer_st * ps, extension_priv_data_t * _priv)
 	priv->selected_protocol_size = priv->protocol_size[0];
 	priv->selected_protocol = priv->protocols[0];
 
-	epriv.ptr = priv;
+	epriv = priv;
 	*_priv = epriv;
 
 	return 0;
