@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2002-2016 Free Software Foundation, Inc.
- * Copyright (C) 2015-2016 Red Hat, Inc.
+ * Copyright (C) 2002-2012 Free Software Foundation, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -151,16 +150,16 @@ _gnutls_sign_algorithm_parse_data(gnutls_session_t session,
 		     gnutls_sign_get_name(sig));
 
 		if (sig != GNUTLS_SIGN_UNKNOWN) {
-			if (priv->sign_algorithms_size ==
-			    MAX_SIGNATURE_ALGORITHMS)
-				break;
 			priv->sign_algorithms[priv->
 					      sign_algorithms_size++] =
 			    sig;
+			if (priv->sign_algorithms_size ==
+			    MAX_SIGNATURE_ALGORITHMS)
+				break;
 		}
 	}
 
-	epriv.ptr = priv;
+	epriv = priv;
 	_gnutls_ext_set_session_data(session,
 				     GNUTLS_EXTENSION_SIGNATURE_ALGORITHMS,
 				     epriv);
@@ -196,7 +195,7 @@ _gnutls_signature_algorithm_recv_params(gnutls_session_t session,
 	} else {
 		/* SERVER SIDE - we must check if the sent cert type is the right one
 		 */
-		if (data_size >= 2) {
+		if (data_size > 2) {
 			uint16_t len;
 
 			DECR_LEN(data_size, 2);
@@ -277,10 +276,12 @@ _gnutls_session_get_sign_algo(gnutls_session_t session,
 	    _gnutls_ext_get_session_data(session,
 					 GNUTLS_EXTENSION_SIGNATURE_ALGORITHMS,
 					 &epriv);
-	priv = epriv.ptr;
+	priv = epriv;
 
-	if (ret < 0 || !_gnutls_version_has_selectable_sighash(ver)) {
+	if (ret < 0 || !_gnutls_version_has_selectable_sighash(ver)
+	    || priv->sign_algorithms_size == 0)
 		/* none set, allow SHA-1 only */
+	{
 		ret = gnutls_pk_to_sign(cert_algo, GNUTLS_DIG_SHA1);
 		if (_gnutls_session_sign_algo_enabled(session, ret) < 0)
 			goto fail;
@@ -338,14 +339,14 @@ _gnutls_session_sign_algo_enabled(gnutls_session_t session,
 
 static void signature_algorithms_deinit_data(extension_priv_data_t priv)
 {
-	gnutls_free(priv.ptr);
+	gnutls_free(priv);
 }
 
 static int
 signature_algorithms_pack(extension_priv_data_t epriv,
 			  gnutls_buffer_st * ps)
 {
-	sig_ext_st *priv = epriv.ptr;
+	sig_ext_st *priv = epriv;
 	int ret, i;
 
 	BUFFER_APPEND_NUM(ps, priv->sign_algorithms_size);
@@ -374,7 +375,7 @@ signature_algorithms_unpack(gnutls_buffer_st * ps,
 		BUFFER_POP_NUM(ps, priv->sign_algorithms[i]);
 	}
 
-	epriv.ptr = priv;
+	epriv = priv;
 	*_priv = epriv;
 
 	return 0;
@@ -388,7 +389,7 @@ signature_algorithms_unpack(gnutls_buffer_st * ps,
 
 /**
  * gnutls_sign_algorithm_get_requested:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @indx: is an index of the signature algorithm to return
  * @algo: the returned certificate type will be stored there
  *
@@ -428,7 +429,7 @@ gnutls_sign_algorithm_get_requested(gnutls_session_t session,
 		gnutls_assert();
 		return ret;
 	}
-	priv = epriv.ptr;
+	priv = epriv;
 
 	if (!_gnutls_version_has_selectable_sighash(ver)
 	    || priv->sign_algorithms_size == 0) {
@@ -444,7 +445,7 @@ gnutls_sign_algorithm_get_requested(gnutls_session_t session,
 
 /**
  * gnutls_sign_algorithm_get:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Returns the signature algorithm that is (or will be) used in this 
  * session by the server to sign data.
@@ -460,7 +461,7 @@ int gnutls_sign_algorithm_get(gnutls_session_t session)
 
 /**
  * gnutls_sign_algorithm_get_client:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Returns the signature algorithm that is (or will be) used in this 
  * session by the client to sign data.
