@@ -107,6 +107,14 @@ transmit_message(gnutls_session_t session,
 		else
 			frag_len = mtu;
 
+		/* we normally allow fragments of zero length, to allow
+		 * the packets which have zero size. On the others don't
+		 * send such fragments */
+		if (frag_len == 0 && data_size > 0) {
+			ret = 0;
+			break;
+		}
+
 		/* Fragment offset */
 		_gnutls_write_uint24(offset, &mtu_data[6]);
 
@@ -228,7 +236,7 @@ int _dtls_transmit(gnutls_session_t session)
 	 * return.
 	 */
 	if (session->internals.dtls.flight_init != 0
-	    && session->internals.dtls.blocking == 0) {
+	    && session->internals.blocking == 0) {
 		/* just in case previous run was interrupted */
 		ret = _gnutls_io_write_flush(session);
 		if (ret < 0) {
@@ -280,9 +288,8 @@ int _dtls_transmit(gnutls_session_t session)
 
 		diff =
 		    timespec_sub_ms(&now,
-				    &session->internals.dtls.
-				    handshake_start_time);
-		if (diff >= session->internals.dtls.total_timeout_ms) {
+				    &session->internals.handshake_start_time);
+		if (diff >= session->internals.handshake_timeout_ms) {
 			_gnutls_dtls_log("Session timeout: %u ms\n", diff);
 			ret = gnutls_assert_val(GNUTLS_E_TIMEDOUT);
 			goto end_flight;
@@ -348,7 +355,7 @@ int _dtls_transmit(gnutls_session_t session)
 			goto cleanup;
 		} else {	/* all other messages -> implicit ack (receive of next flight) */
 
-			if (session->internals.dtls.blocking != 0)
+			if (session->internals.blocking != 0)
 				ret =
 				    _gnutls_io_check_recv(session,
 							  timeout);
@@ -415,7 +422,7 @@ int _dtls_wait_and_retransmit(gnutls_session_t session)
 {
 	int ret;
 
-	if (session->internals.dtls.blocking != 0)
+	if (session->internals.blocking != 0)
 		ret = _gnutls_io_check_recv(session, TIMER_WINDOW);
 	else
 		ret = _gnutls_io_check_recv(session, 0);
@@ -529,7 +536,7 @@ int _dtls_record_check(struct record_parameters_st *rp, uint64 * _seq)
 
 /**
  * gnutls_dtls_set_timeouts:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @retrans_timeout: The time at which a retransmission will occur in milliseconds
  * @total_timeout: The time at which the connection will be aborted, in milliseconds.
  *
@@ -540,10 +547,7 @@ int _dtls_record_check(struct record_parameters_st *rp, uint64 * _seq)
  * handshake will be aborted with %GNUTLS_E_TIMEDOUT.
  *
  * The DTLS protocol recommends the values of 1 sec and 60 seconds
- * respectively.
- *
- * If the retransmission timeout is zero then the handshake will operate
- * in a non-blocking way, i.e., return %GNUTLS_E_AGAIN.
+ * respectively, and these are the default values.
  *
  * To disable retransmissions set a @retrans_timeout larger than the @total_timeout.
  *
@@ -554,18 +558,18 @@ void gnutls_dtls_set_timeouts(gnutls_session_t session,
 			      unsigned int total_timeout)
 {
 	session->internals.dtls.retrans_timeout_ms = retrans_timeout;
-	session->internals.dtls.total_timeout_ms = total_timeout;
+	session->internals.handshake_timeout_ms = total_timeout;
 }
 
 /**
  * gnutls_dtls_set_mtu:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @mtu: The maximum transfer unit of the transport
  *
  * This function will set the maximum transfer unit of the transport
  * that DTLS packets are sent over. Note that this should exclude
  * the IP (or IPv6) and UDP headers. So for DTLS over IPv6 on an
- * Ethenet device with MTU 1500, the DTLS MTU set with this function
+ * Ethernet device with MTU 1500, the DTLS MTU set with this function
  * would be 1500 - 40 (IPV6 header) - 8 (UDP header) = 1452.
  *
  * Since: 3.0
@@ -577,24 +581,22 @@ void gnutls_dtls_set_mtu(gnutls_session_t session, unsigned int mtu)
 
 static int record_overhead(const cipher_entry_st * cipher,
 			   const mac_entry_st * mac,
-			   gnutls_compression_method_t comp,
-			   unsigned new_padding)
+			   gnutls_compression_method_t comp)
 {
 	int total = 0;
 	int t, ret;
 
-	if (_gnutls_cipher_is_block(cipher) == CIPHER_BLOCK) {
-		t = _gnutls_cipher_get_implicit_iv_size(cipher);
+	if (_gnutls_cipher_type(cipher) == CIPHER_BLOCK) {
+		t = _gnutls_cipher_get_explicit_iv_size(cipher);
 		total += t;
 
 		/* padding */
 		t = _gnutls_cipher_get_block_size(cipher);
-		if (new_padding == 0)
-			total += t;
+		total += t;
 	}
 
 	if (mac->id == GNUTLS_MAC_AEAD) {
-		total += AEAD_EXPLICIT_DATA_SIZE;
+		total += cipher->explicit_iv;
 		total += _gnutls_cipher_get_tag_size(cipher);
 	} else {
 		ret = _gnutls_mac_get_algo_len(mac);
@@ -604,9 +606,6 @@ static int record_overhead(const cipher_entry_st * cipher,
 		total += ret;
 	}
 
-	if (new_padding != 0)
-		total += 2;
-
 	if (comp != GNUTLS_COMP_NULL)
 		total += EXTRA_COMP_SIZE;
 
@@ -659,7 +658,7 @@ size_t gnutls_est_record_overhead_size(gnutls_protocol_t version,
 	else
 		total = DTLS_RECORD_HEADER_SIZE;
 
-	total += record_overhead(c, m, comp, 0);
+	total += record_overhead(c, m, comp);
 
 	return total;
 }
@@ -687,9 +686,7 @@ static int record_overhead_rt(gnutls_session_t session)
 
 	/* requires padding */
 	return record_overhead(params->cipher, params->mac,
-			       params->compression_algorithm,
-			       session->security_parameters.
-			       new_record_padding);
+			       params->compression_algorithm);
 }
 
 /**
@@ -720,7 +717,7 @@ size_t gnutls_record_overhead_size(gnutls_session_t session)
 
 /**
  * gnutls_dtls_get_data_mtu:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * This function will return the actual maximum transfer unit for
  * application data. I.e. DTLS headers are subtracted from the
@@ -746,7 +743,7 @@ unsigned int gnutls_dtls_get_data_mtu(gnutls_session_t session)
 
 /**
  * gnutls_dtls_set_data_mtu:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @mtu: The maximum unencrypted transfer unit of the session
  *
  * This function will set the maximum size of the *unencrypted* records
@@ -784,7 +781,7 @@ int gnutls_dtls_set_data_mtu(gnutls_session_t session, unsigned int mtu)
 
 /**
  * gnutls_dtls_get_mtu:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * This function will return the MTU size as set with
  * gnutls_dtls_set_mtu(). This is not the actual MTU
@@ -802,7 +799,7 @@ unsigned int gnutls_dtls_get_mtu(gnutls_session_t session)
 
 /**
  * gnutls_dtls_get_timeout:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * This function will return the milliseconds remaining
  * for a retransmission of the previously sent handshake
@@ -1079,7 +1076,7 @@ void gnutls_dtls_prestate_set(gnutls_session_t session,
 
 /**
  * gnutls_record_get_discarded:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Returns the number of discarded packets in a
  * DTLS connection.
