@@ -23,8 +23,6 @@
 #ifndef GNUTLS_MEM_H
 #define GNUTLS_MEM_H
 
-#include <config.h>
-
 /* this realloc function will return ptr if size==0, and
  * will free the ptr if the new allocation failed.
  */
@@ -32,7 +30,6 @@ void *gnutls_realloc_fast(void *ptr, size_t size);
 
 void *_gnutls_calloc(size_t nmemb, size_t size);
 char *_gnutls_strdup(const char *);
-void gnutls_memset(void *data, int c, size_t size);
 
 /* To avoid undefined behavior when s1 or s2 are null and n = 0 */
 inline static
@@ -43,10 +40,6 @@ int safe_memcmp(const void *s1, const void *s2, size_t n)
         return memcmp(s1, s2, n);
 }
 
-#ifndef HAVE_SECURE_GETENV
-# define secure_getenv getenv
-#endif
-
 #define zrelease_mpi_key(mpi) if (*mpi!=NULL) { \
                 _gnutls_mpi_clear(*mpi); \
                 _gnutls_mpi_release(mpi); \
