@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2001-2014 Free Software Foundation, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -174,17 +174,24 @@ int _gnutls_pk_params_copy(gnutls_pk_params_st * dst,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	dst->flags = src->flags;
+	dst->algo = src->algo;
+
 	for (i = 0; i < src->params_nr; i++) {
-		dst->params[i] = _gnutls_mpi_set(NULL, src->params[i]);
+		dst->params[i] = _gnutls_mpi_copy(src->params[i]);
 		if (dst->params[i] == NULL) {
-			for (j = 0; j < i; j++)
-				_gnutls_mpi_release(&dst->params[j]);
-			return GNUTLS_E_MEMORY_ERROR;
+			goto fail;
 		}
+
 		dst->params_nr++;
 	}
 
 	return 0;
+
+fail:
+	for (j = 0; j < i; j++)
+		_gnutls_mpi_release(&dst->params[j]);
+	return GNUTLS_E_MEMORY_ERROR;
 }
 
 void gnutls_pk_params_init(gnutls_pk_params_st * p)
@@ -210,23 +217,6 @@ void gnutls_pk_params_clear(gnutls_pk_params_st * p)
 	}
 }
 
-int
-_gnutls_pk_get_hash_algorithm(gnutls_pk_algorithm_t pk,
-			      gnutls_pk_params_st * params,
-			      gnutls_digest_algorithm_t * dig,
-			      unsigned int *mand)
-{
-	if (mand) {
-		if (pk == GNUTLS_PK_DSA)
-			*mand = 1;
-		else
-			*mand = 0;
-	}
-
-	return _gnutls_x509_verify_algorithm(dig, NULL, pk, params);
-
-}
-
 /* Writes the digest information and the digest in a DER encoded
  * structure. The digest info is allocated and stored into the info structure.
  */
@@ -286,7 +276,12 @@ encode_ber_digest_info(const mac_entry_st * e,
 	}
 
 	tmp_output_size = 0;
-	asn1_der_coding(dinfo, "", NULL, &tmp_output_size, NULL);
+	result = asn1_der_coding(dinfo, "", NULL, &tmp_output_size, NULL);
+	if (result != ASN1_MEM_ERROR) {
+		gnutls_assert();
+		asn1_delete_structure(&dinfo);
+		return _gnutls_asn2err(result);
+	}
 
 	tmp_output = gnutls_malloc(tmp_output_size);
 	if (tmp_output == NULL) {
@@ -322,7 +317,7 @@ decode_ber_digest_info(const gnutls_datum_t * info,
 {
 	ASN1_TYPE dinfo = ASN1_TYPE_EMPTY;
 	int result;
-	char str[1024];
+	char str[MAX(MAX_OID_SIZE, MAX_HASH_SIZE)];
 	int len;
 
 	if ((result = asn1_create_element(_gnutls_get_gnutls_asn(),
@@ -348,7 +343,7 @@ decode_ber_digest_info(const gnutls_datum_t * info,
 		return _gnutls_asn2err(result);
 	}
 
-	*hash = _gnutls_x509_oid_to_digest(str);
+	*hash = gnutls_oid_to_digest(str);
 
 	if (*hash == GNUTLS_DIG_UNKNOWN) {
 
@@ -388,3 +383,255 @@ decode_ber_digest_info(const gnutls_datum_t * info,
 
 	return 0;
 }
+
+int
+_gnutls_params_get_rsa_raw(const gnutls_pk_params_st* params,
+				    gnutls_datum_t * m, gnutls_datum_t * e,
+				    gnutls_datum_t * d, gnutls_datum_t * p,
+				    gnutls_datum_t * q, gnutls_datum_t * u,
+				    gnutls_datum_t * e1,
+				    gnutls_datum_t * e2)
+{
+	int ret;
+
+	if (params == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	if (params->algo != GNUTLS_PK_RSA) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	if (m) {
+		ret = _gnutls_mpi_dprint_lz(params->params[0], m);
+		if (ret < 0) {
+			gnutls_assert();
+			goto error;
+		}
+	}
+
+	/* E */
+	if (e) {
+		ret = _gnutls_mpi_dprint_lz(params->params[1], e);
+		if (ret < 0) {
+			gnutls_assert();
+			goto error;
+		}
+	}
+
+	/* D */
+	if (d && params->params[2]) {
+		ret = _gnutls_mpi_dprint_lz(params->params[2], d);
+		if (ret < 0) {
+			gnutls_assert();
+			goto error;
+		}
+	} else if (d) {
+		d->data = NULL;
+		d->size = 0;
+	}
+
+	/* P */
+	if (p && params->params[3]) {
+		ret = _gnutls_mpi_dprint_lz(params->params[3], p);
+		if (ret < 0) {
+			gnutls_assert();
+			goto error;
+		}
+	} else if (p) {
+		p->data = NULL;
+		p->size = 0;
+	}
+
+	/* Q */
+	if (q && params->params[4]) {
+		ret = _gnutls_mpi_dprint_lz(params->params[4], q);
+		if (ret < 0) {
+			gnutls_assert();
+			goto error;
+		}
+	} else if (q) {
+		q->data = NULL;
+		q->size = 0;
+	}
+
+	/* U */
+	if (u && params->params[5]) {
+		ret = _gnutls_mpi_dprint_lz(params->params[5], u);
+		if (ret < 0) {
+			gnutls_assert();
+			goto error;
+		}
+	} else if (u) {
+		u->data = NULL;
+		u->size = 0;
+	}
+
+	/* E1 */
+	if (e1 && params->params[6]) {
+		ret = _gnutls_mpi_dprint_lz(params->params[6], e1);
+		if (ret < 0) {
+			gnutls_assert();
+			goto error;
+		}
+	} else if (e1) {
+		e1->data = NULL;
+		e1->size = 0;
+	}
+
+	/* E2 */
+	if (e2 && params->params[7]) {
+		ret = _gnutls_mpi_dprint_lz(params->params[7], e2);
+		if (ret < 0) {
+			gnutls_assert();
+			goto error;
+		}
+	} else if (e2) {
+		e2->data = NULL;
+		e2->size = 0;
+	}
+
+	return 0;
+
+      error:
+	_gnutls_free_datum(m);
+	_gnutls_free_datum(d);
+	_gnutls_free_datum(e);
+	_gnutls_free_datum(e1);
+	_gnutls_free_datum(e2);
+	_gnutls_free_datum(p);
+	_gnutls_free_datum(q);
+
+	return ret;
+}
+
+int
+_gnutls_params_get_dsa_raw(const gnutls_pk_params_st* params,
+			     gnutls_datum_t * p, gnutls_datum_t * q,
+			     gnutls_datum_t * g, gnutls_datum_t * y,
+			     gnutls_datum_t * x)
+{
+	int ret;
+
+	if (params == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	if (params->algo != GNUTLS_PK_DSA) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	/* P */
+	if (p) {
+		ret = _gnutls_mpi_dprint_lz(params->params[0], p);
+		if (ret < 0) {
+			gnutls_assert();
+			return ret;
+		}
+	}
+
+	/* Q */
+	if (q) {
+		ret = _gnutls_mpi_dprint_lz(params->params[1], q);
+		if (ret < 0) {
+			gnutls_assert();
+			_gnutls_free_datum(p);
+			return ret;
+		}
+	}
+
+
+	/* G */
+	if (g) {
+		ret = _gnutls_mpi_dprint_lz(params->params[2], g);
+		if (ret < 0) {
+			gnutls_assert();
+			_gnutls_free_datum(p);
+			_gnutls_free_datum(q);
+			return ret;
+		}
+	}
+
+
+	/* Y */
+	if (y) {
+		ret = _gnutls_mpi_dprint_lz(params->params[3], y);
+		if (ret < 0) {
+			gnutls_assert();
+			_gnutls_free_datum(p);
+			_gnutls_free_datum(g);
+			_gnutls_free_datum(q);
+			return ret;
+		}
+	}
+
+	/* X */
+	if (x) {
+		ret = _gnutls_mpi_dprint_lz(params->params[4], x);
+		if (ret < 0) {
+			gnutls_assert();
+			_gnutls_free_datum(y);
+			_gnutls_free_datum(p);
+			_gnutls_free_datum(g);
+			_gnutls_free_datum(q);
+			return ret;
+		}
+	}
+
+	return 0;
+}
+
+int _gnutls_params_get_ecc_raw(const gnutls_pk_params_st* params,
+				       gnutls_ecc_curve_t * curve,
+				       gnutls_datum_t * x,
+				       gnutls_datum_t * y,
+				       gnutls_datum_t * k)
+{
+	int ret;
+
+	if (params == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	if (curve)
+		*curve = params->flags;
+
+	/* X */
+	if (x) {
+		ret = _gnutls_mpi_dprint_lz(params->params[ECC_X], x);
+		if (ret < 0) {
+			gnutls_assert();
+			return ret;
+		}
+	}
+
+	/* Y */
+	if (y) {
+		ret = _gnutls_mpi_dprint_lz(params->params[ECC_Y], y);
+		if (ret < 0) {
+			gnutls_assert();
+			_gnutls_free_datum(x);
+			return ret;
+		}
+	}
+
+
+	/* K */
+	if (k) {
+		ret = _gnutls_mpi_dprint_lz(params->params[ECC_K], k);
+		if (ret < 0) {
+			gnutls_assert();
+			_gnutls_free_datum(x);
+			_gnutls_free_datum(y);
+			return ret;
+		}
+	}
+
+	return 0;
+
+}
