@@ -58,13 +58,8 @@ _gnutls_range_max_lh_pad(gnutls_session_t session, ssize_t data_length,
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 	}
 
-	if (session->security_parameters.new_record_padding != 0) {
-		max_pad = max_user_send_size(session, record_params);
-		fixed_pad = 2;
-	} else {
-		max_pad = MAX_PAD_SIZE;
-		fixed_pad = 1;
-	}
+	max_pad = MAX_PAD_SIZE;
+	fixed_pad = 1;
 
 	this_pad = MIN(max_pad, max_frag - data_length);
 
@@ -72,7 +67,8 @@ _gnutls_range_max_lh_pad(gnutls_session_t session, ssize_t data_length,
 	tag_size =
 	    _gnutls_auth_cipher_tag_len(&record_params->write.
 					cipher_state);
-	switch (_gnutls_cipher_is_block(record_params->cipher)) {
+	switch (_gnutls_cipher_type(record_params->cipher)) {
+	case CIPHER_AEAD:
 	case CIPHER_STREAM:
 		return this_pad;
 
@@ -92,7 +88,7 @@ _gnutls_range_max_lh_pad(gnutls_session_t session, ssize_t data_length,
 
 /**
  * gnutls_record_can_use_length_hiding:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * If the session supports length-hiding padding, you can
  * invoke gnutls_range_send_message() to send a message whose
@@ -110,9 +106,6 @@ int gnutls_record_can_use_length_hiding(gnutls_session_t session)
 	int ret;
 	record_parameters_st *record_params;
 
-	if (session->security_parameters.new_record_padding != 0)
-		return 1;
-
 	if (get_num_version(session) == GNUTLS_SSL3)
 		return 0;
 
@@ -123,10 +116,11 @@ int gnutls_record_can_use_length_hiding(gnutls_session_t session)
 		return 0;
 	}
 
-	switch (_gnutls_cipher_is_block(record_params->cipher)) {
+	switch (_gnutls_cipher_type(record_params->cipher)) {
 	case CIPHER_BLOCK:
 		return 1;
 	case CIPHER_STREAM:
+	case CIPHER_AEAD:
 	default:
 		return 0;
 	}
@@ -134,7 +128,7 @@ int gnutls_record_can_use_length_hiding(gnutls_session_t session)
 
 /**
  * gnutls_range_split:
- * @session: is a #gnutls_session_t structure
+ * @session: is a #gnutls_session_t type
  * @orig: is the original range provided by the user
  * @next: is the returned range that can be conveyed in a TLS record
  * @remainder: is the returned remaining range
@@ -210,7 +204,7 @@ _gnutls_range_fragment(size_t data_size, gnutls_range_st cur,
 
 /**
  * gnutls_record_send_range:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @data: contains the data to send.
  * @data_size: is the length of the data.
  * @range: is the range of lengths in which the real data length must be hidden.
@@ -248,10 +242,6 @@ gnutls_record_send_range(gnutls_session_t session, const void *data,
 	if (ret == 0)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
-	if (ret == 0 && range->low != range->high)
-		/* Cannot use LH, but a range was given */
-		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
-
 	_gnutls_set_range(&cur_range, range->low, range->high);
 
 	_gnutls_record_log
