@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2002-2013 Free Software Foundation, Inc.
+ * Copyright (C) 2002-2015 Free Software Foundation, Inc.
+ * Copyright (C) 2014-2015 Nikos Mavrogiannopoulos
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -45,6 +46,8 @@
 #include <gnutls_extensions.h>
 #include <system.h>
 #include <random.h>
+#include <fips.h>
+#include <intprops.h>
 #include <gnutls/dtls.h>
 
 /* These should really be static, but src/tests.c calls them.  Make
@@ -74,7 +77,7 @@ _gnutls_session_ecc_curve_set(gnutls_session_t session,
 
 /**
  * gnutls_cipher_get:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Get currently used cipher.
  *
@@ -96,7 +99,7 @@ gnutls_cipher_algorithm_t gnutls_cipher_get(gnutls_session_t session)
 
 /**
  * gnutls_certificate_type_get:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * The certificate type is by default X.509, unless it is negotiated
  * as a TLS extension.
@@ -112,7 +115,7 @@ gnutls_certificate_type_get(gnutls_session_t session)
 
 /**
  * gnutls_kx_get:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Get currently used key exchange algorithm.
  *
@@ -126,7 +129,7 @@ gnutls_kx_algorithm_t gnutls_kx_get(gnutls_session_t session)
 
 /**
  * gnutls_mac_get:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Get currently used MAC algorithm.
  *
@@ -148,7 +151,7 @@ gnutls_mac_algorithm_t gnutls_mac_get(gnutls_session_t session)
 
 /**
  * gnutls_compression_get:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Get currently used compression algorithm.
  *
@@ -183,14 +186,12 @@ _gnutls_session_cert_type_supported(gnutls_session_t session,
 
 	if (session->security_parameters.entity == GNUTLS_SERVER) {
 		cred = (gnutls_certificate_credentials_t)
-		    _gnutls_get_cred(session, GNUTLS_CRD_CERTIFICATE,
-				     NULL);
+		    _gnutls_get_cred(session, GNUTLS_CRD_CERTIFICATE);
 
 		if (cred == NULL)
 			return GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE;
 
-		if (cred->server_get_cert_callback == NULL
-		    && cred->get_cert_callback == NULL) {
+		if (cred->get_cert_callback == NULL && cred->get_cert_callback2 == NULL) {
 			for (i = 0; i < cred->ncerts; i++) {
 				if (cred->certs[i].cert_list[0].type ==
 				    cert_type) {
@@ -261,9 +262,9 @@ static void _gnutls_handshake_internal_state_init(gnutls_session_t session)
 
 	session->internals.resumable = RESUME_TRUE;
 
+	session->internals.handshake_large_loops = 0;
 	session->internals.dtls.hsk_read_seq = 0;
 	session->internals.dtls.hsk_write_seq = 0;
-	gettime(&session->internals.dtls.handshake_start_time);
 }
 
 void _gnutls_handshake_internal_state_clear(gnutls_session_t session)
@@ -275,11 +276,12 @@ void _gnutls_handshake_internal_state_clear(gnutls_session_t session)
 	_gnutls_epoch_gc(session);
 
 	session->internals.handshake_endtime = 0;
+	session->internals.handshake_in_progress = 0;
 }
 
 /**
  * gnutls_init:
- * @session: is a pointer to a #gnutls_session_t structure.
+ * @session: is a pointer to a #gnutls_session_t type.
  * @flags: indicate if this session is to be used for server or client.
  *
  * This function initializes the current session to null. Every
@@ -287,10 +289,8 @@ void _gnutls_handshake_internal_state_clear(gnutls_session_t session)
  * be allocated.  This function allocates structures which can only
  * be free'd by calling gnutls_deinit().  Returns %GNUTLS_E_SUCCESS (0) on success.
  *
- * @flags can be one of %GNUTLS_CLIENT and %GNUTLS_SERVER. For a DTLS
- * entity, the flags %GNUTLS_DATAGRAM and  %GNUTLS_NONBLOCK are
- * also available. The latter flag will enable a non-blocking
- * operation of the DTLS timers. 
+ * @flags can be one of %GNUTLS_CLIENT, %GNUTLS_SERVER, %GNUTLS_DATAGRAM,
+ * %GNUTLS_NONBLOCK or %GNUTLS_NOSIGNAL (since 3.4.2).
  *
  * The flag %GNUTLS_NO_REPLAY_PROTECTION will disable any 
  * replay protection in DTLS mode. That must only used when 
@@ -307,6 +307,8 @@ int gnutls_init(gnutls_session_t * session, unsigned int flags)
 {
 	int ret;
 	record_parameters_st *epoch;
+	
+	FAIL_IF_LIB_ERROR;
 
 	*session = gnutls_calloc(1, sizeof(struct gnutls_session_int));
 	if (*session == NULL)
@@ -373,15 +375,20 @@ int gnutls_init(gnutls_session_t * session, unsigned int flags)
 	(*session)->internals.priorities.sr = SR_PARTIAL;
 
 #ifdef HAVE_WRITEV
-	gnutls_transport_set_vec_push_function(*session, system_writev);
+#ifdef MSG_NOSIGNAL
+	if (flags & GNUTLS_NO_SIGNAL)
+		gnutls_transport_set_vec_push_function(*session, system_writev_nosignal);
+	else
+#endif
+		gnutls_transport_set_vec_push_function(*session, system_writev);
 #else
 	gnutls_transport_set_push_function(*session, system_write);
 #endif
-	gnutls_transport_set_pull_function(*session, system_read);
-	gnutls_transport_set_errno_function(*session, system_errno);
-	gnutls_transport_set_pull_timeout_function(*session,
-						   system_recv_timeout);
+	(*session)->internals.pull_timeout_func = gnutls_system_recv_timeout;
+	(*session)->internals.pull_func = system_read;
+	(*session)->internals.errno_func = system_errno;
 
+	/* heartbeat timeouts */
 	(*session)->internals.hb_retrans_timeout_ms = 1000;
 	(*session)->internals.hb_total_timeout_ms = 60000;
 
@@ -389,23 +396,30 @@ int gnutls_init(gnutls_session_t * session, unsigned int flags)
 		(*session)->internals.dtls.mtu = DTLS_DEFAULT_MTU;
 		(*session)->internals.transport = GNUTLS_DGRAM;
 
-		(*session)->internals.dtls.retrans_timeout_ms = 1000;
-		(*session)->internals.dtls.total_timeout_ms = 60000;
-	} else
+		gnutls_dtls_set_timeouts(*session, DTLS_RETRANS_TIMEOUT, 60000);
+	} else {
 		(*session)->internals.transport = GNUTLS_STREAM;
+	}
 
 	if (flags & GNUTLS_NONBLOCK)
-		(*session)->internals.dtls.blocking = 0;
+		(*session)->internals.blocking = 0;
 	else
-		(*session)->internals.dtls.blocking = 1;
+		(*session)->internals.blocking = 1;
 
 	/* Enable useful extensions */
 	if ((flags & GNUTLS_CLIENT) && !(flags & GNUTLS_NO_EXTENSIONS)) {
+#ifdef ENABLE_SESSION_TICKETS
 		gnutls_session_ticket_enable_client(*session);
+#endif
+#ifdef ENABLE_OCSP
 		gnutls_ocsp_status_request_enable_client(*session, NULL, 0,
 							 NULL);
+#endif
 	}
 
+	if (!(flags & GNUTLS_NO_EXTENSIONS))
+		(*session)->internals.try_ext_master_secret = 1;
+
 	if (flags & GNUTLS_NO_REPLAY_PROTECTION)
 		(*session)->internals.no_replay_protection = 1;
 
@@ -422,7 +436,7 @@ int _gnutls_session_is_resumable(gnutls_session_t session)
 
 /**
  * gnutls_deinit:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * This function clears all buffers associated with the @session.
  * This function will also remove session data from the session
@@ -435,8 +449,6 @@ void gnutls_deinit(gnutls_session_t session)
 	if (session == NULL)
 		return;
 
-	_gnutls_rnd_refresh();
-
 	/* remove auth info firstly */
 	_gnutls_free_auth_info(session);
 
@@ -464,26 +476,28 @@ void gnutls_deinit(gnutls_session_t session)
 	_gnutls_selected_certs_deinit(session);
 
 	gnutls_pk_params_release(&session->key.ecdh_params);
-	_gnutls_mpi_release(&session->key.ecdh_x);
-	_gnutls_mpi_release(&session->key.ecdh_y);
+	gnutls_pk_params_release(&session->key.dh_params);
+	zrelease_temp_mpi_key(&session->key.ecdh_x);
+	zrelease_temp_mpi_key(&session->key.ecdh_y);
 
-	_gnutls_mpi_release(&session->key.KEY);
-	_gnutls_mpi_release(&session->key.client_Y);
-	_gnutls_mpi_release(&session->key.client_p);
-	_gnutls_mpi_release(&session->key.client_g);
+	zrelease_temp_mpi_key(&session->key.client_Y);
 
-	_gnutls_mpi_release(&session->key.u);
-	_gnutls_mpi_release(&session->key.a);
-	_gnutls_mpi_release(&session->key.x);
-	_gnutls_mpi_release(&session->key.A);
-	_gnutls_mpi_release(&session->key.B);
-	_gnutls_mpi_release(&session->key.b);
+	zrelease_temp_mpi_key(&session->key.srp_p);
+	zrelease_temp_mpi_key(&session->key.srp_g);
+	zrelease_temp_mpi_key(&session->key.srp_key);
+
+	zrelease_temp_mpi_key(&session->key.u);
+	zrelease_temp_mpi_key(&session->key.a);
+	zrelease_temp_mpi_key(&session->key.x);
+	zrelease_temp_mpi_key(&session->key.A);
+	zrelease_temp_mpi_key(&session->key.B);
+	zrelease_temp_mpi_key(&session->key.b);
 
 	/* RSA */
-	_gnutls_mpi_release(&session->key.rsa[0]);
-	_gnutls_mpi_release(&session->key.rsa[1]);
+	zrelease_temp_mpi_key(&session->key.rsa[0]);
+	zrelease_temp_mpi_key(&session->key.rsa[1]);
 
-	_gnutls_mpi_release(&session->key.dh_secret);
+	_gnutls_free_temp_key_datum(&session->key.key);
 
 	gnutls_free(session);
 }
@@ -499,9 +513,9 @@ int _gnutls_dh_set_peer_public(gnutls_session_t session, bigint_t public)
 	case GNUTLS_CRD_ANON:
 		{
 			anon_auth_info_t info;
-			info = _gnutls_get_auth_info(session);
+			info = _gnutls_get_auth_info(session, GNUTLS_CRD_ANON);
 			if (info == NULL)
-				return GNUTLS_E_INTERNAL_ERROR;
+				return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
 			dh = &info->dh;
 			break;
@@ -509,9 +523,9 @@ int _gnutls_dh_set_peer_public(gnutls_session_t session, bigint_t public)
 	case GNUTLS_CRD_PSK:
 		{
 			psk_auth_info_t info;
-			info = _gnutls_get_auth_info(session);
+			info = _gnutls_get_auth_info(session, GNUTLS_CRD_PSK);
 			if (info == NULL)
-				return GNUTLS_E_INTERNAL_ERROR;
+				return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
 			dh = &info->dh;
 			break;
@@ -520,16 +534,15 @@ int _gnutls_dh_set_peer_public(gnutls_session_t session, bigint_t public)
 		{
 			cert_auth_info_t info;
 
-			info = _gnutls_get_auth_info(session);
+			info = _gnutls_get_auth_info(session, GNUTLS_CRD_CERTIFICATE);
 			if (info == NULL)
-				return GNUTLS_E_INTERNAL_ERROR;
+				return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
 			dh = &info->dh;
 			break;
 		}
 	default:
-		gnutls_assert();
-		return GNUTLS_E_INTERNAL_ERROR;
+		return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 	}
 
 	if (dh->public_key.data)
@@ -550,18 +563,18 @@ int _gnutls_dh_set_secret_bits(gnutls_session_t session, unsigned bits)
 	case GNUTLS_CRD_ANON:
 		{
 			anon_auth_info_t info;
-			info = _gnutls_get_auth_info(session);
+			info = _gnutls_get_auth_info(session, GNUTLS_CRD_ANON);
 			if (info == NULL)
-				return GNUTLS_E_INTERNAL_ERROR;
+				return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 			info->dh.secret_bits = bits;
 			break;
 		}
 	case GNUTLS_CRD_PSK:
 		{
 			psk_auth_info_t info;
-			info = _gnutls_get_auth_info(session);
+			info = _gnutls_get_auth_info(session, GNUTLS_CRD_PSK);
 			if (info == NULL)
-				return GNUTLS_E_INTERNAL_ERROR;
+				return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 			info->dh.secret_bits = bits;
 			break;
 		}
@@ -569,15 +582,14 @@ int _gnutls_dh_set_secret_bits(gnutls_session_t session, unsigned bits)
 		{
 			cert_auth_info_t info;
 
-			info = _gnutls_get_auth_info(session);
+			info = _gnutls_get_auth_info(session, GNUTLS_CRD_CERTIFICATE);
 			if (info == NULL)
-				return GNUTLS_E_INTERNAL_ERROR;
+				return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
 			info->dh.secret_bits = bits;
 			break;
 	default:
-			gnutls_assert();
-			return GNUTLS_E_INTERNAL_ERROR;
+			return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 		}
 	}
 
@@ -597,9 +609,9 @@ _gnutls_dh_set_group(gnutls_session_t session, bigint_t gen,
 	case GNUTLS_CRD_ANON:
 		{
 			anon_auth_info_t info;
-			info = _gnutls_get_auth_info(session);
+			info = _gnutls_get_auth_info(session, GNUTLS_CRD_ANON);
 			if (info == NULL)
-				return GNUTLS_E_INTERNAL_ERROR;
+				return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
 			dh = &info->dh;
 			break;
@@ -607,9 +619,9 @@ _gnutls_dh_set_group(gnutls_session_t session, bigint_t gen,
 	case GNUTLS_CRD_PSK:
 		{
 			psk_auth_info_t info;
-			info = _gnutls_get_auth_info(session);
+			info = _gnutls_get_auth_info(session, GNUTLS_CRD_PSK);
 			if (info == NULL)
-				return GNUTLS_E_INTERNAL_ERROR;
+				return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
 			dh = &info->dh;
 			break;
@@ -618,16 +630,15 @@ _gnutls_dh_set_group(gnutls_session_t session, bigint_t gen,
 		{
 			cert_auth_info_t info;
 
-			info = _gnutls_get_auth_info(session);
+			info = _gnutls_get_auth_info(session, GNUTLS_CRD_CERTIFICATE);
 			if (info == NULL)
-				return GNUTLS_E_INTERNAL_ERROR;
+				return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
 			dh = &info->dh;
 			break;
 		}
 	default:
-		gnutls_assert();
-		return GNUTLS_E_INTERNAL_ERROR;
+		return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 	}
 
 	if (dh->prime.data)
@@ -659,7 +670,7 @@ _gnutls_dh_set_group(gnutls_session_t session, bigint_t gen,
 #ifdef ENABLE_OPENPGP
 /**
  * gnutls_openpgp_send_cert:
- * @session: is a pointer to a #gnutls_session_t structure.
+ * @session: a #gnutls_session_t type.
  * @status: is one of GNUTLS_OPENPGP_CERT, or GNUTLS_OPENPGP_CERT_FINGERPRINT
  *
  * This function will order gnutls to send the key fingerprint
@@ -677,7 +688,7 @@ gnutls_openpgp_send_cert(gnutls_session_t session,
 
 /**
  * gnutls_certificate_send_x509_rdn_sequence:
- * @session: is a pointer to a #gnutls_session_t structure.
+ * @session: a #gnutls_session_t type.
  * @status: is 0 or 1
  *
  * If status is non zero, this function will order gnutls not to send
@@ -705,7 +716,7 @@ int _gnutls_openpgp_send_fingerprint(gnutls_session_t session)
 
 /*-
  * _gnutls_record_set_default_version - Used to set the default version for the first record packet
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @major: is a tls major version
  * @minor: is a tls minor version
  *
@@ -722,9 +733,28 @@ _gnutls_record_set_default_version(gnutls_session_t session,
 	session->internals.default_record_version[1] = minor;
 }
 
+/*-
+ * _gnutls_hello_set_default_version - Used to set the default version for the first record packet
+ * @session: is a #gnutls_session_t type.
+ * @major: is a tls major version
+ * @minor: is a tls minor version
+ *
+ * This function sets the default version that we will use in the first
+ * record packet (client hello). This function is only useful to people
+ * that know TLS internals and want to debug other implementations.
+ -*/
+void
+_gnutls_hello_set_default_version(gnutls_session_t session,
+				   unsigned char major,
+				   unsigned char minor)
+{
+	session->internals.default_hello_version[0] = major;
+	session->internals.default_hello_version[1] = minor;
+}
+
 /**
  * gnutls_handshake_set_private_extensions:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @allow: is an integer (0 or 1)
  *
  * This function will enable or disable the use of private cipher
@@ -835,22 +865,22 @@ P_hash(gnutls_mac_algorithm_t algorithm,
 
 #define MAX_PRF_BYTES 200
 
-/* The PRF function expands a given secret 
- * needed by the TLS specification. ret must have a least total_bytes
- * available.
+/* This function operates as _gnutls_PRF(), but does not require
+ * a pointer to the current session. It takes the @mac algorithm
+ * explicitly. For legacy TLS/SSL sessions before TLS 1.2 the MAC
+ * must be set to %GNUTLS_MAC_UNKNOWN.
  */
-int
-_gnutls_PRF(gnutls_session_t session,
-	    const uint8_t * secret, unsigned int secret_size,
-	    const char *label, int label_size, const uint8_t * seed,
-	    int seed_size, int total_bytes, void *ret)
+static int
+_gnutls_PRF_raw(gnutls_mac_algorithm_t mac,
+	    	const uint8_t * secret, unsigned int secret_size,
+	    	const char *label, int label_size, const uint8_t * seed,
+	    	int seed_size, int total_bytes, void *ret)
 {
 	int l_s, s_seed_size;
 	const uint8_t *s1, *s2;
 	uint8_t s_seed[MAX_SEED_SIZE];
 	uint8_t o1[MAX_PRF_BYTES], o2[MAX_PRF_BYTES];
 	int result;
-	const version_entry_st *ver = get_version(session);
 
 	if (total_bytes > MAX_PRF_BYTES) {
 		gnutls_assert();
@@ -867,11 +897,10 @@ _gnutls_PRF(gnutls_session_t session,
 	memcpy(s_seed, label, label_size);
 	memcpy(&s_seed[label_size], seed, seed_size);
 
-	if (_gnutls_version_has_selectable_prf(ver)) {
+	if (mac != GNUTLS_MAC_UNKNOWN) {
 		result =
-		    P_hash(_gnutls_cipher_suite_get_prf
-			   (session->security_parameters.cipher_suite),
-			   secret, secret_size, s_seed, s_seed_size,
+		    P_hash(mac, secret, secret_size,
+		    	   s_seed, s_seed_size,
 			   total_bytes, ret);
 		if (result < 0) {
 			gnutls_assert();
@@ -909,12 +938,83 @@ _gnutls_PRF(gnutls_session_t session,
 	}
 
 	return 0;		/* ok */
+}
+
+/* The PRF function expands a given secret 
+ * needed by the TLS specification. ret must have a least total_bytes
+ * available.
+ */
+int
+_gnutls_PRF(gnutls_session_t session,
+	    const uint8_t * secret, unsigned int secret_size,
+	    const char *label, int label_size, const uint8_t * seed,
+	    int seed_size, int total_bytes, void *ret)
+{
+	const version_entry_st *ver = get_version(session);
+
+	if (_gnutls_version_has_selectable_prf(ver)) {
+		return _gnutls_PRF_raw(
+			_gnutls_cipher_suite_get_prf(session->security_parameters.cipher_suite),
+			secret, secret_size,
+			label, label_size,
+			seed, seed_size,
+			total_bytes,
+			ret);
+	} else {
+		return _gnutls_PRF_raw(
+			GNUTLS_MAC_UNKNOWN,
+			secret, secret_size,
+			label, label_size,
+			seed, seed_size,
+			total_bytes,
+			ret);
+	}
+}
+
+#ifdef ENABLE_FIPS140
+int
+_gnutls_prf_raw(gnutls_mac_algorithm_t mac,
+	        size_t master_size, const void *master,
+	        size_t label_size, const char *label,
+	        size_t seed_size, const char *seed, size_t outsize,
+	        char *out);
+
+/*-
+ * _gnutls_prf_raw:
+ * @mac: the MAC algorithm to use, set to %GNUTLS_MAC_UNKNOWN for the TLS1.0 mac
+ * @master_size: length of the @master variable.
+ * @master: the master secret used in PRF computation
+ * @label_size: length of the @label variable.
+ * @label: label used in PRF computation, typically a short string.
+ * @seed_size: length of the @seed variable.
+ * @seed: optional extra data to seed the PRF with.
+ * @outsize: size of pre-allocated output buffer to hold the output.
+ * @out: pre-allocated buffer to hold the generated data.
+ *
+ * Apply the TLS Pseudo-Random-Function (PRF) on the master secret
+ * and the provided data.
+ *
+ * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
+ -*/
+int
+_gnutls_prf_raw(gnutls_mac_algorithm_t mac,
+	        size_t master_size, const void *master,
+	        size_t label_size, const char *label,
+	        size_t seed_size, const char *seed, size_t outsize,
+	        char *out)
+{
+	return _gnutls_PRF_raw(mac,
+			  master, master_size,
+			  label, label_size,
+			  (uint8_t *) seed, seed_size,
+			  outsize, out);
 
 }
+#endif
 
 /**
  * gnutls_prf_raw:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @label_size: length of the @label variable.
  * @label: label used in PRF computation, typically a short string.
  * @seed_size: length of the @seed variable.
@@ -961,13 +1061,12 @@ gnutls_prf_raw(gnutls_session_t session,
 }
 
 /**
- * gnutls_prf:
- * @session: is a #gnutls_session_t structure.
+ * gnutls_prf_rfc5705:
+ * @session: is a #gnutls_session_t type.
  * @label_size: length of the @label variable.
  * @label: label used in PRF computation, typically a short string.
- * @server_random_first: non-0 if server random field should be first in seed
- * @extra_size: length of the @extra variable.
- * @extra: optional extra data to seed the PRF with.
+ * @context_size: length of the @extra variable.
+ * @context: optional extra data to seed the PRF with.
  * @outsize: size of pre-allocated output buffer to hold the output.
  * @out: pre-allocated buffer to hold the generated data.
  *
@@ -978,7 +1077,75 @@ gnutls_prf_raw(gnutls_session_t session,
  * The @label variable usually contains a string denoting the purpose
  * for the generated data.  The @server_random_first indicates whether
  * the client random field or the server random field should be first
- * in the seed.  Non-0 indicates that the server random field is first,
+ * in the seed.  Non-zero indicates that the server random field is first,
+ * 0 that the client random field is first.
+ *
+ * The @context variable can be used to add more data to the seed, after
+ * the random variables.  It can be used to make sure the
+ * generated output is strongly connected to some additional data
+ * (e.g., a string used in user authentication). 
+ *
+ * The output is placed in @out, which must be pre-allocated.
+ *
+ * Note that, to provide the RFC5705 context, the @contect variable
+ * must be non-null.
+ *
+ * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
+ *
+ * Since: 3.4.4
+ **/
+int
+gnutls_prf_rfc5705(gnutls_session_t session,
+		   size_t label_size, const char *label,
+		   size_t context_size, const char *context,
+		   size_t outsize, char *out)
+{
+	char *pctx = NULL;
+	int ret;
+
+	if (context != NULL && context_size > 65535)  {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	if (context != NULL) {
+		pctx = gnutls_malloc(context_size+2);
+		if (!pctx) {
+			gnutls_assert();
+			return GNUTLS_E_MEMORY_ERROR;
+		}
+
+		memcpy(pctx+2, context, context_size);
+		_gnutls_write_uint16(context_size, (void*)pctx);
+		context_size += 2;
+	}
+
+	ret = gnutls_prf(session, label_size, label, 0,
+			 context_size, pctx, outsize, out);
+
+	gnutls_free(pctx);
+	return ret;
+}
+
+/**
+ * gnutls_prf:
+ * @session: is a #gnutls_session_t type.
+ * @label_size: length of the @label variable.
+ * @label: label used in PRF computation, typically a short string.
+ * @server_random_first: non-zero if server random field should be first in seed
+ * @extra_size: length of the @extra variable.
+ * @extra: optional extra data to seed the PRF with.
+ * @outsize: size of pre-allocated output buffer to hold the output.
+ * @out: pre-allocated buffer to hold the generated data.
+ *
+ * Applies the TLS Pseudo-Random-Function (PRF) on the master secret
+ * and the provided data, seeded with the client and server random fields.
+ * For the key expansion specified in RFC5705 see gnutls_prf_rfc5705().
+ *
+ * The @label variable usually contains a string denoting the purpose
+ * for the generated data.  The @server_random_first indicates whether
+ * the client random field or the server random field should be first
+ * in the seed.  Non-zero indicates that the server random field is first,
  * 0 that the client random field is first.
  *
  * The @extra variable can be used to add more data to the seed, after
@@ -995,7 +1162,8 @@ gnutls_prf(gnutls_session_t session,
 	   size_t label_size,
 	   const char *label,
 	   int server_random_first,
-	   size_t extra_size, const char *extra, size_t outsize, char *out)
+	   size_t extra_size, const char *extra,
+	   size_t outsize, char *out)
 {
 	int ret;
 	uint8_t *seed;
@@ -1016,7 +1184,9 @@ gnutls_prf(gnutls_session_t session,
 	       client_random : session->security_parameters.server_random,
 	       GNUTLS_RANDOM_SIZE);
 
-	memcpy(seed + 2 * GNUTLS_RANDOM_SIZE, extra, extra_size);
+	if (extra && extra_size) {
+		memcpy(seed + 2 * GNUTLS_RANDOM_SIZE, extra, extra_size);
+	}
 
 	ret =
 	    _gnutls_PRF(session,
@@ -1031,7 +1201,7 @@ gnutls_prf(gnutls_session_t session,
 
 /**
  * gnutls_session_is_resumed:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Check whether session is resumed or not.
  *
@@ -1062,7 +1232,7 @@ int gnutls_session_is_resumed(gnutls_session_t session)
 
 /**
  * gnutls_session_resumption_requested:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Check whether the client has asked for session resumption.
  * This function is valid only on server side.
@@ -1080,7 +1250,7 @@ int gnutls_session_resumption_requested(gnutls_session_t session)
 
 /*-
  * _gnutls_session_is_psk - Used to check whether this session uses PSK kx
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * This function will return non zero if this session uses a PSK key
  * exchange algorithm.
@@ -1100,7 +1270,7 @@ int _gnutls_session_is_psk(gnutls_session_t session)
 
 /*-
  * _gnutls_session_is_ecc - Used to check whether this session uses ECC kx
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * This function will return non zero if this session uses an elliptic
  * curves key exchange exchange algorithm.
@@ -1120,7 +1290,7 @@ int _gnutls_session_is_ecc(gnutls_session_t session)
 
 /**
  * gnutls_session_get_ptr:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Get user pointer for session.  Useful in callbacks.  This is the
  *   pointer set with gnutls_session_set_ptr().
@@ -1135,7 +1305,7 @@ void *gnutls_session_get_ptr(gnutls_session_t session)
 
 /**
  * gnutls_session_set_ptr:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @ptr: is the user pointer
  *
  * This function will set (associate) the user given pointer @ptr to
@@ -1147,10 +1317,41 @@ void gnutls_session_set_ptr(gnutls_session_t session, void *ptr)
 	session->internals.user_ptr = ptr;
 }
 
+/**
+ * gnutls_session_set_verify_function:
+ * @session: is a #gnutls_session_t type.
+ * @func: is the callback function
+ *
+ * This function sets a callback to be called when peer's certificate
+ * has been received in order to verify it on receipt rather than
+ * doing after the handshake is completed. This overrides any callback
+ * set using gnutls_certificate_set_verify_function().
+ *
+ * The callback's function prototype is:
+ * int (*callback)(gnutls_session_t);
+ *
+ * If the callback function is provided then gnutls will call it, in the
+ * handshake, just after the certificate message has been received.
+ * To verify or obtain the certificate the gnutls_certificate_verify_peers2(),
+ * gnutls_certificate_type_get(), gnutls_certificate_get_peers() functions
+ * can be used.
+ *
+ * The callback function should return 0 for the handshake to continue
+ * or non-zero to terminate.
+ *
+ * Since: 3.4.6
+ **/
+void
+ gnutls_session_set_verify_function
+    (gnutls_session_t session,
+     gnutls_certificate_verify_function * func)
+{
+	session->internals.verify_callback = func;
+}
 
 /**
  * gnutls_record_get_direction:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * This function provides information about the internals of the
  * record protocol and is only useful if a prior gnutls function call
@@ -1163,6 +1364,9 @@ void gnutls_session_set_ptr(gnutls_session_t session, void *ptr)
  * interrupted function was trying to read data, and 1 if it was
  * trying to write data.
  *
+ * This function's output is unreliable if you are using the
+ * @session in different threads, for sending and receiving.
+ *
  * Returns: 0 if trying to read data, 1 if trying to write data.
  **/
 int gnutls_record_get_direction(gnutls_session_t session)
@@ -1172,7 +1376,7 @@ int gnutls_record_get_direction(gnutls_session_t session)
 
 /*-
  * _gnutls_rsa_pms_set_version - Sets a version to be used at the RSA PMS
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @major: is the major version to use
  * @minor: is the minor version to use
  *
@@ -1190,7 +1394,7 @@ _gnutls_rsa_pms_set_version(gnutls_session_t session,
 
 /**
  * gnutls_handshake_set_post_client_hello_function:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @func: is the function to be called
  *
  * This function will set a callback to be called after the client
@@ -1204,6 +1408,11 @@ _gnutls_rsa_pms_set_version(gnutls_session_t session,
  * This callback must return 0 on success or a gnutls error code to
  * terminate the handshake.
  *
+ * Since GnuTLS 3.3.5 the callback is
+ * allowed to return %GNUTLS_E_AGAIN or %GNUTLS_E_INTERRUPTED to
+ * put the handshake on hold. In that case gnutls_handshake()
+ * will return %GNUTLS_E_INTERRUPTED and can be resumed when needed.
+ *
  * Warning: You should not use this function to terminate the
  * handshake based on client input unless you know what you are
  * doing. Before the handshake is finished there is no way to know if
@@ -1220,7 +1429,7 @@ gnutls_handshake_set_post_client_hello_function(gnutls_session_t session,
 
 /**
  * gnutls_session_enable_compatibility_mode:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * This function can be used to disable certain (security) features in
  * TLS in order to maintain maximum compatibility with buggy
@@ -1240,7 +1449,7 @@ void gnutls_session_enable_compatibility_mode(gnutls_session_t session)
 
 /**
  * gnutls_session_channel_binding:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @cbtype: an #gnutls_channel_binding_t enumeration type
  * @cb: output buffer array with data
  *
@@ -1277,7 +1486,7 @@ gnutls_session_channel_binding(gnutls_session_t session,
 
 /**
  * gnutls_ecc_curve_get:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Returns the currently used elliptic curve. Only valid
  * when using an elliptic curve ciphersuite.
@@ -1294,7 +1503,7 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session)
 
 /**
  * gnutls_protocol_get_version:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Get TLS version, a #gnutls_protocol_t value.
  *
@@ -1307,7 +1516,7 @@ gnutls_protocol_t gnutls_protocol_get_version(gnutls_session_t session)
 
 /**
  * gnutls_session_get_random:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @client: the client part of the random
  * @server: the server part of the random
  *
@@ -1339,13 +1548,19 @@ gnutls_session_get_random(gnutls_session_t session,
 
 unsigned int timespec_sub_ms(struct timespec *a, struct timespec *b)
 {
-	return (a->tv_sec * 1000 + a->tv_nsec / (1000 * 1000) -
-		(b->tv_sec * 1000 + b->tv_nsec / (1000 * 1000)));
+	time_t dsecs;
+
+	dsecs = a->tv_sec - b->tv_sec;
+	if (!INT_MULTIPLY_OVERFLOW(dsecs, 1000)) {
+		return (dsecs*1000 + (a->tv_nsec - b->tv_nsec) / (1000 * 1000));
+	} else {
+		return UINT_MAX;
+	}
 }
 
 /**
  * gnutls_handshake_set_random:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @random: a random value of 32-bytes
  *
  * This function will explicitly set the server or client hello 
@@ -1381,7 +1596,7 @@ gnutls_handshake_set_random(gnutls_session_t session,
 
 /**
  * gnutls_handshake_set_hook_function:
- * @session: is a #gnutls_session_t structure
+ * @session: is a #gnutls_session_t type
  * @htype: the %gnutls_handshake_description_t of the message to hook at
  * @post: %GNUTLS_HOOK_* depending on when the hook function should be called
  * @func: is the function to be called
@@ -1390,7 +1605,7 @@ gnutls_handshake_set_random(gnutls_session_t session,
  * handshake message has been received or generated. This is a
  * generalization of gnutls_handshake_set_post_client_hello_function().
  *
- * To call the hook function prior to the message being sent/generation use 
+ * To call the hook function prior to the message being sent/generated use
  * %GNUTLS_HOOK_PRE as @post parameter, %GNUTLS_HOOK_POST to call
  * after, and %GNUTLS_HOOK_BOTH for both cases.
  *
@@ -1403,8 +1618,6 @@ gnutls_handshake_set_random(gnutls_session_t session,
  * handshake based on client input unless you know what you are
  * doing. Before the handshake is finished there is no way to know if
  * there is a man-in-the-middle attack being performed.
- *
- * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
  **/
 void
 gnutls_handshake_set_hook_function(gnutls_session_t session,
@@ -1416,3 +1629,103 @@ gnutls_handshake_set_hook_function(gnutls_session_t session,
 	session->internals.h_type = htype;
 	session->internals.h_post = post;
 }
+
+/**
+ * gnutls_record_get_state:
+ * @session: is a #gnutls_session_t type
+ * @read: if non-zero the read parameters are returned, otherwise the write
+ * @mac_key: the key used for MAC (if a MAC is used)
+ * @IV: the initialization vector or nonce used
+ * @cipher_key: the cipher key
+ * @seq_number: A 64-bit sequence number
+ *
+ * This function will return the parameters of the current record state.
+ * These are only useful to be provided to an external off-loading device
+ * or subsystem.
+ *
+ * In that case, to sync the state you must call gnutls_record_set_state().
+ *
+ * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
+ *
+ * Since 3.4.0
+ **/
+int
+gnutls_record_get_state(gnutls_session_t session,
+			unsigned read,
+			gnutls_datum_t *mac_key,
+			gnutls_datum_t *IV,
+			gnutls_datum_t *cipher_key,
+			unsigned char seq_number[8])
+{
+	record_parameters_st *record_params;
+	record_state_st *record_state;
+	unsigned int epoch;
+	int ret;
+
+	if (read)
+		epoch = EPOCH_READ_CURRENT;
+	else
+		epoch = EPOCH_WRITE_CURRENT;
+
+	ret = _gnutls_epoch_get(session, epoch, &record_params);
+	if (ret < 0)
+		return gnutls_assert_val(ret);
+
+	if (!record_params->initialized)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+	if (read)
+		record_state = &record_params->read;
+	else
+		record_state = &record_params->write;
+
+	memcpy(mac_key, &record_state->mac_secret, sizeof(gnutls_datum_t));
+	memcpy(IV, &record_state->IV, sizeof(gnutls_datum_t));
+	memcpy(cipher_key, &record_state->key, sizeof(gnutls_datum_t));
+	memcpy(seq_number, UINT64DATA(record_state->sequence_number), 8);
+	return 0;
+}
+
+/**
+ * gnutls_record_set_state:
+ * @session: is a #gnutls_session_t type
+ * @read: if non-zero the read parameters are returned, otherwise the write
+ * @seq_number: A 64-bit sequence number
+ *
+ * This function will set the sequence number in the current record state.
+ * This function is useful if sending and receiving are offloaded from
+ * gnutls. That is, if gnutls_record_get_state() was used.
+ *
+ * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
+ *
+ * Since 3.4.0
+ **/
+int
+gnutls_record_set_state(gnutls_session_t session,
+			unsigned read,
+			unsigned char seq_number[8])
+{
+	record_parameters_st *record_params;
+	record_state_st *record_state;
+	int epoch, ret;
+
+	if (read)
+		epoch = EPOCH_READ_CURRENT;
+	else
+		epoch = EPOCH_WRITE_CURRENT;
+
+	ret = _gnutls_epoch_get(session, epoch, &record_params);
+	if (ret < 0)
+		return gnutls_assert_val(ret);
+
+	if (!record_params->initialized)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+	if (read)
+		record_state = &record_params->read;
+	else
+		record_state = &record_params->write;
+
+	memcpy(UINT64DATA(record_state->sequence_number), seq_number, 8);
+	return 0;
+}
