@@ -15,13 +15,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  */
 
-#include "gnutls_int.h"
+#include <gnutls_int.h>
 #include <gnutls/pkcs11.h>
 #include <pin.h>
-#include "errors.h"
+#include <gnutls_errors.h>
 
 gnutls_pin_callback_t _gnutls_pin_func;
 void *_gnutls_pin_data;
@@ -37,7 +37,8 @@ void *_gnutls_pin_data;
  *
  * Since: 2.12.0
  **/
-void gnutls_pkcs11_set_pin_function(gnutls_pin_callback_t fn, void *userdata)
+void
+gnutls_pkcs11_set_pin_function(gnutls_pin_callback_t fn, void *userdata)
 {
 	_gnutls_pin_func = fn;
 	_gnutls_pin_data = userdata;
@@ -63,18 +64,23 @@ gnutls_pin_callback_t gnutls_pkcs11_get_pin_function(void **userdata)
 	return NULL;
 }
 
-int _gnutls_retrieve_pin(struct pin_info_st *pin_info, const char *url,
-			 const char *label, unsigned flags, char *pin,
-			 unsigned pin_size)
+int
+_gnutls_retrieve_pin(struct pin_info_st *pin_info, const char *url, const char *label,
+		     unsigned flags,
+		     char pin[GNUTLS_PKCS11_MAX_PIN_LEN], unsigned pin_size)
 {
 	int ret;
 
 	if (pin_info && pin_info->cb)
-		ret = pin_info->cb(pin_info->data, 0, (char *)url, label, flags,
-				   pin, pin_size);
+		ret =
+		    pin_info->cb(pin_info->data, 0,
+				 (char *) url, label, flags,
+				 pin, pin_size);
 	else if (_gnutls_pin_func)
-		ret = _gnutls_pin_func(_gnutls_pin_data, 0, (char *)url, label,
-				       flags, pin, pin_size);
+		ret =
+		    _gnutls_pin_func(_gnutls_pin_data, 0,
+				     (char *) url, label, flags,
+				     pin, pin_size);
 	else
 		ret = gnutls_assert_val(GNUTLS_E_PKCS11_PIN_ERROR);
 
