@@ -37,25 +37,23 @@ typedef unsigned char ck_bool_t;
 struct pkcs11_session_info {
 	struct ck_function_list *module;
 	struct ck_token_info tinfo;
+	struct ck_slot_info slot_info;
 	ck_session_handle_t pks;
-	unsigned int init;
-};
-
-struct token_info {
-	struct ck_token_info tinfo;
-	struct ck_slot_info sinfo;
 	ck_slot_id_t sid;
-	struct gnutls_pkcs11_provider_st *prov;
+	unsigned int init;
 };
 
 struct gnutls_pkcs11_obj_st {
 	gnutls_datum_t raw;
 	gnutls_pkcs11_obj_type_t type;
+	ck_object_class_t class;
+
 	unsigned int flags;
 	struct p11_kit_uri *info;
 
 	/* only when pubkey */
 	gnutls_datum_t pubkey[MAX_PUBLIC_PARAMS_SIZE];
+	unsigned pubkey_size;
 	gnutls_pk_algorithm_t pk_algorithm;
 	unsigned int key_usage;
 
@@ -66,6 +64,14 @@ struct gnutls_pkcs11_obj_st {
  * directly */
 int _gnutls_pkcs11_check_init(void);
 
+#define FIX_KEY_USAGE(pk, usage) \
+	if (usage == 0) { \
+		if (pk == GNUTLS_PK_RSA) \
+			usage = GNUTLS_KEY_DECIPHER_ONLY|GNUTLS_KEY_DIGITAL_SIGNATURE; \
+		else \
+			usage = GNUTLS_KEY_DIGITAL_SIGNATURE; \
+	}
+
 #define PKCS11_CHECK_INIT \
 	ret = _gnutls_pkcs11_check_init(); \
 	if (ret < 0) \
@@ -80,19 +86,20 @@ int _gnutls_pkcs11_check_init(void);
  * function. Once everything is traversed it is called with NULL tinfo.
  * It should return 0 if found what it was looking for.
  */
-typedef int (*find_func_t) (struct pkcs11_session_info *,
-			    struct token_info * tinfo, struct ck_info *,
+typedef int (*find_func_t) (struct ck_function_list *, struct pkcs11_session_info *,
+			    struct ck_token_info * tinfo, struct ck_info *,
 			    void *input);
 
 int pkcs11_rv_to_err(ck_rv_t rv);
-int pkcs11_url_to_info(const char *url, struct p11_kit_uri **info);
+int pkcs11_url_to_info(const char *url, struct p11_kit_uri **info, unsigned flags);
 int
 pkcs11_find_slot(struct ck_function_list **module, ck_slot_id_t * slot,
-		 struct p11_kit_uri *info, struct token_info *_tinfo);
+		 struct p11_kit_uri *info, struct ck_token_info *_tinfo,
+		 struct ck_slot_info *_slot_info);
 
 int pkcs11_read_pubkey(struct ck_function_list *module,
 		       ck_session_handle_t pks, ck_object_handle_t obj,
-		       ck_key_type_t key_type, gnutls_datum_t * pubkey);
+		       ck_key_type_t key_type, gnutls_pkcs11_obj_t pobj);
 
 int pkcs11_override_cert_exts(struct pkcs11_session_info *sinfo, gnutls_datum_t *spki, gnutls_datum_t *der);
 
@@ -101,8 +108,8 @@ int pkcs11_get_info(struct p11_kit_uri *info,
 		    size_t * output_size);
 int pkcs11_login(struct pkcs11_session_info *sinfo,
 		 struct pin_info_st *pin_info,
-		 const struct token_info *tokinfo,
-		 struct p11_kit_uri *info, int so);
+		 struct p11_kit_uri *info, unsigned so,
+		 unsigned reauth);
 
 int pkcs11_call_token_func(struct p11_kit_uri *info, const unsigned retry);
 
@@ -113,6 +120,10 @@ void pkcs11_rescan_slots(void);
 int pkcs11_info_to_url(struct p11_kit_uri *info,
 		       gnutls_pkcs11_url_type_t detailed, char **url);
 
+int
+_gnutls_x509_crt_import_pkcs11_url(gnutls_x509_crt_t crt,
+				  const char *url, unsigned int flags);
+
 #define SESSION_WRITE (1<<0)
 #define SESSION_LOGIN (1<<1)
 #define SESSION_SO (1<<2)	/* security officer session */
@@ -126,6 +137,13 @@ int _pkcs11_traverse_tokens(find_func_t find_func, void *input,
 			    unsigned int flags);
 ck_object_class_t pkcs11_strtype_to_class(const char *type);
 
+/* Additional internal flags for gnutls_pkcs11_obj_flags */
+/* @GNUTLS_PKCS11_OBJ_FLAG_EXPECT_CERT: When importing an object, provide a hint on the type, to allow incomplete URLs
+ * @GNUTLS_PKCS11_OBJ_FLAG_EXPECT_PRIVKEY: Hint for private key */
+#define GNUTLS_PKCS11_OBJ_FLAG_EXPECT_CERT (1<<29)
+#define GNUTLS_PKCS11_OBJ_FLAG_EXPECT_PRIVKEY (1<<30)
+#define GNUTLS_PKCS11_OBJ_FLAG_EXPECT_PUBKEY ((unsigned int)1<<31)
+
 int pkcs11_token_matches_info(struct p11_kit_uri *info,
 			      struct ck_token_info *tinfo,
 			      struct ck_info *lib_info);
@@ -156,7 +174,17 @@ static inline int pk_to_mech(gnutls_pk_algorithm_t pk)
 		return CKM_RSA_PKCS;
 }
 
-static inline gnutls_pk_algorithm_t mech_to_pk(ck_key_type_t m)
+static inline int pk_to_key_type(gnutls_pk_algorithm_t pk)
+{
+	if (pk == GNUTLS_PK_DSA)
+		return CKK_DSA;
+	else if (pk == GNUTLS_PK_EC)
+		return CKK_ECDSA;
+	else
+		return CKK_RSA;
+}
+
+static inline gnutls_pk_algorithm_t key_type_to_pk(ck_key_type_t m)
 {
 	if (m == CKK_RSA)
 		return GNUTLS_PK_RSA;
@@ -182,6 +210,8 @@ static inline int pk_to_genmech(gnutls_pk_algorithm_t pk, ck_key_type_t *type)
 	}
 }
 
+ck_object_class_t pkcs11_type_to_class(gnutls_pkcs11_obj_type_t type);
+
 ck_rv_t
 pkcs11_generate_key_pair(struct ck_function_list * module,
 			 ck_session_handle_t sess,
@@ -226,6 +256,13 @@ ck_rv_t pkcs11_find_objects_final(struct pkcs11_session_info *);
 
 ck_rv_t pkcs11_close_session(struct pkcs11_session_info *);
 
+ck_rv_t
+pkcs11_set_attribute_value(struct ck_function_list * module,
+			   ck_session_handle_t sess,
+			   ck_object_handle_t object,
+			   struct ck_attribute * templ,
+			   unsigned long count);
+
 ck_rv_t
 pkcs11_get_attribute_value(struct ck_function_list *module,
 			   ck_session_handle_t sess,
@@ -305,6 +342,13 @@ _gnutls_pkcs11_get_random(struct ck_function_list *module,
 
 const char *pkcs11_strerror(ck_rv_t rv);
 
+inline static bool is_object_pkcs11_url(const char *url)
+{
+	if (strstr(url, "id=") != 0 || strstr(url, "object=") != 0)
+		return 1;
+	return 0;
+}
+
 #endif				/* ENABLE_PKCS11 */
 
 #endif
