@@ -7,19 +7,28 @@
 const asn1_static_node pkix_asn1_tab[] = {
   { "PKIX1", 536875024, NULL },
   { NULL, 1073741836, NULL },
+  { "id-pkix", 1879048204, NULL },
+  { "iso", 1073741825, "1"},
+  { "identified-organization", 1073741825, "3"},
+  { "dod", 1073741825, "6"},
+  { "internet", 1073741825, "1"},
+  { "security", 1073741825, "5"},
+  { "mechanisms", 1073741825, "5"},
+  { "pkix", 1, "7"},
   { "PrivateKeyUsagePeriod", 1610612741, NULL },
   { "notBefore", 1610637349, NULL },
   { NULL, 4104, "0"},
   { "notAfter", 536895525, NULL },
   { NULL, 4104, "1"},
   { "AuthorityKeyIdentifier", 1610612741, NULL },
-  { "keyIdentifier", 1610637319, NULL },
+  { "keyIdentifier", 1610637314, "KeyIdentifier"},
   { NULL, 4104, "0"},
   { "authorityCertIssuer", 1610637314, "GeneralNames"},
   { NULL, 4104, "1"},
   { "authorityCertSerialNumber", 536895490, "CertificateSerialNumber"},
   { NULL, 4104, "2"},
-  { "SubjectKeyIdentifier", 1073741831, NULL },
+  { "KeyIdentifier", 1073741831, NULL },
+  { "SubjectKeyIdentifier", 1073741826, "KeyIdentifier"},
   { "KeyUsage", 1073741830, NULL },
   { "DirectoryString", 1610612754, NULL },
   { "teletexString", 1612709918, NULL },
@@ -30,7 +39,9 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "MAX", 524298, "1"},
   { "utf8String", 1612709922, NULL },
   { "MAX", 524298, "1"},
-  { "bmpString", 538968097, NULL },
+  { "bmpString", 1612709921, NULL },
+  { "MAX", 524298, "1"},
+  { "ia5String", 538968093, NULL },
   { "MAX", 524298, "1"},
   { "SubjectAltName", 1073741826, "GeneralNames"},
   { "GeneralNames", 1612709899, NULL },
@@ -45,9 +56,8 @@ const asn1_static_node pkix_asn1_tab[] = {
   { NULL, 4104, "2"},
   { "x400Address", 1610620941, NULL },
   { NULL, 4104, "3"},
-  { "directoryName", 1610620939, NULL },
-  { NULL, 1073743880, "4"},
-  { NULL, 2, "RelativeDistinguishedName"},
+  { "directoryName", 1610620930, "RDNSequence"},
+  { NULL, 2056, "4"},
   { "ediPartyName", 1610620941, NULL },
   { NULL, 4104, "5"},
   { "uniformResourceIdentifier", 1610620957, NULL },
@@ -65,7 +75,8 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "BasicConstraints", 1610612741, NULL },
   { "cA", 1610645508, NULL },
   { NULL, 131081, NULL },
-  { "pathLenConstraint", 16387, NULL },
+  { "pathLenConstraint", 537411587, NULL },
+  { "0", 10, "MAX"},
   { "CRLDistributionPoints", 1612709899, NULL },
   { "MAX", 1074266122, "1"},
   { NULL, 2, "DistributionPoint"},
@@ -84,7 +95,8 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "ReasonFlags", 1073741830, NULL },
   { "ExtKeyUsageSyntax", 1612709899, NULL },
   { "MAX", 1074266122, "1"},
-  { NULL, 12, NULL },
+  { NULL, 2, "KeyPurposeId"},
+  { "KeyPurposeId", 1073741836, NULL },
   { "AuthorityInfoAccessSyntax", 1612709899, NULL },
   { "MAX", 1074266122, "1"},
   { NULL, 2, "AccessDescription"},
@@ -92,17 +104,32 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "accessMethod", 1073741836, NULL },
   { "accessLocation", 2, "GeneralName"},
   { "Attribute", 1610612741, NULL },
-  { "type", 1073741836, NULL },
+  { "type", 1073741826, "AttributeType"},
   { "values", 536870927, NULL },
-  { NULL, 13, NULL },
+  { NULL, 2, "AttributeValue"},
+  { "AttributeType", 1073741836, NULL },
+  { "AttributeValue", 1614807053, NULL },
+  { "type", 1, NULL },
   { "AttributeTypeAndValue", 1610612741, NULL },
-  { "type", 1073741836, NULL },
-  { "value", 13, NULL },
+  { "type", 1073741826, "AttributeType"},
+  { "value", 2, "AttributeValue"},
+  { "id-at", 1879048204, NULL },
+  { "joint-iso-ccitt", 1073741825, "2"},
+  { "ds", 1073741825, "5"},
+  { NULL, 1, "4"},
+  { "emailAddress", 1880096780, "AttributeType"},
+  { "iso", 1073741825, "1"},
+  { "member-body", 1073741825, "2"},
+  { "us", 1073741825, "840"},
+  { "rsadsi", 1073741825, "113549"},
+  { "pkcs", 1073741825, "1"},
+  { NULL, 1073741825, "9"},
+  { NULL, 1, "1"},
   { "Name", 1610612754, NULL },
-  { "rdnSequence", 536870923, NULL },
-  { NULL, 2, "RelativeDistinguishedName"},
-  { "DistinguishedName", 1610612747, NULL },
+  { "rdnSequence", 2, "RDNSequence"},
+  { "RDNSequence", 1610612747, NULL },
   { NULL, 2, "RelativeDistinguishedName"},
+  { "DistinguishedName", 1073741826, "RDNSequence"},
   { "RelativeDistinguishedName", 1612709903, NULL },
   { "MAX", 1074266122, "1"},
   { NULL, 2, "AttributeTypeAndValue"},
@@ -169,18 +196,44 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "Dss-Sig-Value", 1610612741, NULL },
   { "r", 1073741827, NULL },
   { "s", 3, NULL },
+  { "DomainParameters", 1610612741, NULL },
+  { "p", 1073741827, NULL },
+  { "g", 1073741827, NULL },
+  { "q", 1073741827, NULL },
+  { "j", 1073758211, NULL },
+  { "validationParms", 16386, "ValidationParms"},
+  { "ValidationParms", 1610612741, NULL },
+  { "seed", 1073741830, NULL },
+  { "pgenCounter", 3, NULL },
   { "Dss-Parms", 1610612741, NULL },
   { "p", 1073741827, NULL },
   { "q", 1073741827, NULL },
   { "g", 3, NULL },
+  { "CountryName", 1610620946, NULL },
+  { NULL, 1073746952, "1"},
+  { "x121-dcc-code", 1612709916, NULL },
+  { NULL, 1048586, "ub-country-name-numeric-length"},
+  { "iso-3166-alpha2-code", 538968095, NULL },
+  { NULL, 1048586, "ub-country-name-alpha-length"},
+  { "OrganizationName", 1612709919, NULL },
+  { "ub-organization-name-length", 524298, "1"},
+  { "NumericUserIdentifier", 1612709916, NULL },
+  { "ub-numeric-user-id-length", 524298, "1"},
+  { "OrganizationalUnitNames", 1612709899, NULL },
+  { "ub-organizational-units", 1074266122, "1"},
+  { NULL, 2, "OrganizationalUnitName"},
+  { "OrganizationalUnitName", 1612709919, NULL },
+  { "ub-organizational-unit-name-length", 524298, "1"},
+  { "CommonName", 1073741855, NULL },
   { "pkcs-7-ContentInfo", 1610612741, NULL },
-  { "contentType", 1073741836, NULL },
+  { "contentType", 1073741826, "pkcs-7-ContentType"},
   { "content", 541073421, NULL },
   { NULL, 1073743880, "0"},
   { "contentType", 1, NULL },
   { "pkcs-7-DigestInfo", 1610612741, NULL },
   { "digestAlgorithm", 1073741826, "AlgorithmIdentifier"},
   { "digest", 7, NULL },
+  { "pkcs-7-ContentType", 1073741836, NULL },
   { "pkcs-7-SignedData", 1610612741, NULL },
   { "version", 1073741827, NULL },
   { "digestAlgorithms", 1073741826, "pkcs-7-DigestAlgorithmIdentifiers"},
@@ -193,8 +246,8 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "pkcs-7-DigestAlgorithmIdentifiers", 1610612751, NULL },
   { NULL, 2, "AlgorithmIdentifier"},
   { "pkcs-7-EncapsulatedContentInfo", 1610612741, NULL },
-  { "eContentType", 1073741836, NULL },
-  { "eContent", 536895501, NULL },
+  { "eContentType", 1073741826, "pkcs-7-ContentType"},
+  { "eContent", 536895495, NULL },
   { NULL, 2056, "0"},
   { "pkcs-7-CertificateRevocationLists", 1610612751, NULL },
   { NULL, 13, NULL },
@@ -220,7 +273,7 @@ const asn1_static_node pkix_asn1_tab[] = {
   { NULL, 2, "Attribute"},
   { "SignerIdentifier", 1610612754, NULL },
   { "issuerAndSerialNumber", 1073741826, "IssuerAndSerialNumber"},
-  { "subjectKeyIdentifier", 536879111, NULL },
+  { "subjectKeyIdentifier", 536879106, "SubjectKeyIdentifier"},
   { NULL, 4104, "0"},
   { "pkcs-7-SignerInfos", 1610612751, NULL },
   { NULL, 2, "pkcs-7-SignerInfo"},
@@ -252,10 +305,10 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "version", 1073741827, NULL },
   { "privateKeyAlgorithm", 1073741826, "AlgorithmIdentifier"},
   { "privateKey", 1073741831, NULL },
-  { "attributes", 1610637314, "Attributes"},
+  { "attributes", 536895490, "Attributes"},
   { NULL, 4104, "0"},
-  { "publicKey", 536895494, NULL },
-  { NULL, 4104, "1"},
+  { "pkcs-8-Attributes", 1610612751, NULL },
+  { NULL, 2, "Attribute"},
   { "pkcs-8-EncryptedPrivateKeyInfo", 1610612741, NULL },
   { "encryptionAlgorithm", 1073741826, "AlgorithmIdentifier"},
   { "encryptedData", 2, "pkcs-8-EncryptedData"},
@@ -270,24 +323,17 @@ const asn1_static_node pkix_asn1_tab[] = {
   { NULL, 1048586, "16"},
   { "pkcs-5-aes256-CBC-params", 1612709895, NULL },
   { NULL, 1048586, "16"},
-  { "Gost28147-89-Parameters", 1610612741, NULL },
-  { "iv", 1073741831, NULL },
-  { "encryptionParamSet", 12, NULL },
-  { "pkcs-5-PBE-params", 1610612741, NULL },
-  { "salt", 1073741831, NULL },
-  { "iterationCount", 3, NULL },
   { "pkcs-5-PBES2-params", 1610612741, NULL },
   { "keyDerivationFunc", 1073741826, "AlgorithmIdentifier"},
   { "encryptionScheme", 2, "AlgorithmIdentifier"},
-  { "pkcs-5-PBMAC1-params", 1610612741, NULL },
-  { "keyDerivationFunc", 1073741826, "AlgorithmIdentifier"},
-  { "messageAuthScheme", 2, "AlgorithmIdentifier"},
   { "pkcs-5-PBKDF2-params", 1610612741, NULL },
   { "salt", 1610612754, NULL },
   { "specified", 1073741831, NULL },
   { "otherSource", 2, "AlgorithmIdentifier"},
-  { "iterationCount", 1073741827, NULL },
-  { "keyLength", 1073758211, NULL },
+  { "iterationCount", 1611137027, NULL },
+  { "1", 10, "MAX"},
+  { "keyLength", 1611153411, NULL },
+  { "1", 10, "MAX"},
   { "prf", 16386, "AlgorithmIdentifier"},
   { "pkcs-12-PFX", 1610612741, NULL },
   { "version", 1610874883, NULL },
@@ -335,7 +381,7 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "unprotectedAttrs", 536895490, "pkcs-7-UnprotectedAttributes"},
   { NULL, 4104, "1"},
   { "pkcs-7-EncryptedContentInfo", 1610612741, NULL },
-  { "contentType", 1073741836, NULL },
+  { "contentType", 1073741826, "pkcs-7-ContentType"},
   { "contentEncryptionAlgorithm", 1073741826, "pkcs-7-ContentEncryptionAlgorithmIdentifier"},
   { "encryptedContent", 536895495, NULL },
   { NULL, 4104, "0"},
@@ -344,7 +390,8 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "MAX", 1074266122, "1"},
   { NULL, 2, "Attribute"},
   { "ProxyCertInfo", 1610612741, NULL },
-  { "pCPathLenConstraint", 1073758211, NULL },
+  { "pCPathLenConstraint", 1611153411, NULL },
+  { "0", 10, "MAX"},
   { "proxyPolicy", 2, "ProxyPolicy"},
   { "ProxyPolicy", 1610612741, NULL },
   { "policyLanguage", 1073741836, NULL },
@@ -439,11 +486,10 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "responseExtensions", 536895490, "Extensions"},
   { NULL, 2056, "1"},
   { "ResponderID", 1610612754, NULL },
-  { "byName", 1610620939, NULL },
-  { NULL, 1073743880, "1"},
-  { NULL, 2, "RelativeDistinguishedName"},
+  { "byName", 1610620930, "RDNSequence"},
+  { NULL, 2056, "1"},
   { "byKey", 536879111, NULL },
-  { NULL, 2056, "2"},
+  { NULL, 4104, "2"},
   { "SingleResponse", 1610612741, NULL },
   { "certID", 1073741826, "CertID"},
   { "certStatus", 1073741826, "CertStatus"},
@@ -461,10 +507,20 @@ const asn1_static_node pkix_asn1_tab[] = {
   { NULL, 4104, "2"},
   { "RevokedInfo", 1610612741, NULL },
   { "revocationTime", 1073741861, NULL },
-  { "revocationReason", 537157653, NULL },
-  { NULL, 1073743880, "0"},
-  { "unspecified", 1, "0"},
+  { "revocationReason", 536895490, "CRLReason"},
+  { NULL, 2056, "0"},
   { "UnknownInfo", 1073741844, NULL },
+  { "CRLReason", 1610874901, NULL },
+  { "unspecified", 1073741825, "0"},
+  { "keyCompromise", 1073741825, "1"},
+  { "cACompromise", 1073741825, "2"},
+  { "affiliationChanged", 1073741825, "3"},
+  { "superseded", 1073741825, "4"},
+  { "cessationOfOperation", 1073741825, "5"},
+  { "certificateHold", 1073741825, "6"},
+  { "removeFromCRL", 1073741825, "8"},
+  { "privilegeWithdrawn", 1073741825, "9"},
+  { "aACompromise", 1, "10"},
   { "NameConstraints", 1610612741, NULL },
   { "permittedSubtrees", 1610637314, "GeneralSubtrees"},
   { NULL, 4104, "0"},
@@ -473,14 +529,12 @@ const asn1_static_node pkix_asn1_tab[] = {
   { "GeneralSubtrees", 1612709899, NULL },
   { "MAX", 1074266122, "1"},
   { NULL, 2, "GeneralSubtree"},
-  { "GeneralSubtree", 1610612741, NULL },
+  { "GeneralSubtree", 536870917, NULL },
   { "base", 1073741826, "GeneralName"},
   { "minimum", 1610653699, NULL },
   { NULL, 1073741833, "0"},
   { NULL, 4104, "0"},
   { "maximum", 536895491, NULL },
   { NULL, 4104, "1"},
-  { "TlsFeatures", 536870923, NULL },
-  { NULL, 3, NULL },
   { NULL, 0, NULL }
 };
