@@ -71,7 +71,6 @@ typedef TSS_RESULT (*Tspi_DecodeBER_TssBlob_func)(UINT32, BYTE*, UINT32*, UINT32
 typedef TSS_RESULT (*Tspi_Context_LoadKeyByBlob_func)(TSS_HCONTEXT, TSS_HKEY, UINT32, BYTE*, TSS_HKEY*);
 typedef TSS_RESULT (*Tspi_Policy_AssignToObject_func)(TSS_HPOLICY, TSS_HOBJECT);
 typedef TSS_RESULT (*Tspi_GetAttribData_func)(TSS_HOBJECT, TSS_FLAG, TSS_FLAG, UINT32*, BYTE**);
-typedef TSS_RESULT (*Tspi_GetAttribUint32_func)(TSS_HOBJECT, TSS_FLAG, TSS_FLAG, UINT32*);
 typedef TSS_RESULT (*Tspi_Context_GetTpmObject_func)(TSS_HCONTEXT, TSS_HTPM*);
 typedef TSS_RESULT (*Tspi_TPM_StirRandom_func)(TSS_HTPM, UINT32, BYTE*);
 typedef TSS_RESULT (*Tspi_SetAttribUint32_func)(TSS_HOBJECT, TSS_FLAG, TSS_FLAG, UINT32);
@@ -97,7 +96,6 @@ static Tspi_DecodeBER_TssBlob_func pTspi_DecodeBER_TssBlob;
 static Tspi_Context_LoadKeyByBlob_func pTspi_Context_LoadKeyByBlob;
 static Tspi_Policy_AssignToObject_func pTspi_Policy_AssignToObject;
 static Tspi_GetAttribData_func pTspi_GetAttribData;
-static Tspi_GetAttribUint32_func pTspi_GetAttribUint32;
 static Tspi_Context_GetTpmObject_func pTspi_Context_GetTpmObject;
 static Tspi_TPM_StirRandom_func pTspi_TPM_StirRandom;
 static Tspi_SetAttribUint32_func pTspi_SetAttribUint32;
@@ -146,7 +144,6 @@ static int check_init(void)
 		_DLSYM(tpm_dl,Tspi_Context_LoadKeyByBlob);
 		_DLSYM(tpm_dl,Tspi_Policy_AssignToObject);
 		_DLSYM(tpm_dl,Tspi_GetAttribData);
-		_DLSYM(tpm_dl,Tspi_GetAttribUint32);
 		_DLSYM(tpm_dl,Tspi_Context_GetTpmObject);
 		_DLSYM(tpm_dl,Tspi_TPM_StirRandom);
 		_DLSYM(tpm_dl,Tspi_SetAttribUint32);
@@ -336,17 +333,7 @@ static int tpm_pin(struct pin_info_st *pin_info, const TSS_UUID * uuid,
 	} else
 		label = "unknown";
 
-	if (pin_info && pin_info->cb)
-		ret =
-		    pin_info->cb(pin_info->data, attempts, url, label,
-				 flags, pin, pin_size);
-	else if (_gnutls_pin_func)
-		ret =
-		    _gnutls_pin_func(_gnutls_pin_data, attempts, url,
-				     label, flags, pin, pin_size);
-	else
-		ret = gnutls_assert_val(GNUTLS_E_TPM_KEY_PASSWORD_ERROR);	/* doesn't really matter */
-
+	ret = _gnutls_retrieve_pin(pin_info, url, label, flags, pin, pin_size);
 	if (ret < 0) {
 		gnutls_assert();
 		goto cleanup;
@@ -507,8 +494,8 @@ static int load_key(TSS_HCONTEXT tpm_ctx, TSS_HKEY srk,
 		gnutls_datum_t td;
 
 		ret =
-		    gnutls_pem_base64_decode_alloc("TSS KEY BLOB", fdata,
-						   &asn1);
+		    gnutls_pem_base64_decode2("TSS KEY BLOB", fdata,
+					      &asn1);
 		if (ret) {
 			gnutls_assert();
 			_gnutls_debug_log
@@ -581,7 +568,6 @@ import_tpm_key(gnutls_privkey_t pkey,
 	int err, ret;
 	struct tpm_ctx_st *s;
 	gnutls_datum_t tmp_sig;
-	uint32_t authusage;
 
 	s = gnutls_malloc(sizeof(*s));
 	if (s == NULL) {
@@ -619,50 +605,6 @@ import_tpm_key(gnutls_privkey_t pkey,
 		goto out_session;
 	}
 
-	err = pTspi_GetAttribUint32(s->tpm_key, TSS_TSPATTRIB_KEY_INFO,
-				    TSS_TSPATTRIB_KEYINFO_AUTHUSAGE,
-				    &authusage);
-	if (err) {
-		gnutls_assert();
-		ret = tss_err(err);
-		goto out_session;
-	}
-
-	if (authusage) {
-		if (!key_password) {
-			ret = GNUTLS_E_TPM_KEY_PASSWORD_ERROR;
-			goto out_session;
-		}
-
-		err = pTspi_Context_CreateObject(s->tpm_ctx,
-						 TSS_OBJECT_TYPE_POLICY,
-						 TSS_POLICY_USAGE,
-						 &s->tpm_key_policy);
-		if (err) {
-			gnutls_assert();
-			ret = tss_err(err);
-			goto out_key;
-		}
-
-		err = pTspi_Policy_AssignToObject(s->tpm_key_policy,
-						       s->tpm_key);
-		if (err) {
-			gnutls_assert();
-			ret = tss_err(err);
-			goto out_key_policy;
-		}
-
-		err = myTspi_Policy_SetSecret(s->tpm_key_policy,
-					      SAFE_LEN(key_password),
-					      (void *) key_password);
-
-		if (err) {
-			gnutls_assert();
-			ret = tss_err_key(err);
-			goto out_key_policy;
-		}
-	}
-
 	ret =
 	    gnutls_privkey_import_ext2(pkey, GNUTLS_PK_RSA, s,
 				       tpm_sign_fn, NULL, tpm_deinit_fn,
@@ -675,7 +617,39 @@ import_tpm_key(gnutls_privkey_t pkey,
 	ret =
 	    gnutls_privkey_sign_data(pkey, GNUTLS_DIG_SHA1, 0, &nulldata,
 				     &tmp_sig);
-	if (ret < 0) {
+	if (ret == GNUTLS_E_TPM_KEY_PASSWORD_ERROR) {
+		if (!s->tpm_key_policy) {
+			err = pTspi_Context_CreateObject(s->tpm_ctx,
+							TSS_OBJECT_TYPE_POLICY,
+							TSS_POLICY_USAGE,
+							&s->
+							tpm_key_policy);
+			if (err) {
+				gnutls_assert();
+				ret = tss_err(err);
+				goto out_key;
+			}
+
+			err =
+			    pTspi_Policy_AssignToObject(s->tpm_key_policy,
+						       s->tpm_key);
+			if (err) {
+				gnutls_assert();
+				ret = tss_err(err);
+				goto out_key_policy;
+			}
+		}
+
+		err = myTspi_Policy_SetSecret(s->tpm_key_policy,
+					      SAFE_LEN(key_password),
+					      (void *) key_password);
+
+		if (err) {
+			gnutls_assert();
+			ret = tss_err_key(err);
+			goto out_key_policy;
+		}
+	} else if (ret < 0) {
 		gnutls_assert();
 		goto out_session;
 	}
@@ -688,7 +662,6 @@ import_tpm_key(gnutls_privkey_t pkey,
 	pTspi_Context_CloseObject(s->tpm_ctx, s->tpm_key);
 	s->tpm_key = 0;
       out_session:
-	_gnutls_privkey_cleanup(pkey);
 	tpm_close_session(s);
       out_ctx:
 	gnutls_free(s);
@@ -705,7 +678,7 @@ import_tpm_key(gnutls_privkey_t pkey,
  * @flags: should be zero
  *
  * This function will import the given private key to the abstract
- * #gnutls_privkey_t structure. 
+ * #gnutls_privkey_t type. 
  *
  * With respect to passwords the same as in gnutls_privkey_import_tpm_url() apply.
  *
@@ -877,7 +850,7 @@ static int encode_tpmkey_url(char **url, const TSS_UUID * uuid,
 		goto cleanup;
 	}
 
-	ret = _gnutls_buffer_to_datum(&buf, &dret);
+	ret = _gnutls_buffer_to_datum(&buf, &dret, 1);
 	if (ret < 0) {
 		gnutls_assert();
 		goto cleanup;
@@ -920,6 +893,7 @@ static int decode_tpmkey_url(const char *url, struct tpmkey_url_st *s)
 	} else if ((p = strstr(url, "uuid=")) != NULL) {
 		char tmp_uuid[33];
 		uint8_t raw_uuid[16];
+		gnutls_datum_t tmp;
 
 		p += sizeof("uuid=") - 1;
 		size = strlen(p);
@@ -933,10 +907,12 @@ static int decode_tpmkey_url(const char *url, struct tpmkey_url_st *s)
 		}
 		tmp_uuid[j] = 0;
 
+		tmp.data = (void*)tmp_uuid;
+		tmp.size = strlen(tmp_uuid);
 		size = sizeof(raw_uuid);
 		ret =
-		    _gnutls_hex2bin(tmp_uuid, strlen(tmp_uuid), raw_uuid,
-				    &size);
+		    gnutls_hex_decode(&tmp, raw_uuid,
+				      &size);
 		if (ret < 0) {
 			gnutls_assert();
 			goto cleanup;
@@ -974,7 +950,7 @@ static int decode_tpmkey_url(const char *url, struct tpmkey_url_st *s)
  * @flags: One of the GNUTLS_PRIVKEY_* flags
  *
  * This function will import the given private key to the abstract
- * #gnutls_privkey_t structure.
+ * #gnutls_privkey_t type.
  *
  * Note that unless %GNUTLS_PRIVKEY_DISABLE_CALLBACKS
  * is specified, if incorrect (or NULL) passwords are given
@@ -1239,7 +1215,7 @@ gnutls_pubkey_import_tpm_raw(gnutls_pubkey_t pkey,
  * @flags: should be zero
  *
  * This function will import the given private key to the abstract
- * #gnutls_privkey_t structure. 
+ * #gnutls_privkey_t type. 
  *
  * Note that unless %GNUTLS_PUBKEY_DISABLE_CALLBACKS
  * is specified, if incorrect (or NULL) passwords are given
@@ -1339,7 +1315,7 @@ gnutls_pubkey_import_tpm_url(gnutls_pubkey_t pkey,
  * Allowed flags are:
  *
  * %GNUTLS_TPM_KEY_SIGNING: Generate a signing key instead of a legacy,
-
+ *
  * %GNUTLS_TPM_REGISTER_KEY: Register the generate key in TPM. In that
  * case @privkey would contain a URL with the UUID.
  *
