@@ -30,106 +30,128 @@
 #include <gnutls_num.h>
 #include <x509_b64.h>
 #include "x509_int.h"
+#include "extras/hex.h"
 #include <common.h>
 #include <c-ctype.h>
 
 static int
 data2hex(const void *data, size_t data_size,
-	 void *_out, size_t * sizeof_out);
+	 gnutls_datum_t *out);
 
 struct oid_to_string {
 	const char *oid;
+	unsigned oid_size;
 	const char *ldap_desc;
+	unsigned ldap_desc_size;
 	const char *asn_desc;	/* description in the pkix file if complex type */
 	unsigned int etype;	/* the libtasn1 ASN1_ETYPE or INVALID
 				 * if cannot be simply parsed */
 };
 
+#define ENTRY(oid, ldap, asn, etype) {oid, sizeof(oid)-1, ldap, sizeof(ldap)-1, asn, etype}
+
+/* when there is no ldap description */
+#define ENTRY_ND(oid, asn, etype) {oid, sizeof(oid)-1, NULL, 0, asn, etype}
+
 /* This list contains all the OIDs that may be
  * contained in a rdnSequence and are printable.
  */
 static const struct oid_to_string _oid2str[] = {
 	/* PKIX
 	 */
-	{"1.3.6.1.5.5.7.9.2", "placeOfBirth", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
-	{"1.3.6.1.5.5.7.9.3", "gender", NULL, ASN1_ETYPE_PRINTABLE_STRING},
-	{"1.3.6.1.5.5.7.9.4", "countryOfCitizenship", NULL,
-	 ASN1_ETYPE_PRINTABLE_STRING},
-	{"1.3.6.1.5.5.7.9.5", "countryOfResidence", NULL,
-	 ASN1_ETYPE_PRINTABLE_STRING},
+	ENTRY("1.3.6.1.5.5.7.9.2", "placeOfBirth", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
+	ENTRY("1.3.6.1.5.5.7.9.3", "gender", NULL, ASN1_ETYPE_PRINTABLE_STRING),
+	ENTRY("1.3.6.1.5.5.7.9.4", "countryOfCitizenship", NULL,
+	 ASN1_ETYPE_PRINTABLE_STRING),
+	ENTRY("1.3.6.1.5.5.7.9.5", "countryOfResidence", NULL,
+	 ASN1_ETYPE_PRINTABLE_STRING),
 
-	{"2.5.4.6", "C", NULL, ASN1_ETYPE_PRINTABLE_STRING},
-	{"2.5.4.9", "street", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-	{"2.5.4.12", "title", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-	{"2.5.4.10", "O", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-	{"2.5.4.11", "OU", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-	{"2.5.4.3", "CN", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-	{"2.5.4.7", "L", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-	{"2.5.4.8", "ST", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-	{"2.5.4.13", "description", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
+	ENTRY("2.5.4.6", "C", NULL, ASN1_ETYPE_PRINTABLE_STRING),
+	ENTRY("2.5.4.9", "street", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.12", "title", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.10", "O", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.11", "OU", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.3", "CN", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.7", "L", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.8", "ST", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.13", "description", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
 
-	{"2.5.4.5", "serialNumber", NULL, ASN1_ETYPE_PRINTABLE_STRING},
-	{"2.5.4.20", "telephoneNumber", NULL, ASN1_ETYPE_PRINTABLE_STRING},
-	{"2.5.4.4", "surName", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
-	{"2.5.4.43", "initials", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
-	{"2.5.4.44", "generationQualifier", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
-	{"2.5.4.42", "givenName", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
-	{"2.5.4.65", "pseudonym", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
-	{"2.5.4.46", "dnQualifier", NULL, ASN1_ETYPE_PRINTABLE_STRING},
-	{"2.5.4.17", "postalCode", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
-	{"2.5.4.41", "name", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-	{"2.5.4.15", "businessCategory", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
+	ENTRY("2.5.4.5", "serialNumber", NULL, ASN1_ETYPE_PRINTABLE_STRING),
+	ENTRY("2.5.4.20", "telephoneNumber", NULL, ASN1_ETYPE_PRINTABLE_STRING),
+	ENTRY("2.5.4.4", "surName", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.43", "initials", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.44", "generationQualifier", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.42", "givenName", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.65", "pseudonym", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.46", "dnQualifier", NULL, ASN1_ETYPE_PRINTABLE_STRING),
+	ENTRY("2.5.4.17", "postalCode", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.41", "name", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID),
+	ENTRY("2.5.4.15", "businessCategory", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
 
-	{"0.9.2342.19200300.100.1.25", "DC", NULL, ASN1_ETYPE_IA5_STRING},
-	{"0.9.2342.19200300.100.1.1", "UID", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
-	{"1.2.840.113556.1.4.656", "userPrincipalName", "PKIX1.DirectoryString",
-	 ASN1_ETYPE_INVALID},
+	ENTRY("0.9.2342.19200300.100.1.25", "DC", NULL, ASN1_ETYPE_IA5_STRING),
+	ENTRY("0.9.2342.19200300.100.1.1", "UID", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
+	ENTRY("1.2.840.113556.1.4.656", "userPrincipalName", "PKIX1.DirectoryString",
+	 ASN1_ETYPE_INVALID),
 
 	/* Extended validation
 	 */
-	{"1.3.6.1.4.1.311.60.2.1.1",
+	ENTRY("1.3.6.1.4.1.311.60.2.1.1",
 	 "jurisdictionOfIncorporationLocalityName",
-	 "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-	{"1.3.6.1.4.1.311.60.2.1.2",
+	 "PKIX1.DirectoryString", ASN1_ETYPE_INVALID),
+	ENTRY("1.3.6.1.4.1.311.60.2.1.2",
 	 "jurisdictionOfIncorporationStateOrProvinceName",
-	 "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-	{"1.3.6.1.4.1.311.60.2.1.3",
+	 "PKIX1.DirectoryString", ASN1_ETYPE_INVALID),
+	ENTRY("1.3.6.1.4.1.311.60.2.1.3",
 	 "jurisdictionOfIncorporationCountryName",
-	 NULL, ASN1_ETYPE_PRINTABLE_STRING},
+	 NULL, ASN1_ETYPE_PRINTABLE_STRING),
 
 	/* PKCS #9
 	 */
-	{"1.2.840.113549.1.9.1", "EMAIL", NULL, ASN1_ETYPE_IA5_STRING},
-	{"1.2.840.113549.1.9.7", NULL, "PKIX1.pkcs-9-challengePassword",
-	 ASN1_ETYPE_INVALID},
+	ENTRY("1.2.840.113549.1.9.1", "EMAIL", NULL, ASN1_ETYPE_IA5_STRING),
+	ENTRY_ND("1.2.840.113549.1.9.7", "PKIX1.pkcs-9-challengePassword",
+	 ASN1_ETYPE_INVALID),
 
 	/* friendly name */
-	{"1.2.840.113549.1.9.20", NULL, NULL, ASN1_ETYPE_BMP_STRING},
+	ENTRY_ND("1.2.840.113549.1.9.20", NULL, ASN1_ETYPE_BMP_STRING),
 	/* local key id */
-	{"1.2.840.113549.1.9.21", NULL, NULL, ASN1_ETYPE_OCTET_STRING},
+	ENTRY_ND("1.2.840.113549.1.9.21", NULL, ASN1_ETYPE_OCTET_STRING),
+	ENTRY_ND("1.2.840.113549.1.9.4", NULL, ASN1_ETYPE_OCTET_STRING),
 
 	/* rfc3920 section 5.1.1 */
-	{"1.3.6.1.5.5.7.8.5", "XmppAddr", NULL, ASN1_ETYPE_UTF8_STRING},
+	ENTRY("1.3.6.1.5.5.7.8.5", "XmppAddr", NULL, ASN1_ETYPE_UTF8_STRING),
 
-	{NULL, NULL, NULL, 0}
+	{NULL, 0, NULL, 0, NULL, 0}
 };
 
+int _san_othername_to_virtual(const char *oid, size_t size)
+{
+	if (oid) {
+		if ((unsigned) size == (sizeof(XMPP_OID)-1)
+		    && memcmp(oid, XMPP_OID, sizeof(XMPP_OID)-1) == 0)
+			return GNUTLS_SAN_OTHERNAME_XMPP;
+	}
+
+	return GNUTLS_SAN_OTHERNAME;
+} 
+
 static const struct oid_to_string *get_oid_entry(const char *oid)
 {
 	unsigned int i = 0;
+	unsigned len = strlen(oid);
 
 	do {
-		if (strcmp(_oid2str[i].oid, oid) == 0)
+		if (len == _oid2str[i].oid_size &&
+			strcmp(_oid2str[i].oid, oid) == 0)
 			return &_oid2str[i];
 		i++;
 	}
@@ -144,7 +166,7 @@ const char *_gnutls_ldap_string_to_oid(const char *str, unsigned str_len)
 
 	do {
 		if ((_oid2str[i].ldap_desc != NULL) &&
-		    (str_len == strlen(_oid2str[i].ldap_desc)) &&
+		    (str_len == _oid2str[i].ldap_desc_size) &&
 		    (strncasecmp(_oid2str[i].ldap_desc, str, str_len) ==
 		     0))
 			return _oid2str[i].oid;
@@ -221,9 +243,11 @@ static int str_escape(const gnutls_datum_t * str, gnutls_datum_t * escaped)
 int gnutls_x509_dn_oid_known(const char *oid)
 {
 	unsigned int i = 0;
+	unsigned len = strlen(oid);
 
 	do {
-		if (strcmp(_oid2str[i].oid, oid) == 0)
+		if (len == _oid2str[i].oid_size &&
+			strcmp(_oid2str[i].oid, oid) == 0)
 			return 1;
 		i++;
 	}
@@ -249,9 +273,11 @@ int gnutls_x509_dn_oid_known(const char *oid)
 const char *gnutls_x509_dn_oid_name(const char *oid, unsigned int flags)
 {
 	unsigned int i = 0;
+	unsigned len = strlen(oid);
 
 	do {
-		if (strcmp(_oid2str[i].oid, oid) == 0 && _oid2str[i].ldap_desc != NULL)
+		if ((_oid2str[i].oid_size == len) && 
+			strcmp(_oid2str[i].oid, oid) == 0 && _oid2str[i].ldap_desc != NULL)
 			return _oid2str[i].ldap_desc;
 		i++;
 	}
@@ -270,10 +296,9 @@ make_printable_string(unsigned etype, const gnutls_datum_t * input,
 	int printable = 0;
 	int ret;
 	unsigned int i;
-	size_t size;
 
 	if (etype == ASN1_ETYPE_BMP_STRING) {
-		ret = _gnutls_ucs2_to_utf8(input->data, input->size, out);
+		ret = _gnutls_ucs2_to_utf8(input->data, input->size, out, 1);
 		if (ret < 0) {
 			/* could not convert. Handle it as non-printable */
 			printable = 0;
@@ -306,25 +331,14 @@ make_printable_string(unsigned etype, const gnutls_datum_t * input,
 		return GNUTLS_E_INVALID_REQUEST;
 
 	if (printable == 0) {	/* need to allocate out */
-		out->size = input->size * 2 + 2;
-		out->data = gnutls_malloc(out->size);
-		if (out->data == NULL)
-			return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
-
-		size = out->size;
-		ret = data2hex(input->data, input->size, out->data, &size);
+		ret = data2hex(input->data, input->size, out);
 		if (ret < 0) {
 			gnutls_assert();
-			goto cleanup;
+			return ret;
 		}
-		out->size = size;
 	}
 
 	return 0;
-
-      cleanup:
-	_gnutls_free_datum(out);
-	return ret;
 }
 
 static int
@@ -336,7 +350,7 @@ decode_complex_string(const struct oid_to_string *oentry, void *value,
 	ASN1_TYPE tmpasn = ASN1_TYPE_EMPTY;
 	char asn1_err[ASN1_MAX_ERROR_DESCRIPTION_SIZE] = "";
 	unsigned int etype;
-	gnutls_datum_t td;
+	gnutls_datum_t td = {NULL, 0};
 
 	if (oentry->asn_desc == NULL) {
 		gnutls_assert();
@@ -351,10 +365,10 @@ decode_complex_string(const struct oid_to_string *oentry, void *value,
 	}
 
 	if ((result =
-	     asn1_der_decoding(&tmpasn, value, value_size,
+	     _asn1_strict_der_decode(&tmpasn, value, value_size,
 			       asn1_err)) != ASN1_SUCCESS) {
 		gnutls_assert();
-		_gnutls_debug_log("asn1_der_decoding: %s\n", asn1_err);
+		_gnutls_debug_log("_asn1_strict_der_decode: %s\n", asn1_err);
 		asn1_delete_structure(&tmpasn);
 		return _gnutls_asn2err(result);
 	}
@@ -399,7 +413,7 @@ decode_complex_string(const struct oid_to_string *oentry, void *value,
 	} else {
 		out->data = td.data;
 		out->size = td.size;
-		out->data[out->size] = 0;
+		/* _gnutls_x509_read_value always null terminates */
 	}
 
 	/* Refuse to deal with strings containing NULs. */
@@ -425,7 +439,6 @@ _gnutls_x509_dn_to_string(const char *oid, void *value,
 	const struct oid_to_string *oentry;
 	int ret;
 	gnutls_datum_t tmp;
-	size_t size;
 
 	if (value == NULL || value_size <= 0) {
 		gnutls_assert();
@@ -434,33 +447,30 @@ _gnutls_x509_dn_to_string(const char *oid, void *value,
 
 	oentry = get_oid_entry(oid);
 	if (oentry == NULL) {	/* unknown OID -> hex */
-		str->size = value_size * 2 + 2;
-		str->data = gnutls_malloc(str->size);
-		if (str->data == NULL)
-			return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
-
-		size = str->size;
-		ret = data2hex(value, value_size, str->data, &size);
+ unknown_oid:
+		ret = data2hex(value, value_size, str);
 		if (ret < 0) {
 			gnutls_assert();
-			gnutls_free(str->data);
 			return ret;
 		}
-		str->size = size;
 		return 0;
 	}
 
 	if (oentry->asn_desc != NULL) {	/* complex */
 		ret =
 		    decode_complex_string(oentry, value, value_size, &tmp);
-		if (ret < 0)
-			return gnutls_assert_val(ret);
+		if (ret < 0) {
+			/* we failed decoding -> handle it as unknown OID */
+			goto unknown_oid;
+		}
 	} else {
 		ret =
 		    _gnutls_x509_decode_string(oentry->etype, value,
-					       value_size, &tmp);
-		if (ret < 0)
-			return gnutls_assert_val(ret);
+					       value_size, &tmp, 0);
+		if (ret < 0) {
+			/* we failed decoding -> handle it as unknown OID */
+			goto unknown_oid;
+		}
 	}
 
 	ret = str_escape(&tmp, str);
@@ -477,39 +487,35 @@ _gnutls_x509_dn_to_string(const char *oid, void *value,
  */
 static int
 data2hex(const void *data, size_t data_size,
-	 void *_out, size_t * sizeof_out)
+	 gnutls_datum_t *out)
 {
-	char *res;
-	char escaped[MAX_STRING_LEN];
-	unsigned int size, res_size;
-	char *out = _out;
+	gnutls_datum_t tmp, td;
+	int ret;
+	size_t size;
 
-	if (2 * data_size + 1 > MAX_STRING_LEN) {
+	td.size = hex_str_size(data_size) + 1; /* +1 for '#' */
+	td.data = gnutls_malloc(td.size);
+	if (td.data == NULL)
+		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+
+	tmp.data = (void*)data;
+	tmp.size = data_size;
+
+	td.data[0] = '#';
+	size = td.size-1; /* don't include '#' */
+	ret =
+	    gnutls_hex_encode(&tmp,
+			    (char*)&td.data[1], &size);
+	if (ret < 0) {
 		gnutls_assert();
-		return GNUTLS_E_INTERNAL_ERROR;
-	}
-
-	res =
-	    _gnutls_bin2hex(data, data_size, escaped, sizeof(escaped),
-			    NULL);
-	if (!res) {
-		gnutls_assert();
-		return GNUTLS_E_INTERNAL_ERROR;
-	}
-
-	res_size = strlen(res);
-	size = res_size + 1;	/* +1 for the '#' */
-	if (size + 1 > *sizeof_out) {
-		*sizeof_out = size + 1;
+		gnutls_free(td.data);
 		return GNUTLS_E_SHORT_MEMORY_BUFFER;
 	}
-	*sizeof_out = size;	/* -1 for the null +1 for the '#' */
 
-	if (out) {
-		out[0] = '#';
-		memcpy(&out[1], res, res_size);
-		out[size] = 0;
-	}
+	td.size--; /* don't include null */
+
+	out->data = td.data;
+	out->size = td.size;
 
 	return 0;
 }
@@ -520,7 +526,7 @@ data2hex(const void *data, size_t data_size,
  *
  */
 
-/* This is an emulations of the struct tm.
+/* This is an emulation of the struct tm.
  * Since we do not use libc's functions, we don't need to
  * depend on the libc structure.
  */
@@ -703,8 +709,48 @@ time_t _gnutls_x509_generalTime2gtime(const char *ttime)
 	return time2gtime(ttime, year);
 }
 
+/* tag will contain ASN1_TAG_UTCTime or ASN1_TAG_GENERALIZEDTime */
 static int
-gtime2generalTime(time_t gtime, char *str_time, size_t str_time_size)
+gtime_to_suitable_time(time_t gtime, char *str_time, size_t str_time_size, unsigned *tag)
+{
+	size_t ret;
+	struct tm _tm;
+
+	if (gtime == (time_t)-1
+#if SIZEOF_LONG == 8
+		|| gtime >= 253402210800
+#endif
+	 ) {
+        	if (tag)
+        		*tag = ASN1_TAG_GENERALIZEDTime;
+        	snprintf(str_time, str_time_size, "99991231235959Z");
+        	return 0;
+	}
+
+	if (!gmtime_r(&gtime, &_tm)) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
+
+	if (_tm.tm_year >= 150) {
+		if (tag)
+        		*tag = ASN1_TAG_GENERALIZEDTime;
+		ret = strftime(str_time, str_time_size, "%Y%m%d%H%M%SZ", &_tm);
+	} else {
+		if (tag)
+        		*tag = ASN1_TAG_UTCTime;
+		ret = strftime(str_time, str_time_size, "%y%m%d%H%M%SZ", &_tm);
+	}
+	if (!ret) {
+		gnutls_assert();
+		return GNUTLS_E_SHORT_MEMORY_BUFFER;
+	}
+
+	return 0;
+}
+
+static int
+gtime_to_generalTime(time_t gtime, char *str_time, size_t str_time_size)
 {
 	size_t ret;
 	struct tm _tm;
@@ -729,7 +775,6 @@ gtime2generalTime(time_t gtime, char *str_time, size_t str_time_size)
 		return GNUTLS_E_SHORT_MEMORY_BUFFER;
 	}
 
-
 	return 0;
 }
 
@@ -738,7 +783,7 @@ gtime2generalTime(time_t gtime, char *str_time, size_t str_time_size)
  * be something like "tbsCertList.thisUpdate".
  */
 #define MAX_TIME 64
-time_t _gnutls_x509_get_time(ASN1_TYPE c2, const char *when, int nochoice)
+time_t _gnutls_x509_get_time(ASN1_TYPE c2, const char *where, int force_general)
 {
 	char ttime[MAX_TIME];
 	char name[128];
@@ -746,28 +791,35 @@ time_t _gnutls_x509_get_time(ASN1_TYPE c2, const char *when, int nochoice)
 	int len, result;
 
 	len = sizeof(ttime) - 1;
-	result = asn1_read_value(c2, when, ttime, &len);
+	result = asn1_read_value(c2, where, ttime, &len);
 	if (result != ASN1_SUCCESS) {
 		gnutls_assert();
 		return (time_t) (-1);
 	}
 
-	if (nochoice != 0) {
+	if (force_general != 0) {
 		c_time = _gnutls_x509_generalTime2gtime(ttime);
 	} else {
-		_gnutls_str_cpy(name, sizeof(name), when);
+		_gnutls_str_cpy(name, sizeof(name), where);
 
 		/* choice */
 		if (strcmp(ttime, "generalTime") == 0) {
-			_gnutls_str_cat(name, sizeof(name),
-					".generalTime");
+			if (name[0] == 0)
+				_gnutls_str_cpy(name, sizeof(name),
+						"generalTime");
+			else
+				_gnutls_str_cat(name, sizeof(name),
+						".generalTime");
 			len = sizeof(ttime) - 1;
 			result = asn1_read_value(c2, name, ttime, &len);
 			if (result == ASN1_SUCCESS)
 				c_time =
 				    _gnutls_x509_generalTime2gtime(ttime);
 		} else {	/* UTCTIME */
-			_gnutls_str_cat(name, sizeof(name), ".utcTime");
+			if (name[0] == 0)
+				_gnutls_str_cpy(name, sizeof(name), "utcTime");
+			else
+				_gnutls_str_cat(name, sizeof(name), ".utcTime");
 			len = sizeof(ttime) - 1;
 			result = asn1_read_value(c2, name, ttime, &len);
 			if (result == ASN1_SUCCESS)
@@ -791,15 +843,16 @@ time_t _gnutls_x509_get_time(ASN1_TYPE c2, const char *when, int nochoice)
  */
 int
 _gnutls_x509_set_time(ASN1_TYPE c2, const char *where, time_t tim,
-		      int nochoice)
+		      int force_general)
 {
 	char str_time[MAX_TIME];
 	char name[128];
 	int result, len;
+	unsigned tag;
 
-	if (nochoice != 0) {
+	if (force_general != 0) {
 		result =
-		    gtime2generalTime(tim, str_time, sizeof(str_time));
+		    gtime_to_generalTime(tim, str_time, sizeof(str_time));
 		if (result < 0)
 			return gnutls_assert_val(result);
 		len = strlen(str_time);
@@ -810,20 +863,26 @@ _gnutls_x509_set_time(ASN1_TYPE c2, const char *where, time_t tim,
 		return 0;
 	}
 
-	_gnutls_str_cpy(name, sizeof(name), where);
-
-	if ((result = asn1_write_value(c2, name, "generalTime", 1)) < 0) {
-		gnutls_assert();
-		return _gnutls_asn2err(result);
-	}
-
-	result = gtime2generalTime(tim, str_time, sizeof(str_time));
+	result = gtime_to_suitable_time(tim, str_time, sizeof(str_time), &tag);
 	if (result < 0) {
 		gnutls_assert();
 		return result;
 	}
 
-	_gnutls_str_cat(name, sizeof(name), ".generalTime");
+	_gnutls_str_cpy(name, sizeof(name), where);
+	if (tag == ASN1_TAG_UTCTime) {
+		if ((result = asn1_write_value(c2, where, "utcTime", 1)) < 0) {
+			gnutls_assert();
+			return _gnutls_asn2err(result);
+		}
+		_gnutls_str_cat(name, sizeof(name), ".utcTime");
+	} else {
+		if ((result = asn1_write_value(c2, where, "generalTime", 1)) < 0) {
+			gnutls_assert();
+			return _gnutls_asn2err(result);
+		}
+		_gnutls_str_cat(name, sizeof(name), ".generalTime");
+	}
 
 	len = strlen(str_time);
 	result = asn1_write_value(c2, name, str_time, len);
@@ -835,6 +894,38 @@ _gnutls_x509_set_time(ASN1_TYPE c2, const char *where, time_t tim,
 	return 0;
 }
 
+/* This will set a DER encoded Time element. To be used in fields
+ * which are of the ANY.
+ */
+int
+_gnutls_x509_set_raw_time(ASN1_TYPE c2, const char *where, time_t tim)
+{
+	char str_time[MAX_TIME];
+	uint8_t buf[128];
+	int result, len, der_len;
+	unsigned tag;
+
+	result =
+	    gtime_to_suitable_time(tim, str_time, sizeof(str_time), &tag);
+	if (result < 0)
+		return gnutls_assert_val(result);
+	len = strlen(str_time);
+
+	buf[0] = tag;
+	asn1_length_der(len, buf+1, &der_len);
+
+	if ((unsigned)len > sizeof(buf)-der_len-1) {
+		return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+	}
+
+	memcpy(buf+1+der_len, str_time, len);
+
+	result = asn1_write_value(c2, where, buf, len+1+der_len);
+	if (result != ASN1_SUCCESS)
+		return gnutls_assert_val(_gnutls_asn2err(result));
+	return 0;
+}
+
 
 gnutls_x509_subject_alt_name_t _gnutls_x509_san_find_type(char *str_type)
 {
@@ -864,7 +955,7 @@ _gnutls_x509_export_int_named(ASN1_TYPE asn1_data, const char *name,
 			      size_t * output_data_size)
 {
 	int ret;
-	gnutls_datum_t out;
+	gnutls_datum_t out = {NULL,0};
 	size_t size;
 
 	ret = _gnutls_x509_export_int_named2(asn1_data, name,
@@ -939,15 +1030,19 @@ _gnutls_x509_export_int_named2(ASN1_TYPE asn1_data, const char *name,
 int
 _gnutls_x509_decode_string(unsigned int etype,
 			   const uint8_t * der, size_t der_size,
-			   gnutls_datum_t * output)
+			   gnutls_datum_t * output, unsigned allow_ber)
 {
 	int ret;
-	const uint8_t *str;
+	uint8_t *str;
 	unsigned int str_size, len;
 	gnutls_datum_t td;
 
-	ret =
-	    asn1_decode_simple_der(etype, der, der_size, &str, &str_size);
+	if (allow_ber)
+		ret =
+		    asn1_decode_simple_ber(etype, der, der_size, &str, &str_size, NULL);
+	else
+		ret =
+		    asn1_decode_simple_der(etype, der, der_size, (const uint8_t**)&str, &str_size);
 	if (ret != ASN1_SUCCESS) {
 		gnutls_assert();
 		ret = _gnutls_asn2err(ret);
@@ -962,6 +1057,9 @@ _gnutls_x509_decode_string(unsigned int etype,
 	memcpy(td.data, str, str_size);
 	td.data[str_size] = 0;
 
+	if (allow_ber)
+		free(str);
+
 	ret = make_printable_string(etype, &td, output);
 	if (ret == GNUTLS_E_INVALID_REQUEST) {	/* unsupported etype */
 		output->data = td.data;
@@ -991,8 +1089,8 @@ _gnutls_x509_decode_string(unsigned int etype,
 /* Reads a value from an ASN1 tree, and puts the output
  * in an allocated variable in the given datum.
  *
- * Note that this function always places allocates one plus
- * the required data size (to allow for a null byte).
+ * Note that this function always allocates one plus
+ * the required data size (and places a null byte).
  */
 int
 _gnutls_x509_read_value(ASN1_TYPE c, const char *root,
@@ -1003,6 +1101,11 @@ _gnutls_x509_read_value(ASN1_TYPE c, const char *root,
 	unsigned int etype;
 
 	result = asn1_read_value_type(c, root, NULL, &len, &etype);
+	if (result == 0 && len == 0) {
+		/* don't allow null strings */
+		return gnutls_assert_val(GNUTLS_E_ASN1_DER_ERROR);
+	}
+
 	if (result != ASN1_MEM_ERROR) {
 		gnutls_assert();
 		result = _gnutls_asn2err(result);
@@ -1010,8 +1113,7 @@ _gnutls_x509_read_value(ASN1_TYPE c, const char *root,
 	}
 
 	if (etype == ASN1_ETYPE_BIT_STRING) {
-		len /= 8;
-		len++;
+		len = (len + 7)/8;
 	}
 
 	tmp = gnutls_malloc((size_t) len + 1);
@@ -1029,12 +1131,12 @@ _gnutls_x509_read_value(ASN1_TYPE c, const char *root,
 	}
 
 	if (etype == ASN1_ETYPE_BIT_STRING) {
-		ret->size = len / 8;
-		if (len % 8 > 0)
-			ret->size++;
-	} else
+		ret->size = (len+7) / 8;
+	} else {
 		ret->size = (unsigned) len;
+	}
 
+	tmp[ret->size] = 0;
 	ret->data = tmp;
 
 	return 0;
@@ -1052,7 +1154,7 @@ _gnutls_x509_read_value(ASN1_TYPE c, const char *root,
  */
 int
 _gnutls_x509_read_string(ASN1_TYPE c, const char *root,
-			 gnutls_datum_t * ret, unsigned int etype)
+			 gnutls_datum_t * ret, unsigned int etype, unsigned int allow_ber)
 {
 	int len = 0, result;
 	size_t slen;
@@ -1090,7 +1192,7 @@ _gnutls_x509_read_string(ASN1_TYPE c, const char *root,
 	 */
 	slen = (size_t) len;
 
-	result = _gnutls_x509_decode_string(etype, tmp, slen, ret);
+	result = _gnutls_x509_decode_string(etype, tmp, slen, ret, allow_ber);
 	if (result < 0) {
 		gnutls_assert();
 		goto cleanup;
@@ -1151,10 +1253,10 @@ _gnutls_x509_der_encode(ASN1_TYPE src, const char *src_name,
 
 	size = 0;
 	result = asn1_der_coding(src, src_name, NULL, &size, NULL);
+	/* this check explicitly covers the case where size == 0 && result == 0 */
 	if (result != ASN1_MEM_ERROR) {
 		gnutls_assert();
-		result = _gnutls_asn2err(result);
-		goto cleanup;
+		return _gnutls_asn2err(result);
 	}
 
 	/* allocate data for the der
@@ -1167,8 +1269,7 @@ _gnutls_x509_der_encode(ASN1_TYPE src, const char *src_name,
 	data = gnutls_malloc((size_t) size);
 	if (data == NULL) {
 		gnutls_assert();
-		result = GNUTLS_E_MEMORY_ERROR;
-		goto cleanup;
+		return GNUTLS_E_MEMORY_ERROR;
 	}
 
 	result = asn1_der_coding(src, src_name, data, &size, NULL);
@@ -1333,7 +1434,7 @@ _gnutls_x509_encode_and_copy_PKI_params(ASN1_TYPE dst,
 	int result;
 	char name[128];
 
-	pk = _gnutls_x509_pk_to_oid(pk_algorithm);
+	pk = gnutls_pk_get_oid(pk_algorithm);
 	if (pk == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_UNKNOWN_PK_ALGORITHM;
@@ -1361,7 +1462,6 @@ _gnutls_x509_encode_and_copy_PKI_params(ASN1_TYPE dst,
 			    ".algorithm.parameters");
 
 	result = asn1_write_value(dst, name, der.data, der.size);
-
 	_gnutls_free_datum(&der);
 
 	if (result != ASN1_SUCCESS) {
@@ -1455,7 +1555,7 @@ _gnutls_x509_get_pk_algorithm(ASN1_TYPE src, const char *src_name,
 		return _gnutls_asn2err(result);
 	}
 
-	algo = _gnutls_x509_oid2pk_algorithm(oid);
+	algo = gnutls_oid_to_pk(oid);
 	if (algo == GNUTLS_PK_UNKNOWN) {
 		_gnutls_debug_log
 		    ("%s: unknown public key algorithm: %s\n", __func__,
@@ -1482,21 +1582,19 @@ _gnutls_x509_get_pk_algorithm(ASN1_TYPE src, const char *src_name,
  * returns them into signed_data.
  */
 int
-_gnutls_x509_get_signed_data(ASN1_TYPE src, const char *src_name,
+_gnutls_x509_get_signed_data(ASN1_TYPE src,  const gnutls_datum *der,
+			     const char *src_name,
 			     gnutls_datum_t * signed_data)
 {
-	gnutls_datum_t der;
 	int start, end, result;
 
-	result = _gnutls_x509_der_encode(src, "", &der, 0);
-	if (result < 0) {
-		gnutls_assert();
-		return result;
+	if (der == NULL || der->size == 0) {
+		return _gnutls_x509_der_encode(src, src_name, signed_data, 0);
 	}
 
 	/* Get the signed data
 	 */
-	result = asn1_der_decoding_startEnd(src, der.data, der.size,
+	result = asn1_der_decoding_startEnd(src, der->data, der->size,
 					    src_name, &start, &end);
 	if (result != ASN1_SUCCESS) {
 		result = _gnutls_asn2err(result);
@@ -1505,7 +1603,7 @@ _gnutls_x509_get_signed_data(ASN1_TYPE src, const char *src_name,
 	}
 
 	result =
-	    _gnutls_set_datum(signed_data, &der.data[start],
+	    _gnutls_set_datum(signed_data, &der->data[start],
 			      end - start + 1);
 
 	if (result < 0) {
@@ -1516,8 +1614,6 @@ _gnutls_x509_get_signed_data(ASN1_TYPE src, const char *src_name,
 	result = 0;
 
       cleanup:
-	_gnutls_free_datum(&der);
-
 	return result;
 }
 
@@ -1549,7 +1645,7 @@ _gnutls_x509_get_signature_algorithm(ASN1_TYPE src, const char *src_name)
 		return result;
 	}
 
-	result = _gnutls_x509_oid2sign_algorithm((char *) sa.data);
+	result = gnutls_oid_to_sign((char *) sa.data);
 
 	_gnutls_free_datum(&sa);
 
@@ -1565,7 +1661,7 @@ _gnutls_x509_get_signature(ASN1_TYPE src, const char *src_name,
 			   gnutls_datum_t * signature)
 {
 	int result, len;
-	unsigned int bits;
+	int bits;
 
 	signature->data = NULL;
 	signature->size = 0;
@@ -1582,7 +1678,7 @@ _gnutls_x509_get_signature(ASN1_TYPE src, const char *src_name,
 	}
 
 	bits = len;
-	if (bits % 8 != 0) {
+	if (bits % 8 != 0 || bits < 8) {
 		gnutls_assert();
 		result = GNUTLS_E_CERTIFICATE_ERROR;
 		goto cleanup;
@@ -1601,7 +1697,7 @@ _gnutls_x509_get_signature(ASN1_TYPE src, const char *src_name,
 	 */
 	bits = len;
 	result =
-	    asn1_read_value(src, src_name, signature->data, (int *) &bits);
+	    asn1_read_value(src, src_name, signature->data, &bits);
 
 	if (result != ASN1_SUCCESS) {
 		result = _gnutls_asn2err(result);
@@ -1805,7 +1901,7 @@ int _gnutls_strdatum_to_buf(gnutls_datum_t * d, void *buf,
 }
 
 int
-_gnutls_x509_get_raw_dn2(ASN1_TYPE c2, gnutls_datum_t * raw,
+_gnutls_x509_get_raw_field2(ASN1_TYPE c2, gnutls_datum_t * raw,
 			 const char *whom, gnutls_datum_t * dn)
 {
 	int result, len1;
@@ -1822,14 +1918,246 @@ _gnutls_x509_get_raw_dn2(ASN1_TYPE c2, gnutls_datum_t * raw,
 
 	len1 = end1 - start1 + 1;
 
-	result = _gnutls_set_datum(dn, &raw->data[start1], len1);
-	if (result < 0) {
-		gnutls_assert();
-		goto cleanup;
-	}
-
+	dn->data = &raw->data[start1];
+	dn->size = len1;
 	result = 0;
 
       cleanup:
 	return result;
 }
+
+int _gnutls_copy_string(gnutls_datum_t* str, uint8_t *out, size_t *out_size)
+{
+unsigned size_to_check;
+
+	size_to_check = str->size + 1;
+
+	if ((unsigned) size_to_check > *out_size) {
+		gnutls_assert();
+		(*out_size) = size_to_check;
+		return GNUTLS_E_SHORT_MEMORY_BUFFER;
+	}
+
+	if (out != NULL && str->data != NULL) {
+		memcpy(out, str->data, str->size);
+		out[str->size] = 0;
+	} else if (out != NULL) {
+		out[0] = 0;
+	}
+	*out_size = str->size;
+
+	return 0;
+}
+
+int _gnutls_copy_data(gnutls_datum_t* str, uint8_t *out, size_t *out_size)
+{
+	if ((unsigned) str->size > *out_size) {
+		gnutls_assert();
+		(*out_size) = str->size;
+		return GNUTLS_E_SHORT_MEMORY_BUFFER;
+	}
+
+	if (out != NULL && str->data != NULL) {
+		memcpy(out, str->data, str->size);
+	}
+	*out_size = str->size;
+
+	return 0;
+}
+
+/* Converts an X.509 certificate to subjectPublicKeyInfo */
+int x509_crt_to_raw_pubkey(gnutls_x509_crt_t crt,
+			   gnutls_datum_t * rpubkey)
+{
+	gnutls_pubkey_t pubkey = NULL;
+	int ret;
+
+	ret = gnutls_pubkey_init(&pubkey);
+	if (ret < 0)
+		return gnutls_assert_val(ret);
+
+	ret = gnutls_pubkey_import_x509(pubkey, crt, 0);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
+	ret =
+	    gnutls_pubkey_export2(pubkey, GNUTLS_X509_FMT_DER, rpubkey);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
+	ret = 0;
+
+      cleanup:
+	gnutls_pubkey_deinit(pubkey);
+	return ret;
+}
+
+/* Converts an X.509 certificate to subjectPublicKeyInfo */
+int x509_raw_crt_to_raw_pubkey(const gnutls_datum_t * cert,
+			   gnutls_datum_t * rpubkey)
+{
+	gnutls_x509_crt_t crt = NULL;
+	int ret;
+
+	ret = gnutls_x509_crt_init(&crt);
+	if (ret < 0)
+		return gnutls_assert_val(ret);
+
+	ret = gnutls_x509_crt_import(crt, cert, GNUTLS_X509_FMT_DER);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
+	ret = x509_crt_to_raw_pubkey(crt, rpubkey);
+ cleanup:
+	gnutls_x509_crt_deinit(crt);
+
+	return ret;
+}
+
+bool
+_gnutls_check_valid_key_id(gnutls_datum_t *key_id,
+                           gnutls_x509_crt_t cert, time_t now)
+{
+	uint8_t id[MAX_KEY_ID_SIZE];
+	size_t id_size;
+	bool result = 0;
+
+	if (now > gnutls_x509_crt_get_expiration_time(cert) ||
+	    now < gnutls_x509_crt_get_activation_time(cert)) {
+		/* don't bother, certificate is not yet activated or expired */
+		gnutls_assert();
+		goto out;
+	}
+
+	id_size = sizeof(id);
+	if (gnutls_x509_crt_get_subject_key_id(cert, id, &id_size, NULL) < 0) {
+		gnutls_assert();
+		goto out;
+	}
+
+	if (id_size == key_id->size && !memcmp(id, key_id->data, id_size))
+		result = 1;
+
+ out:
+	return result;
+}
+
+/* Takes a certificate list and orders it with subject, issuer order.
+ *
+ * *clist_size contains the size of the ordered list (which is always less or
+ * equal to the original).
+ * @func: the function to call to elements outside the sort.
+ *
+ * Returns the sorted list which may be the original clist.
+ */
+gnutls_x509_crt_t *_gnutls_sort_clist(gnutls_x509_crt_t
+				     sorted[DEFAULT_MAX_VERIFY_DEPTH],
+				     gnutls_x509_crt_t *clist,
+				     unsigned int *clist_size,
+				     gnutls_cert_vfunc func)
+{
+	int prev;
+	unsigned int j, i;
+	int issuer[DEFAULT_MAX_VERIFY_DEPTH];	/* contain the index of the issuers */
+	unsigned insorted[DEFAULT_MAX_VERIFY_DEPTH]; /* non zero if clist[i] used in sorted list */
+	unsigned orig_size = *clist_size;
+
+	/* Do not bother sorting if too many certificates are given.
+	 * Prevent any DoS attacks.
+	 */
+	if (*clist_size > DEFAULT_MAX_VERIFY_DEPTH)
+		return clist;
+
+	for (i = 0; i < DEFAULT_MAX_VERIFY_DEPTH; i++) {
+		issuer[i] = -1;
+		insorted[i] = 0;
+	}
+
+	/* Find the issuer of each certificate and store it
+	 * in issuer array.
+	 */
+	for (i = 0; i < *clist_size; i++) {
+		for (j = 1; j < *clist_size; j++) {
+			if (i == j)
+				continue;
+
+			if (gnutls_x509_crt_check_issuer(clist[i],
+							 clist[j]) != 0) {
+				issuer[i] = j;
+				break;
+			}
+		}
+	}
+
+	/* always included */
+	sorted[0] = clist[0];
+	insorted[0] = 1;
+
+	if (issuer[0] == -1) {
+		*clist_size = 1;
+		goto exit;
+	}
+
+	prev = 0;
+	for (i = 1; i < *clist_size; i++) {
+		prev = issuer[prev];
+		if (prev < 0) {	/* no issuer */
+			*clist_size = i;
+			break;
+		}
+		sorted[i] = clist[prev];
+		insorted[prev] = 1;
+	}
+
+ exit:
+	if (func) {
+		/* call func() on all the elements that were
+		 * not used in the sorted list */
+		for (i = 1; i < orig_size; i++) {
+			if (insorted[i] == 0) {
+				func(clist[i]);
+			}
+		}
+	}
+
+	return sorted;
+}
+
+int _gnutls_check_if_sorted(gnutls_x509_crt_t * crt, int nr)
+{
+	void *prev_dn = NULL;
+	void *dn;
+	size_t prev_dn_size = 0, dn_size;
+	int i, ret;
+
+	/* check if the X.509 list is ordered */
+	if (nr > 1) {
+		for (i = 0; i < nr; i++) {
+			if (i > 0) {
+				dn = crt[i]->raw_dn.data;
+				dn_size = crt[i]->raw_dn.size;
+
+				if (dn_size != prev_dn_size
+				    || memcmp(dn, prev_dn, dn_size) != 0) {
+					ret =
+					    gnutls_assert_val
+					    (GNUTLS_E_CERTIFICATE_LIST_UNSORTED);
+					goto cleanup;
+				}
+			}
+
+			prev_dn = crt[i]->raw_issuer_dn.data;
+			prev_dn_size = crt[i]->raw_issuer_dn.size;
+		}
+	}
+	ret = 0;
+
+cleanup:
+	return ret;
+}
