@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2003-2016 Free Software Foundation, Inc.
- * Copyright (C) 2015-2016 Red Hat, Inc.
+ * Copyright (C) 2003-2012 Free Software Foundation, Inc.
  * Copyright (C) 2002 Andrew McDonald
  *
  * This file is part of GnuTLS.
@@ -16,18 +15,17 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
-#include "gnutls_int.h"
-#include "str.h"
-#include "x509_int.h"
-#include "common.h"
-#include "errors.h"
-#include "system.h"
-#include <netinet/in.h>
-#include <arpa/inet.h>
+#include <gnutls_int.h>
+#include <gnutls_str.h>
+#include <x509_int.h>
+#include <common.h>
+#include <gnutls_errors.h>
+#include <system.h>
+#include <gnutls-idna.h>
 
 /**
  * gnutls_x509_crt_check_hostname:
@@ -43,13 +41,15 @@
  *
  * Returns: non-zero for a successful match, and zero on failure.
  **/
-unsigned gnutls_x509_crt_check_hostname(gnutls_x509_crt_t cert,
-					const char *hostname)
+int
+gnutls_x509_crt_check_hostname(gnutls_x509_crt_t cert,
+			       const char *hostname)
 {
 	return gnutls_x509_crt_check_hostname2(cert, hostname, 0);
 }
 
-static int check_ip(gnutls_x509_crt_t cert, const void *ip, unsigned ip_size)
+static int
+check_ip(gnutls_x509_crt_t cert, const void *ip, unsigned ip_size, unsigned flags)
 {
 	char temp[16];
 	size_t temp_size;
@@ -66,12 +66,13 @@ static int check_ip(gnutls_x509_crt_t cert, const void *ip, unsigned ip_size)
 	 */
 	for (i = 0; !(ret < 0); i++) {
 		temp_size = sizeof(temp);
-		ret = gnutls_x509_crt_get_subject_alt_name(cert, i, temp,
-							   &temp_size, NULL);
+		ret = gnutls_x509_crt_get_subject_alt_name(cert, i,
+							   temp,
+							   &temp_size,
+							   NULL);
 
 		if (ret == GNUTLS_SAN_IPADDRESS) {
-			if (temp_size == ip_size &&
-			    memcmp(temp, ip, ip_size) == 0)
+			if (temp_size == ip_size && memcmp(temp, ip, ip_size) == 0)
 				return 1;
 		} else if (ret == GNUTLS_E_SHORT_MEMORY_BUFFER) {
 			ret = 0;
@@ -83,36 +84,15 @@ static int check_ip(gnutls_x509_crt_t cert, const void *ip, unsigned ip_size)
 	return 0;
 }
 
-/**
- * gnutls_x509_crt_check_ip:
- * @cert: should contain an gnutls_x509_crt_t type
- * @ip: A pointer to the raw IP address
- * @ip_size: the number of bytes in ip (4 or 16)
- * @flags: should be zero
- *
- * This function will check if the IP allowed IP addresses in 
- * the certificate's subject alternative name match the provided
- * IP address.
- *
- * Returns: non-zero for a successful match, and zero on failure.
- **/
-unsigned gnutls_x509_crt_check_ip(gnutls_x509_crt_t cert,
-				  const unsigned char *ip, unsigned int ip_size,
-				  unsigned int flags)
+static int has_embedded_null(const char *str, unsigned size)
 {
-	return check_ip(cert, ip, ip_size);
+	if (strlen(str) != size)
+		return 1;
+	return 0;
 }
 
-/* whether gnutls_x509_crt_check_hostname2() will consider these
- * alternative name types. This is to satisfy RFC6125 requirement
- * that we do not fallback to CN-ID if we encounter a supported name
- * type.
- */
-#define IS_SAN_SUPPORTED(san) \
-	(san == GNUTLS_SAN_DNSNAME || san == GNUTLS_SAN_IPADDRESS)
-
 /**
- * gnutls_x509_crt_check_hostname2:
+ * gnutls_x509_crt_check_hostname:
  * @cert: should contain an gnutls_x509_crt_t type
  * @hostname: A null terminated string that contains a DNS name
  * @flags: gnutls_certificate_verify_flags
@@ -125,45 +105,34 @@ unsigned gnutls_x509_crt_check_ip(gnutls_x509_crt_t cert,
  * IPv4 addresses are accepted by this function in the dotted-decimal
  * format (e.g, ddd.ddd.ddd.ddd), and IPv6 addresses in the hexadecimal
  * x:x:x:x:x:x:x:x format. For them the IPAddress subject alternative
- * name extension is consulted. Previous versions to 3.6.0 of GnuTLS
- * in case of a non-match would consult (in a non-standard extension)
- * the DNSname and CN fields. This is no longer the case.
+ * name extension is consulted, as well as the DNSNames in case of a non-match.
+ * The latter fallback exists due to misconfiguration of many servers
+ * which place an IPAddress inside the DNSName extension.
  *
  * When the flag %GNUTLS_VERIFY_DO_NOT_ALLOW_WILDCARDS is specified no
  * wildcards are considered. Otherwise they are only considered if the
  * domain name consists of three components or more, and the wildcard
  * starts at the leftmost position.
-
- * When the flag %GNUTLS_VERIFY_DO_NOT_ALLOW_IP_MATCHES is specified,
- * the input will be treated as a DNS name, and matching of textual IP addresses
- * against the IPAddress part of the alternative name will not be allowed.
- *
- * The function gnutls_x509_crt_check_ip() is available for matching
- * IP addresses.
  *
  * Returns: non-zero for a successful match, and zero on failure.
- *
- * Since: 3.3.0
  **/
-unsigned gnutls_x509_crt_check_hostname2(gnutls_x509_crt_t cert,
-					 const char *hostname,
-					 unsigned int flags)
+int
+gnutls_x509_crt_check_hostname2(gnutls_x509_crt_t cert,
+			        const char *hostname, unsigned int flags)
 {
 	char dnsname[MAX_CN];
 	size_t dnsnamesize;
 	int found_dnsname = 0;
-	int ret = 0;
+	int ret = 0, rc;
 	int i = 0;
 	struct in_addr ipv4;
 	char *p = NULL;
 	char *a_hostname;
-	unsigned have_other_addresses = 0;
-	gnutls_datum_t out;
+	char *a_dnsname;
 
 	/* check whether @hostname is an ip address */
-	if (!(flags & GNUTLS_VERIFY_DO_NOT_ALLOW_IP_MATCHES) &&
-	    ((p = strchr(hostname, ':')) != NULL ||
-	     inet_pton(AF_INET, hostname, &ipv4) != 0)) {
+	if ((p=strchr(hostname, ':')) != NULL || inet_aton(hostname, &ipv4) != 0) {
+
 		if (p != NULL) {
 			struct in6_addr ipv6;
 
@@ -172,26 +141,25 @@ unsigned gnutls_x509_crt_check_hostname2(gnutls_x509_crt_t cert,
 				gnutls_assert();
 				goto hostname_fallback;
 			}
-			ret = check_ip(cert, &ipv6, 16);
+			ret = check_ip(cert, &ipv6, 16, flags);
 		} else {
-			ret = check_ip(cert, &ipv4, 4);
+			ret = check_ip(cert, &ipv4, 4, flags);
 		}
 
-		/* Prior to 3.6.0 we were accepting misconfigured servers, that place their IP
-		 * in the DNS field of subjectAlternativeName. That is no longer the case. */
-		return ret;
+		if (ret != 0)
+			return ret;
+
+		/* There are several misconfigured servers, that place their IP
+		 * in the DNS field of subjectAlternativeName. Don't break these
+		 * configurations and verify the IP as it would have been a DNS name. */
 	}
 
-hostname_fallback:
+ hostname_fallback:
 	/* convert the provided hostname to ACE-Labels domain. */
-	ret = gnutls_idna_map(hostname, strlen(hostname), &out, 0);
-	if (ret < 0) {
-		_gnutls_debug_log(
-			"unable to convert hostname %s to IDNA format\n",
-			hostname);
-		a_hostname = (char *)hostname;
-	} else {
-		a_hostname = (char *)out.data;
+	rc = idna_to_ascii_8z (hostname, &a_hostname, 0);
+	if (rc != IDNA_SUCCESS) {
+		_gnutls_debug_log("unable to convert hostname %s to IDNA format: %s\n", hostname, idna_strerror (rc));
+		a_hostname = (char*)hostname;
 	}
 
 	/* try matching against:
@@ -209,41 +177,38 @@ hostname_fallback:
 	 * against all those of type dNSName.
 	 */
 	for (i = 0; !(ret < 0); i++) {
+
 		dnsnamesize = sizeof(dnsname);
-		ret = gnutls_x509_crt_get_subject_alt_name(cert, i, dnsname,
-							   &dnsnamesize, NULL);
+		ret = gnutls_x509_crt_get_subject_alt_name(cert, i,
+							   dnsname,
+							   &dnsnamesize,
+							   NULL);
 
 		if (ret == GNUTLS_SAN_DNSNAME) {
 			found_dnsname = 1;
 
-			if (memchr(dnsname, '\0', dnsnamesize)) {
-				_gnutls_debug_log(
-					"certificate has %s with embedded null in name\n",
-					dnsname);
+			if (has_embedded_null(dnsname, dnsnamesize)) {
+				_gnutls_debug_log("certificate has %s with embedded null in name\n", dnsname);
 				continue;
 			}
 
-			if (!_gnutls_str_is_print(dnsname, dnsnamesize)) {
-				_gnutls_debug_log(
-					"invalid (non-ASCII) name in certificate %.*s\n",
-					(int)dnsnamesize, dnsname);
+			rc = idna_to_ascii_8z (dnsname, &a_dnsname, 0);
+			if (rc != IDNA_SUCCESS) {
+				_gnutls_debug_log("unable to convert dnsname %s to IDNA format: %s\n", dnsname, idna_strerror (rc));
 				continue;
 			}
 
-			ret = _gnutls_hostname_compare(dnsname, dnsnamesize,
-						       a_hostname, flags);
+			ret = _gnutls_hostname_compare(a_dnsname, strlen(a_dnsname), a_hostname, flags);
+			idn_free(a_dnsname);
+
 			if (ret != 0) {
 				ret = 1;
 				goto cleanup;
 			}
-		} else {
-			if (IS_SAN_SUPPORTED(ret))
-				have_other_addresses = 1;
 		}
 	}
 
-	if (!have_other_addresses && !found_dnsname &&
-	    _gnutls_check_key_purpose(cert, GNUTLS_KP_TLS_WWW_SERVER, 0) != 0) {
+	if (!found_dnsname && _gnutls_check_key_purpose(cert, GNUTLS_KP_TLS_WWW_SERVER, 0) != 0) {
 		/* did not get the necessary extension, use CN instead, if the
 		 * certificate would have been acceptable for a TLS WWW server purpose.
 		 * That is because only for that purpose the CN is a valid field to
@@ -253,41 +218,40 @@ hostname_fallback:
 		/* enforce the RFC6125 (§1.8) requirement that only
 		 * a single CN must be present */
 		dnsnamesize = sizeof(dnsname);
-		ret = gnutls_x509_crt_get_dn_by_oid(cert, OID_X520_COMMON_NAME,
-						    1, 0, dnsname,
-						    &dnsnamesize);
+		ret = gnutls_x509_crt_get_dn_by_oid
+			(cert, OID_X520_COMMON_NAME, 1, 0, dnsname,
+			 &dnsnamesize);
 		if (ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
 			ret = 0;
 			goto cleanup;
 		}
 
 		dnsnamesize = sizeof(dnsname);
-		ret = gnutls_x509_crt_get_dn_by_oid(cert, OID_X520_COMMON_NAME,
-						    0, 0, dnsname,
-						    &dnsnamesize);
+		ret = gnutls_x509_crt_get_dn_by_oid
+			(cert, OID_X520_COMMON_NAME, 0, 0, dnsname,
+			 &dnsnamesize);
 		if (ret < 0) {
 			ret = 0;
 			goto cleanup;
 		}
 
-		if (memchr(dnsname, '\0', dnsnamesize)) {
-			_gnutls_debug_log(
-				"certificate has CN %s with embedded null in name\n",
-				dnsname);
+		if (has_embedded_null(dnsname, dnsnamesize)) {
+			_gnutls_debug_log("certificate has CN %s with embedded null in name\n", dnsname);
 			ret = 0;
 			goto cleanup;
 		}
 
-		if (!_gnutls_str_is_print(dnsname, dnsnamesize)) {
-			_gnutls_debug_log(
-				"invalid (non-ASCII) name in certificate CN %.*s\n",
-				(int)dnsnamesize, dnsname);
+		rc = idna_to_ascii_8z (dnsname, &a_dnsname, 0);
+		if (rc != IDNA_SUCCESS) {
+			_gnutls_debug_log("unable to convert CN %s to IDNA format: %s\n", dnsname, idna_strerror (rc));
 			ret = 0;
 			goto cleanup;
 		}
 
-		ret = _gnutls_hostname_compare(dnsname, dnsnamesize, a_hostname,
-					       flags);
+		ret = _gnutls_hostname_compare(a_dnsname, strlen(a_dnsname), a_hostname, flags);
+
+		idn_free(a_dnsname);
+
 		if (ret != 0) {
 			ret = 1;
 			goto cleanup;
@@ -297,9 +261,9 @@ hostname_fallback:
 	/* not found a matching name
 	 */
 	ret = 0;
-cleanup:
-	if (a_hostname != hostname) {
-		gnutls_free(a_hostname);
+ cleanup:
+ 	if (a_hostname != hostname) {
+ 		idn_free(a_hostname);
 	}
-	return ret;
+ 	return ret;
 }
