@@ -15,19 +15,19 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
 /* Online Certificate Status Protocol - RFC 2560
  */
 
-#include "gnutls_int.h"
-#include "global.h"
-#include "errors.h"
+#include <gnutls_int.h>
+#include <gnutls_global.h>
+#include <gnutls_errors.h>
 #include <libtasn1.h>
-#include "pk.h"
-#include "str.h"
+#include <gnutls_pk.h>
+#include <gnutls_str.h>
 #include "algorithms.h"
 
 #include <gnutls/ocsp.h>
@@ -35,7 +35,7 @@
 #define addf _gnutls_buffer_append_printf
 #define adds _gnutls_buffer_append_str
 
-static void print_req(gnutls_buffer_st *str, gnutls_ocsp_req_const_t req)
+static void print_req(gnutls_buffer_st * str, gnutls_ocsp_req_t req)
 {
 	int ret;
 	unsigned indx;
@@ -58,8 +58,9 @@ static void print_req(gnutls_buffer_st *str, gnutls_ocsp_req_const_t req)
 		gnutls_digest_algorithm_t digest;
 		gnutls_datum_t in, ik, sn;
 
-		ret = gnutls_ocsp_req_get_cert_id(req, indx, &digest, &in, &ik,
-						  &sn);
+		ret =
+		    gnutls_ocsp_req_get_cert_id(req, indx, &digest, &in,
+						&ik, &sn);
 		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
 			break;
 		addf(str, "\t\tCertificate ID:\n");
@@ -95,8 +96,9 @@ static void print_req(gnutls_buffer_st *str, gnutls_ocsp_req_const_t req)
 		unsigned int critical;
 		gnutls_datum_t data;
 
-		ret = gnutls_ocsp_req_get_extension(req, indx, &oid, &critical,
-						    &data);
+		ret =
+		    gnutls_ocsp_req_get_extension(req, indx, &oid,
+						  &critical, &data);
 		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
 			break;
 		else if (ret != GNUTLS_E_SUCCESS) {
@@ -107,12 +109,14 @@ static void print_req(gnutls_buffer_st *str, gnutls_ocsp_req_const_t req)
 		if (indx == 0)
 			adds(str, "\tExtensions:\n");
 
-		if (oid.size == sizeof(GNUTLS_OCSP_NONCE) - 1 &&
+		if (oid.size == sizeof(GNUTLS_OCSP_NONCE) &&
 		    memcmp(oid.data, GNUTLS_OCSP_NONCE, oid.size) == 0) {
 			gnutls_datum_t nonce;
 			unsigned int ncrit;
 
-			ret = gnutls_ocsp_req_get_nonce(req, &ncrit, &nonce);
+			ret =
+			    gnutls_ocsp_req_get_nonce(req, &ncrit,
+						      &nonce);
 			if (ret != GNUTLS_E_SUCCESS) {
 				addf(str, "error: get_nonce: %s\n",
 				     gnutls_strerror(ret));
@@ -125,16 +129,17 @@ static void print_req(gnutls_buffer_st *str, gnutls_ocsp_req_const_t req)
 				gnutls_free(nonce.data);
 			}
 		} else {
-			addf(str, "\t\tUnknown extension %s (%s):\n", oid.data,
+			addf(str, "\t\tUnknown extension %s (%s):\n",
+			     oid.data,
 			     critical ? "critical" : "not critical");
 
 			adds(str, _("\t\t\tASCII: "));
-			_gnutls_buffer_asciiprint(str, (char *)data.data,
+			_gnutls_buffer_asciiprint(str, (char *) data.data,
 						  data.size);
 			addf(str, "\n");
 
 			adds(str, _("\t\t\tHexdump: "));
-			_gnutls_buffer_hexprint(str, (char *)data.data,
+			_gnutls_buffer_hexprint(str, (char *) data.data,
 						data.size);
 			adds(str, "\n");
 		}
@@ -163,9 +168,10 @@ static void print_req(gnutls_buffer_st *str, gnutls_ocsp_req_const_t req)
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  **/
-int gnutls_ocsp_req_print(gnutls_ocsp_req_const_t req,
-			  gnutls_ocsp_print_formats_t format,
-			  gnutls_datum_t *out)
+int
+gnutls_ocsp_req_print(gnutls_ocsp_req_t req,
+		      gnutls_ocsp_print_formats_t format,
+		      gnutls_datum_t * out)
 {
 	gnutls_buffer_st str;
 	int rc;
@@ -190,8 +196,9 @@ int gnutls_ocsp_req_print(gnutls_ocsp_req_const_t req,
 	return GNUTLS_E_SUCCESS;
 }
 
-static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
-		       gnutls_ocsp_print_formats_t format)
+static void
+print_resp(gnutls_buffer_st * str, gnutls_ocsp_resp_t resp,
+	   gnutls_ocsp_print_formats_t format)
 {
 	int ret;
 	unsigned indx;
@@ -247,13 +254,13 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 		adds(str, "\tResponse Type: ");
 #define OCSP_BASIC "1.3.6.1.5.5.7.48.1.1"
 
-		if (oid.size == sizeof(OCSP_BASIC) - 1 &&
-		    memcmp(oid.data, OCSP_BASIC, oid.size) == 0) {
+		if (oid.size == sizeof(OCSP_BASIC)
+		    && memcmp(oid.data, OCSP_BASIC, oid.size) == 0) {
 			adds(str, "Basic OCSP Response\n");
 			gnutls_free(oid.data);
 		} else {
-			addf(str, "Unknown response type (%.*s)\n", oid.size,
-			     oid.data);
+			addf(str, "Unknown response type (%.*s)\n",
+			     oid.size, oid.data);
 			gnutls_free(oid.data);
 			return;
 		}
@@ -271,28 +278,29 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 
 	/* responderID */
 	{
-		gnutls_datum_t dn = { NULL, 0 };
+		gnutls_datum_t dn;
 
-		ret = gnutls_ocsp_resp_get_responder2(resp, &dn, 0);
-		if (ret < 0) {
-			if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
-				ret = gnutls_ocsp_resp_get_responder_raw_id(
-					resp, GNUTLS_OCSP_RESP_ID_KEY, &dn);
+		ret = gnutls_ocsp_resp_get_responder(resp, &dn);
+		if (ret < 0 || dn.data == NULL) {
+			if (dn.data == 0) {
+				ret = gnutls_ocsp_resp_get_responder_raw_id(resp, GNUTLS_OCSP_RESP_ID_KEY, &dn);
 
 				if (ret >= 0) {
 					addf(str, _("\tResponder Key ID: "));
-					_gnutls_buffer_hexprint(str, dn.data,
-								dn.size);
+					_gnutls_buffer_hexprint(str, dn.data, dn.size);
 					adds(str, "\n");
 				}
 				gnutls_free(dn.data);
 			} else {
-				addf(str, "error: get_responder2: %s\n",
+				addf(str, "error: get_dn: %s\n",
 				     gnutls_strerror(ret));
 			}
 		} else {
-			addf(str, _("\tResponder ID: %s\n"), dn.data);
-			gnutls_free(dn.data);
+			if (dn.data != NULL) {
+				addf(str, _("\tResponder ID: %.*s\n"), dn.size,
+				     dn.data);
+				gnutls_free(dn.data);
+			}
 		}
 	}
 
@@ -302,14 +310,15 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 		struct tm t;
 		time_t tim = gnutls_ocsp_resp_get_produced(resp);
 
-		if (tim == (time_t)-1)
+		if (tim == (time_t) - 1)
 			addf(str, "error: ocsp_resp_get_produced\n");
 		else if (gmtime_r(&tim, &t) == NULL)
 			addf(str, "error: gmtime_r (%ld)\n",
-			     (unsigned long)tim);
-		else if (strftime(s, max, "%a %b %d %H:%M:%S UTC %Y", &t) == 0)
+			     (unsigned long) tim);
+		else if (strftime(s, max, "%a %b %d %H:%M:%S UTC %Y", &t)
+			 == 0)
 			addf(str, "error: strftime (%ld)\n",
-			     (unsigned long)tim);
+			     (unsigned long) tim);
 		else
 			addf(str, _("\tProduced At: %s\n"), s);
 	}
@@ -324,9 +333,12 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 		time_t revocation_time;
 		unsigned int revocation_reason;
 
-		ret = gnutls_ocsp_resp_get_single(resp, indx, &digest, &in, &ik,
-						  &sn, &cert_status,
-						  &this_update, &next_update,
+		ret = gnutls_ocsp_resp_get_single(resp,
+						  indx,
+						  &digest, &in, &ik, &sn,
+						  &cert_status,
+						  &this_update,
+						  &next_update,
 						  &revocation_time,
 						  &revocation_reason);
 		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
@@ -380,7 +392,8 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 			}
 
 			if (p)
-				addf(str, "\t\tCertificate Status: %s\n", p);
+				addf(str, "\t\tCertificate Status: %s\n",
+				     p);
 		}
 
 		/* XXX revocation reason */
@@ -390,17 +403,19 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 			size_t max = sizeof(s);
 			struct tm t;
 
-			if (revocation_time == (time_t)-1)
+			if (revocation_time == (time_t) - 1)
 				addf(str, "error: revocation_time\n");
 			else if (gmtime_r(&revocation_time, &t) == NULL)
 				addf(str, "error: gmtime_r (%ld)\n",
-				     (unsigned long)revocation_time);
-			else if (strftime(s, max, "%a %b %d %H:%M:%S UTC %Y",
-					  &t) == 0)
+				     (unsigned long) revocation_time);
+			else if (strftime
+				 (s, max, "%a %b %d %H:%M:%S UTC %Y",
+				  &t) == 0)
 				addf(str, "error: strftime (%ld)\n",
-				     (unsigned long)revocation_time);
+				     (unsigned long) revocation_time);
 			else
-				addf(str, _("\t\tRevocation time: %s\n"), s);
+				addf(str, _("\t\tRevocation time: %s\n"),
+				     s);
 		}
 
 		{
@@ -408,15 +423,16 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 			size_t max = sizeof(s);
 			struct tm t;
 
-			if (this_update == (time_t)-1)
+			if (this_update == (time_t) - 1)
 				addf(str, "error: this_update\n");
 			else if (gmtime_r(&this_update, &t) == NULL)
 				addf(str, "error: gmtime_r (%ld)\n",
-				     (unsigned long)this_update);
-			else if (strftime(s, max, "%a %b %d %H:%M:%S UTC %Y",
-					  &t) == 0)
+				     (unsigned long) this_update);
+			else if (strftime
+				 (s, max, "%a %b %d %H:%M:%S UTC %Y",
+				  &t) == 0)
 				addf(str, "error: strftime (%ld)\n",
-				     (unsigned long)this_update);
+				     (unsigned long) this_update);
 			else
 				addf(str, _("\t\tThis Update: %s\n"), s);
 		}
@@ -426,18 +442,17 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 			size_t max = sizeof(s);
 			struct tm t;
 
-			if (next_update != (time_t)-1) {
+			if (next_update != (time_t) - 1) {
 				if (gmtime_r(&next_update, &t) == NULL)
 					addf(str, "error: gmtime_r (%ld)\n",
-					     (unsigned long)next_update);
-				else if (strftime(s, max,
-						  "%a %b %d %H:%M:%S UTC %Y",
-						  &t) == 0)
+					     (unsigned long) next_update);
+				else if (strftime
+					 (s, max, "%a %b %d %H:%M:%S UTC %Y",
+					  &t) == 0)
 					addf(str, "error: strftime (%ld)\n",
-					     (unsigned long)next_update);
+					     (unsigned long) next_update);
 				else
-					addf(str, _("\t\tNext Update: %s\n"),
-					     s);
+					addf(str, _("\t\tNext Update: %s\n"), s);
 			}
 		}
 
@@ -450,8 +465,9 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 		unsigned int critical;
 		gnutls_datum_t data;
 
-		ret = gnutls_ocsp_resp_get_extension(resp, indx, &oid,
-						     &critical, &data);
+		ret =
+		    gnutls_ocsp_resp_get_extension(resp, indx, &oid,
+						   &critical, &data);
 		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
 			break;
 		else if (ret != GNUTLS_E_SUCCESS) {
@@ -460,12 +476,14 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 			continue;
 		}
 
-		if (oid.size == sizeof(GNUTLS_OCSP_NONCE) - 1 &&
+		if (oid.size == sizeof(GNUTLS_OCSP_NONCE) &&
 		    memcmp(oid.data, GNUTLS_OCSP_NONCE, oid.size) == 0) {
 			gnutls_datum_t nonce;
 			unsigned int ncrit;
 
-			ret = gnutls_ocsp_resp_get_nonce(resp, &ncrit, &nonce);
+			ret =
+			    gnutls_ocsp_resp_get_nonce(resp, &ncrit,
+						       &nonce);
 			if (ret != GNUTLS_E_SUCCESS) {
 				addf(str, "error: get_nonce: %s\n",
 				     gnutls_strerror(ret));
@@ -478,16 +496,17 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 				gnutls_free(nonce.data);
 			}
 		} else {
-			addf(str, "\t\tUnknown extension %s (%s):\n", oid.data,
+			addf(str, "\t\tUnknown extension %s (%s):\n",
+			     oid.data,
 			     critical ? "critical" : "not critical");
 
 			adds(str, _("\t\t\tASCII: "));
-			_gnutls_buffer_asciiprint(str, (char *)data.data,
+			_gnutls_buffer_asciiprint(str, (char *) data.data,
 						  data.size);
 			addf(str, "\n");
 
 			adds(str, _("\t\t\tHexdump: "));
-			_gnutls_buffer_hexprint(str, (char *)data.data,
+			_gnutls_buffer_hexprint(str, (char *) data.data,
 						data.size);
 			adds(str, "\n");
 		}
@@ -496,40 +515,46 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 		gnutls_free(data.data);
 	}
 
-	ret = gnutls_ocsp_resp_get_signature_algorithm(resp);
-	if (ret < 0)
-		addf(str, "error: get_signature_algorithm: %s\n",
-		     gnutls_strerror(ret));
-	else {
-		const char *name = gnutls_sign_algorithm_get_name(ret);
-		if (name == NULL)
-			name = _("unknown");
-		addf(str, _("\tSignature Algorithm: %s\n"), name);
-	}
-	if (ret != GNUTLS_SIGN_UNKNOWN && gnutls_sign_is_secure(ret) == 0) {
-		adds(str, _("warning: signed using a broken signature "
-			    "algorithm that can be forged.\n"));
-	}
-
+	/* Signature. */
 	if (format == GNUTLS_OCSP_PRINT_FULL) {
 		gnutls_datum_t sig;
-		gnutls_x509_crt_t *certs;
-		size_t ncerts, i;
-		gnutls_datum_t out;
 
-		/* Signature. */
+		ret = gnutls_ocsp_resp_get_signature_algorithm(resp);
+		if (ret < 0)
+			addf(str, "error: get_signature_algorithm: %s\n",
+			     gnutls_strerror(ret));
+		else {
+			const char *name =
+			    gnutls_sign_algorithm_get_name(ret);
+			if (name == NULL)
+				name = _("unknown");
+			addf(str, _("\tSignature Algorithm: %s\n"), name);
+		}
+		if (gnutls_sign_is_secure(ret) == 0) {
+			adds(str,
+			     _("warning: signed using a broken signature "
+			       "algorithm that can be forged.\n"));
+		}
+
 		ret = gnutls_ocsp_resp_get_signature(resp, &sig);
 		if (ret < 0)
 			addf(str, "error: get_signature: %s\n",
 			     gnutls_strerror(ret));
 		else {
 			adds(str, _("\tSignature:\n"));
-			_gnutls_buffer_hexdump(str, sig.data, sig.size, "\t\t");
+			_gnutls_buffer_hexdump(str, sig.data, sig.size,
+					       "\t\t");
 
 			gnutls_free(sig.data);
 		}
+	}
+
+	/* certs */
+	if (format == GNUTLS_OCSP_PRINT_FULL) {
+		gnutls_x509_crt_t *certs;
+		size_t ncerts, i;
+		gnutls_datum_t out;
 
-		/* certs */
 		ret = gnutls_ocsp_resp_get_certs(resp, &certs, &ncerts);
 		if (ret < 0)
 			addf(str, "error: get_certs: %s\n",
@@ -541,38 +566,45 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
 			for (i = 0; i < ncerts; i++) {
 				size_t s = 0;
 
-				ret = gnutls_x509_crt_print(
-					certs[i], GNUTLS_CRT_PRINT_FULL, &out);
+				ret =
+				    gnutls_x509_crt_print(certs[i],
+							  GNUTLS_CRT_PRINT_FULL,
+							  &out);
 				if (ret < 0)
 					addf(str, "error: crt_print: %s\n",
 					     gnutls_strerror(ret));
 				else {
-					addf(str, "%.*s", out.size, out.data);
+					addf(str, "%.*s", out.size,
+					     out.data);
 					gnutls_free(out.data);
 				}
 
-				ret = gnutls_x509_crt_export(
-					certs[i], GNUTLS_X509_FMT_PEM, NULL,
-					&s);
+				ret =
+				    gnutls_x509_crt_export(certs[i],
+							   GNUTLS_X509_FMT_PEM,
+							   NULL, &s);
 				if (ret != GNUTLS_E_SHORT_MEMORY_BUFFER)
-					addf(str, "error: crt_export: %s\n",
+					addf(str,
+					     "error: crt_export: %s\n",
 					     gnutls_strerror(ret));
 				else {
 					out.data = gnutls_malloc(s);
 					if (out.data == NULL)
-						addf(str, "error: malloc: %s\n",
-						     gnutls_strerror(
-							     GNUTLS_E_MEMORY_ERROR));
+						addf(str,
+						     "error: malloc: %s\n",
+						     gnutls_strerror
+						     (GNUTLS_E_MEMORY_ERROR));
 					else {
-						ret = gnutls_x509_crt_export(
-							certs[i],
-							GNUTLS_X509_FMT_PEM,
-							out.data, &s);
+						ret =
+						    gnutls_x509_crt_export
+						    (certs[i],
+						     GNUTLS_X509_FMT_PEM,
+						     out.data, &s);
 						if (ret < 0)
 							addf(str,
 							     "error: crt_export: %s\n",
-							     gnutls_strerror(
-								     ret));
+							     gnutls_strerror
+							     (ret));
 						else {
 							out.size = s;
 							addf(str, "%.*s",
@@ -607,9 +639,10 @@ static void print_resp(gnutls_buffer_st *str, gnutls_ocsp_resp_const_t resp,
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  **/
-int gnutls_ocsp_resp_print(gnutls_ocsp_resp_const_t resp,
-			   gnutls_ocsp_print_formats_t format,
-			   gnutls_datum_t *out)
+int
+gnutls_ocsp_resp_print(gnutls_ocsp_resp_t resp,
+		       gnutls_ocsp_print_formats_t format,
+		       gnutls_datum_t * out)
 {
 	gnutls_buffer_st str;
 	int rc;
