@@ -32,18 +32,21 @@
 #include <gnutls_pk.h>
 #include <gnutls_mpi.h>
 #include <gnutls_ecc.h>
+#include <pin.h>
 
 /**
  * gnutls_x509_privkey_init:
- * @key: The structure to be initialized
+ * @key: A pointer to the type to be initialized
  *
- * This function will initialize an private key structure.
+ * This function will initialize a private key type.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  **/
 int gnutls_x509_privkey_init(gnutls_x509_privkey_t * key)
 {
+	FAIL_IF_LIB_ERROR;
+
 	*key = gnutls_calloc(1, sizeof(gnutls_x509_privkey_int));
 
 	if (*key) {
@@ -55,9 +58,17 @@ int gnutls_x509_privkey_init(gnutls_x509_privkey_t * key)
 	return GNUTLS_E_MEMORY_ERROR;
 }
 
+void _gnutls_x509_privkey_reinit(gnutls_x509_privkey_t key)
+{
+	gnutls_pk_params_clear(&key->params);
+	gnutls_pk_params_release(&key->params);
+	asn1_delete_structure2(&key->key, ASN1_DELETE_FLAG_ZEROIZE);
+	key->key = ASN1_TYPE_EMPTY;
+}
+
 /**
  * gnutls_x509_privkey_deinit:
- * @key: The structure to be deinitialized
+ * @key: The key to be deinitialized
  *
  * This function will deinitialize a private key structure.
  **/
@@ -66,9 +77,7 @@ void gnutls_x509_privkey_deinit(gnutls_x509_privkey_t key)
 	if (!key)
 		return;
 
-	gnutls_pk_params_clear(&key->params);
-	gnutls_pk_params_release(&key->params);
-	asn1_delete_structure(&key->key);
+	_gnutls_x509_privkey_reinit(key);
 	gnutls_free(key);
 }
 
@@ -128,6 +137,8 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 
 	gnutls_pk_params_init(&pkey->params);
 
+	pkey->params.algo = GNUTLS_PK_RSA;
+
 	if ((result =
 	     asn1_create_element(_gnutls_get_gnutls_asn(),
 				 "GNUTLS.RSAPrivateKey",
@@ -137,7 +148,7 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 	}
 
 	result =
-	    asn1_der_decoding(&pkey_asn, raw_key->data, raw_key->size,
+	    _asn1_strict_der_decode(&pkey_asn, raw_key->data, raw_key->size,
 			      NULL);
 	if (result != ASN1_SUCCESS) {
 		gnutls_assert();
@@ -161,14 +172,14 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 	pkey->params.params_nr++;
 
 	if ((result =
-	     _gnutls_x509_read_int(pkey_asn, "privateExponent",
+	     _gnutls_x509_read_key_int(pkey_asn, "privateExponent",
 				   &pkey->params.params[2])) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_int(pkey_asn, "prime1",
+	if ((result = _gnutls_x509_read_key_int(pkey_asn, "prime1",
 					    &pkey->params.params[3])) < 0)
 	{
 		gnutls_assert();
@@ -176,7 +187,7 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_int(pkey_asn, "prime2",
+	if ((result = _gnutls_x509_read_key_int(pkey_asn, "prime2",
 					    &pkey->params.params[4])) < 0)
 	{
 		gnutls_assert();
@@ -184,7 +195,7 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_int(pkey_asn, "coefficient",
+	if ((result = _gnutls_x509_read_key_int(pkey_asn, "coefficient",
 					    &pkey->params.params[5])) < 0)
 	{
 		gnutls_assert();
@@ -192,7 +203,7 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_int(pkey_asn, "exponent1",
+	if ((result = _gnutls_x509_read_key_int(pkey_asn, "exponent1",
 					    &pkey->params.params[6])) < 0)
 	{
 		gnutls_assert();
@@ -200,7 +211,7 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_int(pkey_asn, "exponent2",
+	if ((result = _gnutls_x509_read_key_int(pkey_asn, "exponent2",
 					    &pkey->params.params[7])) < 0)
 	{
 		gnutls_assert();
@@ -220,7 +231,7 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 	return pkey_asn;
 
       error:
-	asn1_delete_structure(&pkey_asn);
+	asn1_delete_structure2(&pkey_asn, ASN1_DELETE_FLAG_ZEROIZE);
 	gnutls_pk_params_clear(&pkey->params);
 	gnutls_pk_params_release(&pkey->params);
 	return NULL;
@@ -230,12 +241,11 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 /* Converts an ECC key to
  * an internal structure (gnutls_private_key)
  */
-ASN1_TYPE
-_gnutls_privkey_decode_ecc_key(const gnutls_datum_t * raw_key,
-			       gnutls_x509_privkey_t pkey)
+int
+_gnutls_privkey_decode_ecc_key(ASN1_TYPE* pkey_asn, const gnutls_datum_t * raw_key,
+			       gnutls_x509_privkey_t pkey, gnutls_ecc_curve_t curve)
 {
 	int ret;
-	ASN1_TYPE pkey_asn;
 	unsigned int version;
 	char oid[MAX_OID_SIZE];
 	int oid_size;
@@ -243,23 +253,25 @@ _gnutls_privkey_decode_ecc_key(const gnutls_datum_t * raw_key,
 
 	gnutls_pk_params_init(&pkey->params);
 
+	pkey->params.algo = GNUTLS_PK_EC;
 	if ((ret =
 	     asn1_create_element(_gnutls_get_gnutls_asn(),
 				 "GNUTLS.ECPrivateKey",
-				 &pkey_asn)) != ASN1_SUCCESS) {
+				 pkey_asn)) != ASN1_SUCCESS) {
 		gnutls_assert();
-		return NULL;
+		return _gnutls_asn2err(ret);
 	}
 
 	ret =
-	    asn1_der_decoding(&pkey_asn, raw_key->data, raw_key->size,
+	    _asn1_strict_der_decode(pkey_asn, raw_key->data, raw_key->size,
 			      NULL);
 	if (ret != ASN1_SUCCESS) {
 		gnutls_assert();
+		ret = _gnutls_asn2err(ret);
 		goto error;
 	}
 
-	ret = _gnutls_x509_read_uint(pkey_asn, "Version", &version);
+	ret = _gnutls_x509_read_uint(*pkey_asn, "Version", &version);
 	if (ret < 0) {
 		gnutls_assert();
 		goto error;
@@ -270,28 +282,37 @@ _gnutls_privkey_decode_ecc_key(const gnutls_datum_t * raw_key,
 		    ("ECC private key version %u is not supported\n",
 		     version);
 		gnutls_assert();
+		ret = GNUTLS_E_ECC_UNSUPPORTED_CURVE;
 		goto error;
 	}
 
 	/* read the curve */
-	oid_size = sizeof(oid);
-	ret =
-	    asn1_read_value(pkey_asn, "parameters.namedCurve", oid,
+	if (curve == GNUTLS_ECC_CURVE_INVALID) {
+		oid_size = sizeof(oid);
+		ret =
+		    asn1_read_value(*pkey_asn, "parameters.namedCurve", oid,
 			    &oid_size);
-	if (ret != ASN1_SUCCESS) {
-		gnutls_assert();
-		goto error;
+		if (ret != ASN1_SUCCESS) {
+			gnutls_assert();
+			ret = _gnutls_asn2err(ret);
+			goto error;
+		}
+
+		pkey->params.flags = gnutls_oid_to_ecc_curve(oid);
+
+		if (pkey->params.flags == GNUTLS_ECC_CURVE_INVALID) {
+			_gnutls_debug_log("Curve %s is not supported\n", oid);
+			gnutls_assert();
+			ret = GNUTLS_E_ECC_UNSUPPORTED_CURVE;
+			goto error;
+		}
+	} else {
+		pkey->params.flags = curve;
 	}
 
-	pkey->params.flags = _gnutls_oid_to_ecc_curve(oid);
-	if (pkey->params.flags == GNUTLS_ECC_CURVE_INVALID) {
-		_gnutls_debug_log("Curve %s is not supported\n", oid);
-		gnutls_assert();
-		goto error;
-	}
 
 	/* read the public key */
-	ret = _gnutls_x509_read_value(pkey_asn, "publicKey", &out);
+	ret = _gnutls_x509_read_value(*pkey_asn, "publicKey", &out);
 	if (ret < 0) {
 		gnutls_assert();
 		goto error;
@@ -311,7 +332,7 @@ _gnutls_privkey_decode_ecc_key(const gnutls_datum_t * raw_key,
 
 	/* read the private key */
 	ret =
-	    _gnutls_x509_read_int(pkey_asn, "privateKey",
+	    _gnutls_x509_read_key_int(*pkey_asn, "privateKey",
 				  &pkey->params.params[ECC_K]);
 	if (ret < 0) {
 		gnutls_assert();
@@ -319,13 +340,13 @@ _gnutls_privkey_decode_ecc_key(const gnutls_datum_t * raw_key,
 	}
 	pkey->params.params_nr++;
 
-	return pkey_asn;
+	return 0;
 
       error:
-	asn1_delete_structure(&pkey_asn);
+	asn1_delete_structure2(pkey_asn, ASN1_DELETE_FLAG_ZEROIZE);
 	gnutls_pk_params_clear(&pkey->params);
 	gnutls_pk_params_release(&pkey->params);
-	return NULL;
+	return ret;
 
 }
 
@@ -344,10 +365,12 @@ decode_dsa_key(const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 		return NULL;
 	}
 
-	pkey->params.params_nr = 0;
+	gnutls_pk_params_init(&pkey->params);
+
+	pkey->params.algo = GNUTLS_PK_DSA;
 
 	result =
-	    asn1_der_decoding(&dsa_asn, raw_key->data, raw_key->size,
+	    _asn1_strict_der_decode(&dsa_asn, raw_key->data, raw_key->size,
 			      NULL);
 	if (result != ASN1_SUCCESS) {
 		gnutls_assert();
@@ -386,7 +409,7 @@ decode_dsa_key(const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_int(dsa_asn, "priv",
+	if ((result = _gnutls_x509_read_key_int(dsa_asn, "priv",
 					    &pkey->params.params[4])) < 0)
 	{
 		gnutls_assert();
@@ -397,7 +420,7 @@ decode_dsa_key(const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 	return dsa_asn;
 
       error:
-	asn1_delete_structure(&dsa_asn);
+	asn1_delete_structure2(&dsa_asn, ASN1_DELETE_FLAG_ZEROIZE);
 	gnutls_pk_params_clear(&pkey->params);
 	gnutls_pk_params_release(&pkey->params);
 	return NULL;
@@ -408,10 +431,11 @@ decode_dsa_key(const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 #define PEM_KEY_DSA "DSA PRIVATE KEY"
 #define PEM_KEY_RSA "RSA PRIVATE KEY"
 #define PEM_KEY_ECC "EC PRIVATE KEY"
+#define PEM_KEY_PKCS8 "PRIVATE KEY"
 
 /**
  * gnutls_x509_privkey_import:
- * @key: The structure to store the parsed key
+ * @key: The data to store the parsed key
  * @data: The DER or PEM encoded certificate.
  * @format: One of DER or PEM
  *
@@ -447,45 +471,94 @@ gnutls_x509_privkey_import(gnutls_x509_privkey_t key,
 	/* If the Certificate is in PEM format then decode it
 	 */
 	if (format == GNUTLS_X509_FMT_PEM) {
-		/* Try the first header */
-		result =
-		    _gnutls_fbase64_decode(PEM_KEY_RSA, data->data,
-					   data->size, &_data);
+		unsigned left;
+		char *ptr;
+		uint8_t *begin_ptr;
 
-		if (result >= 0)
-			key->pk_algorithm = GNUTLS_PK_RSA;
+		ptr = memmem(data->data, data->size, "PRIVATE KEY-----", sizeof("PRIVATE KEY-----")-1);
 
-		if (result == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR) {
-			/* try for the second header */
-			result =
-			    _gnutls_fbase64_decode(PEM_KEY_DSA, data->data,
-						   data->size, &_data);
+		result = GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
 
-			if (result >= 0)
-				key->pk_algorithm = GNUTLS_PK_DSA;
+		if (ptr != NULL) {
+			left = data->size - ((ptrdiff_t)ptr - (ptrdiff_t)data->data);
 
-			if (result ==
-			    GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR) {
-				/* try for the second header */
-				result =
-				    _gnutls_fbase64_decode(PEM_KEY_ECC,
-							   data->data,
-							   data->size,
-							   &_data);
-				if (result >= 0)
-					key->pk_algorithm = GNUTLS_PK_EC;
+			if (data->size - left > 15) {
+				ptr -= 15;
+				left += 15;
+			} else {
+				ptr = (char*)data->data;
+				left = data->size;
 			}
+
+			ptr = memmem(ptr, left, "-----BEGIN ", sizeof("-----BEGIN ")-1);
+			if (ptr != NULL) {
+				begin_ptr = (uint8_t*)ptr;
+				left = data->size - ((ptrdiff_t)begin_ptr - (ptrdiff_t)data->data);
+
+				ptr += sizeof("-----BEGIN ")-1;
+			}
+
+			if (ptr != NULL && left > sizeof(PEM_KEY_RSA)) {
+				if (memcmp(ptr, PEM_KEY_RSA, sizeof(PEM_KEY_RSA)-1) == 0) {
+					result =
+					    _gnutls_fbase64_decode(PEM_KEY_RSA, begin_ptr,
+							   	   left, &_data);
+					if (result >= 0)
+						key->pk_algorithm = GNUTLS_PK_RSA;
+				} else if (memcmp(ptr, PEM_KEY_ECC, sizeof(PEM_KEY_ECC)-1) == 0) {
+					result =
+					    _gnutls_fbase64_decode(PEM_KEY_ECC,
+								   begin_ptr,
+								   left,
+								   &_data);
+					if (result >= 0)
+						key->pk_algorithm = GNUTLS_PK_EC;
+				} else if (memcmp(ptr, PEM_KEY_DSA, sizeof(PEM_KEY_DSA)-1) == 0) {
+					result =
+					    _gnutls_fbase64_decode(PEM_KEY_DSA, begin_ptr,
+								   left, &_data);
+					if (result >= 0)
+						key->pk_algorithm = GNUTLS_PK_DSA;
+				}
+			}
+
+			if (key->pk_algorithm == GNUTLS_PK_UNKNOWN && ptr != NULL && left >= sizeof(PEM_KEY_PKCS8)) {
+				if (memcmp(ptr, PEM_KEY_PKCS8, sizeof(PEM_KEY_PKCS8)-1) == 0) {
+					result =
+					    _gnutls_fbase64_decode(PEM_KEY_PKCS8, begin_ptr,
+							   	   left, &_data);
+					if (result >= 0) {
+						/* signal for PKCS #8 keys */
+						key->pk_algorithm = -1;
+					}
+				}
+			}
+
 		}
 
 		if (result < 0) {
 			gnutls_assert();
-			goto failover;
+			return result;
 		}
 
 		need_free = 1;
 	}
 
-	if (key->pk_algorithm == GNUTLS_PK_RSA) {
+	if (key->expanded) {
+		_gnutls_x509_privkey_reinit(key);
+	}
+	key->expanded = 1;
+
+	if (key->pk_algorithm == (gnutls_pk_algorithm_t)-1) {
+		result =
+		    gnutls_x509_privkey_import_pkcs8(key, data, format,
+						     NULL,
+						     GNUTLS_PKCS_PLAIN);
+		if (result < 0) {
+			gnutls_assert();
+			key->key = NULL;
+		}
+	} else if (key->pk_algorithm == GNUTLS_PK_RSA) {
 		key->key =
 		    _gnutls_privkey_decode_pkcs1_rsa_key(&_data, key);
 		if (key->key == NULL)
@@ -495,11 +568,13 @@ gnutls_x509_privkey_import(gnutls_x509_privkey_t key,
 		if (key->key == NULL)
 			gnutls_assert();
 	} else if (key->pk_algorithm == GNUTLS_PK_EC) {
-		key->key = _gnutls_privkey_decode_ecc_key(&_data, key);
-		if (key->key == NULL)
+		result = _gnutls_privkey_decode_ecc_key(&key->key, &_data, key, 0);
+		if (result < 0) {
 			gnutls_assert();
+			key->key = NULL;
+		}
 	} else {
-		/* Try decoding with both, and accept the one that
+		/* Try decoding each of the keys, and accept the one that
 		 * succeeds.
 		 */
 		key->pk_algorithm = GNUTLS_PK_RSA;
@@ -511,11 +586,19 @@ gnutls_x509_privkey_import(gnutls_x509_privkey_t key,
 			key->key = decode_dsa_key(&_data, key);
 			if (key->key == NULL) {
 				key->pk_algorithm = GNUTLS_PK_EC;
-				key->key =
-				    _gnutls_privkey_decode_ecc_key(&_data,
-								   key);
-				if (key->key == NULL)
-					gnutls_assert();
+				result =
+				    _gnutls_privkey_decode_ecc_key(&key->key, &_data, key, 0);
+				if (result < 0) {
+					result =
+					    gnutls_x509_privkey_import_pkcs8(key, data, format,
+									     NULL,
+									     GNUTLS_PKCS_PLAIN);
+					if (result < 0) {
+						gnutls_assert();
+						key->key = NULL;
+					}
+				}
+
 			}
 		}
 	}
@@ -523,7 +606,8 @@ gnutls_x509_privkey_import(gnutls_x509_privkey_t key,
 	if (key->key == NULL) {
 		gnutls_assert();
 		result = GNUTLS_E_ASN1_DER_ERROR;
-		goto failover;
+	} else {
+		result = 0;
 	}
 
 	if (need_free)
@@ -532,22 +616,6 @@ gnutls_x509_privkey_import(gnutls_x509_privkey_t key,
 	/* The key has now been decoded.
 	 */
 
-	return 0;
-
-      failover:
-	/* Try PKCS #8 */
-	if (result == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR) {
-		_gnutls_debug_log
-		    ("Falling back to PKCS #8 key decoding\n");
-		result =
-		    gnutls_x509_privkey_import_pkcs8(key, data, format,
-						     NULL,
-						     GNUTLS_PKCS_PLAIN);
-	}
-
-	if (need_free)
-		_gnutls_free_datum(&_data);
-
 	return result;
 }
 
@@ -595,7 +663,7 @@ static int import_pkcs12_privkey(gnutls_x509_privkey_t key,
 
 /**
  * gnutls_x509_privkey_import2:
- * @key: The structure to store the parsed key
+ * @key: The data to store the parsed key
  * @data: The DER or PEM encoded key.
  * @format: One of DER or PEM
  * @password: A password (optional)
@@ -609,7 +677,8 @@ static int import_pkcs12_privkey(gnutls_x509_privkey_t key,
  * and the openssl format.
  *
  * If the provided key is encrypted but no password was given, then
- * %GNUTLS_E_DECRYPTION_FAILED is returned.
+ * %GNUTLS_E_DECRYPTION_FAILED is returned. Since GnuTLS 3.4.0 this
+ * function will utilize the PIN callbacks if any.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
@@ -621,9 +690,47 @@ gnutls_x509_privkey_import2(gnutls_x509_privkey_t key,
 			    const char *password, unsigned int flags)
 {
 	int ret = 0;
+	char pin[GNUTLS_PKCS11_MAX_PIN_LEN];
+	unsigned head_enc = 1;
 
-	if (password == NULL && !(flags & GNUTLS_PKCS_NULL_PASSWORD)) {
+	if (format == GNUTLS_X509_FMT_PEM) {
+		size_t left;
+		char *ptr;
+
+		ptr = memmem(data->data, data->size, "PRIVATE KEY-----", sizeof("PRIVATE KEY-----")-1);
+
+		if (ptr != NULL) {
+			left = data->size - ((ptrdiff_t)ptr - (ptrdiff_t)data->data);
+
+			if (data->size - left > 15) {
+				ptr -= 15;
+				left += 15;
+			} else {
+				ptr = (char*)data->data;
+				left = data->size;
+			}
+
+			ptr = memmem(ptr, left, "-----BEGIN ", sizeof("-----BEGIN ")-1);
+			if (ptr != NULL) {
+				ptr += sizeof("-----BEGIN ")-1;
+				left = data->size - ((ptrdiff_t)ptr - (ptrdiff_t)data->data);
+			}
+
+			if (ptr != NULL && left > sizeof(PEM_KEY_RSA)) {
+				if (memcmp(ptr, PEM_KEY_RSA, sizeof(PEM_KEY_RSA)-1) == 0 ||
+				    memcmp(ptr, PEM_KEY_ECC, sizeof(PEM_KEY_ECC)-1) == 0 ||
+				    memcmp(ptr, PEM_KEY_DSA, sizeof(PEM_KEY_DSA)-1) == 0) {
+				    	head_enc = 0;
+				}
+			}
+		}
+	}
+
+	if (head_enc == 0 || (password == NULL && !(flags & GNUTLS_PKCS_NULL_PASSWORD))) {
 		ret = gnutls_x509_privkey_import(key, data, format);
+		if (ret >= 0)
+			return ret;
+
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -631,9 +738,24 @@ gnutls_x509_privkey_import2(gnutls_x509_privkey_t key,
 
 	if ((password != NULL || (flags & GNUTLS_PKCS_NULL_PASSWORD))
 	    || ret < 0) {
+
 		ret =
 		    gnutls_x509_privkey_import_pkcs8(key, data, format,
 						     password, flags);
+
+		if (ret == GNUTLS_E_DECRYPTION_FAILED &&
+		    password == NULL && (!(flags & GNUTLS_PKCS_PLAIN))) {
+		    /* use the callback if any */
+			ret = _gnutls_retrieve_pin(&key->pin, "key:", "", 0, pin, sizeof(pin));
+			if (ret == 0) {
+		    		password = pin;
+			}
+
+			ret =
+			    gnutls_x509_privkey_import_pkcs8(key, data, format,
+						     password, flags);
+		}
+
 		if (ret < 0) {
 			if (ret == GNUTLS_E_DECRYPTION_FAILED)
 				goto cleanup;
@@ -668,7 +790,7 @@ gnutls_x509_privkey_import2(gnutls_x509_privkey_t key,
 
 /**
  * gnutls_x509_privkey_import_rsa_raw:
- * @key: The structure to store the parsed key
+ * @key: The data to store the parsed key
  * @m: holds the modulus
  * @e: holds the public exponent
  * @d: holds the private exponent
@@ -698,15 +820,15 @@ gnutls_x509_privkey_import_rsa_raw(gnutls_x509_privkey_t key,
 
 /**
  * gnutls_x509_privkey_import_rsa_raw2:
- * @key: The structure to store the parsed key
+ * @key: The data to store the parsed key
  * @m: holds the modulus
  * @e: holds the public exponent
  * @d: holds the private exponent
  * @p: holds the first prime (p)
  * @q: holds the second prime (q)
- * @u: holds the coefficient
- * @e1: holds e1 = d mod (p-1)
- * @e2: holds e2 = d mod (q-1)
+ * @u: holds the coefficient (optional)
+ * @e1: holds e1 = d mod (p-1) (optional)
+ * @e2: holds e2 = d mod (q-1) (optional)
  *
  * This function will convert the given RSA raw parameters to the
  * native #gnutls_x509_privkey_t format.  The output will be stored in
@@ -737,7 +859,7 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 	gnutls_pk_params_init(&key->params);
 
 	siz = m->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[0], m->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[0], m->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
@@ -745,7 +867,7 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 	key->params.params_nr++;
 
 	siz = e->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[1], e->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[1], e->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
@@ -753,7 +875,7 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 	key->params.params_nr++;
 
 	siz = d->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[2], d->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[2], d->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
@@ -761,7 +883,7 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 	key->params.params_nr++;
 
 	siz = p->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[3], p->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[3], p->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
@@ -769,25 +891,27 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 	key->params.params_nr++;
 
 	siz = q->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[4], q->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[4], q->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
 	}
 	key->params.params_nr++;
 
-	siz = u->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[5], u->data, siz)) {
-		gnutls_assert();
-		ret = GNUTLS_E_MPI_SCAN_FAILED;
-		goto cleanup;
+	if (u) {
+		siz = u->size;
+		if (_gnutls_mpi_init_scan_nz(&key->params.params[RSA_COEF], u->data, siz)) {
+			gnutls_assert();
+			ret = GNUTLS_E_MPI_SCAN_FAILED;
+			goto cleanup;
+		}
+		key->params.params_nr++;
 	}
-	key->params.params_nr++;
 
 	if (e1 && e2) {
 		siz = e1->size;
-		if (_gnutls_mpi_scan_nz
-		    (&key->params.params[6], e1->data, siz)) {
+		if (_gnutls_mpi_init_scan_nz
+		    (&key->params.params[RSA_E1], e1->data, siz)) {
 			gnutls_assert();
 			ret = GNUTLS_E_MPI_SCAN_FAILED;
 			goto cleanup;
@@ -795,8 +919,8 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 		key->params.params_nr++;
 
 		siz = e2->size;
-		if (_gnutls_mpi_scan_nz
-		    (&key->params.params[7], e2->data, siz)) {
+		if (_gnutls_mpi_init_scan_nz
+		    (&key->params.params[RSA_E2], e2->data, siz)) {
 			gnutls_assert();
 			ret = GNUTLS_E_MPI_SCAN_FAILED;
 			goto cleanup;
@@ -832,7 +956,7 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 
 /**
  * gnutls_x509_privkey_import_dsa_raw:
- * @key: The structure to store the parsed key
+ * @key: The data to store the parsed key
  * @p: holds the p
  * @q: holds the q
  * @g: holds the g
@@ -863,35 +987,35 @@ gnutls_x509_privkey_import_dsa_raw(gnutls_x509_privkey_t key,
 	}
 
 	siz = p->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[0], p->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[0], p->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
 	}
 
 	siz = q->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[1], q->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[1], q->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
 	}
 
 	siz = g->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[2], g->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[2], g->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
 	}
 
 	siz = y->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[3], y->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[3], y->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
 	}
 
 	siz = x->size;
-	if (_gnutls_mpi_scan_nz(&key->params.params[4], x->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[4], x->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
@@ -919,7 +1043,7 @@ gnutls_x509_privkey_import_dsa_raw(gnutls_x509_privkey_t key,
 
 /**
  * gnutls_x509_privkey_import_ecc_raw:
- * @key: The structure to store the parsed key
+ * @key: The data to store the parsed key
  * @curve: holds the curve
  * @x: holds the x
  * @y: holds the y
@@ -950,7 +1074,7 @@ gnutls_x509_privkey_import_ecc_raw(gnutls_x509_privkey_t key,
 
 	key->params.flags = curve;
 
-	if (_gnutls_mpi_scan_nz
+	if (_gnutls_mpi_init_scan_nz
 	    (&key->params.params[ECC_X], x->data, x->size)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
@@ -958,7 +1082,7 @@ gnutls_x509_privkey_import_ecc_raw(gnutls_x509_privkey_t key,
 	}
 	key->params.params_nr++;
 
-	if (_gnutls_mpi_scan_nz
+	if (_gnutls_mpi_init_scan_nz
 	    (&key->params.params[ECC_Y], y->data, y->size)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
@@ -966,7 +1090,7 @@ gnutls_x509_privkey_import_ecc_raw(gnutls_x509_privkey_t key,
 	}
 	key->params.params_nr++;
 
-	if (_gnutls_mpi_scan_nz
+	if (_gnutls_mpi_init_scan_nz
 	    (&key->params.params[ECC_K], k->data, k->size)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
@@ -988,7 +1112,7 @@ gnutls_x509_privkey_import_ecc_raw(gnutls_x509_privkey_t key,
 
 /**
  * gnutls_x509_privkey_get_pk_algorithm:
- * @key: should contain a #gnutls_x509_privkey_t structure
+ * @key: should contain a #gnutls_x509_privkey_t type
  *
  * This function will return the public key algorithm of a private
  * key.
@@ -1008,7 +1132,7 @@ int gnutls_x509_privkey_get_pk_algorithm(gnutls_x509_privkey_t key)
 
 /**
  * gnutls_x509_privkey_get_pk_algorithm2:
- * @key: should contain a #gnutls_x509_privkey_t structure
+ * @key: should contain a #gnutls_x509_privkey_t type
  * @bits: The number of bits in the public key algorithm
  *
  * This function will return the public key algorithm of a private
@@ -1129,7 +1253,7 @@ gnutls_x509_privkey_export2(gnutls_x509_privkey_t key,
 
 /**
  * gnutls_x509_privkey_sec_param:
- * @key: a key structure
+ * @key: a key
  *
  * This function will return the security parameter appropriate with
  * this private key.
@@ -1152,7 +1276,7 @@ gnutls_sec_param_t gnutls_x509_privkey_sec_param(gnutls_x509_privkey_t key)
 
 /**
  * gnutls_x509_privkey_export_ecc_raw:
- * @key: a structure that holds the rsa parameters
+ * @key: a key
  * @curve: will hold the curve
  * @x: will hold the x coordinate
  * @y: will hold the y coordinate
@@ -1173,47 +1297,17 @@ int gnutls_x509_privkey_export_ecc_raw(gnutls_x509_privkey_t key,
 				       gnutls_datum_t * y,
 				       gnutls_datum_t * k)
 {
-	int ret;
-
 	if (key == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-	*curve = key->params.flags;
-
-	/* X */
-	ret = _gnutls_mpi_dprint_lz(key->params.params[ECC_X], x);
-	if (ret < 0) {
-		gnutls_assert();
-		return ret;
-	}
-
-	/* Y */
-	ret = _gnutls_mpi_dprint_lz(key->params.params[ECC_Y], y);
-	if (ret < 0) {
-		gnutls_assert();
-		_gnutls_free_datum(x);
-		return ret;
-	}
-
-
-	/* K */
-	ret = _gnutls_mpi_dprint_lz(key->params.params[ECC_K], k);
-	if (ret < 0) {
-		gnutls_assert();
-		_gnutls_free_datum(x);
-		_gnutls_free_datum(y);
-		return ret;
-	}
-
-	return 0;
-
+	return _gnutls_params_get_ecc_raw(&key->params, curve, x, y, k);
 }
 
 /**
  * gnutls_x509_privkey_export_rsa_raw:
- * @key: a structure that holds the rsa parameters
+ * @key: a key
  * @m: will hold the modulus
  * @e: will hold the public exponent
  * @d: will hold the private exponent
@@ -1234,14 +1328,12 @@ gnutls_x509_privkey_export_rsa_raw(gnutls_x509_privkey_t key,
 				   gnutls_datum_t * d, gnutls_datum_t * p,
 				   gnutls_datum_t * q, gnutls_datum_t * u)
 {
-
-	return gnutls_x509_privkey_export_rsa_raw2(key, m, e, d, p, q, u,
-						   NULL, NULL);
+	return _gnutls_params_get_rsa_raw(&key->params, m, e, d, p, q, u, NULL, NULL);
 }
 
 /**
  * gnutls_x509_privkey_export_rsa_raw2:
- * @key: a structure that holds the rsa parameters
+ * @key: a key
  * @m: will hold the modulus
  * @e: will hold the public exponent
  * @d: will hold the private exponent
@@ -1268,110 +1360,12 @@ gnutls_x509_privkey_export_rsa_raw2(gnutls_x509_privkey_t key,
 				    gnutls_datum_t * e1,
 				    gnutls_datum_t * e2)
 {
-	int ret;
-	gnutls_pk_params_st pk_params;
-
-	gnutls_pk_params_init(&pk_params);
-
-	if (key == NULL) {
-		gnutls_assert();
-		return GNUTLS_E_INVALID_REQUEST;
-	}
-
-	m->data = e->data = d->data = p->data = q->data = u->data = NULL;
-	m->size = e->size = d->size = p->size = q->size = u->size = 0;
-
-	ret = _gnutls_pk_params_copy(&pk_params, &key->params);
-	if (ret < 0) {
-		gnutls_assert();
-		return ret;
-	}
-
-	ret = _gnutls_pk_fixup(GNUTLS_PK_RSA, GNUTLS_EXPORT, &pk_params);
-	if (ret < 0) {
-		gnutls_assert();
-		goto error;
-	}
-
-	ret = _gnutls_mpi_dprint_lz(pk_params.params[0], m);
-	if (ret < 0) {
-		gnutls_assert();
-		goto error;
-	}
-
-	/* E */
-	ret = _gnutls_mpi_dprint_lz(pk_params.params[1], e);
-	if (ret < 0) {
-		gnutls_assert();
-		goto error;
-	}
-
-	/* D */
-	ret = _gnutls_mpi_dprint_lz(pk_params.params[2], d);
-	if (ret < 0) {
-		gnutls_assert();
-		goto error;
-	}
-
-	/* P */
-	ret = _gnutls_mpi_dprint_lz(pk_params.params[3], p);
-	if (ret < 0) {
-		gnutls_assert();
-		goto error;
-	}
-
-	/* Q */
-	ret = _gnutls_mpi_dprint_lz(pk_params.params[4], q);
-	if (ret < 0) {
-		gnutls_assert();
-		goto error;
-	}
-
-	/* U */
-	ret = _gnutls_mpi_dprint_lz(key->params.params[5], u);
-	if (ret < 0) {
-		gnutls_assert();
-		goto error;
-	}
-
-	/* E1 */
-	if (e1) {
-		ret = _gnutls_mpi_dprint_lz(key->params.params[6], e1);
-		if (ret < 0) {
-			gnutls_assert();
-			goto error;
-		}
-	}
-
-	/* E2 */
-	if (e2) {
-		ret = _gnutls_mpi_dprint_lz(key->params.params[7], e2);
-		if (ret < 0) {
-			gnutls_assert();
-			goto error;
-		}
-	}
-
-	gnutls_pk_params_clear(&pk_params);
-	gnutls_pk_params_release(&pk_params);
-
-	return 0;
-
-      error:
-	_gnutls_free_datum(m);
-	_gnutls_free_datum(d);
-	_gnutls_free_datum(e);
-	_gnutls_free_datum(p);
-	_gnutls_free_datum(q);
-	gnutls_pk_params_clear(&pk_params);
-	gnutls_pk_params_release(&pk_params);
-
-	return ret;
+	return _gnutls_params_get_rsa_raw(&key->params, m, e, d, p, q, u, e1, e2);
 }
 
 /**
  * gnutls_x509_privkey_export_dsa_raw:
- * @key: a structure that holds the DSA parameters
+ * @key: a key
  * @p: will hold the p
  * @q: will hold the q
  * @g: will hold the g
@@ -1391,66 +1385,101 @@ gnutls_x509_privkey_export_dsa_raw(gnutls_x509_privkey_t key,
 				   gnutls_datum_t * g, gnutls_datum_t * y,
 				   gnutls_datum_t * x)
 {
-	int ret;
-
-	if (key == NULL) {
-		gnutls_assert();
-		return GNUTLS_E_INVALID_REQUEST;
-	}
-
-	/* P */
-	ret = _gnutls_mpi_dprint_lz(key->params.params[0], p);
-	if (ret < 0) {
-		gnutls_assert();
-		return ret;
-	}
-
-	/* Q */
-	ret = _gnutls_mpi_dprint_lz(key->params.params[1], q);
-	if (ret < 0) {
-		gnutls_assert();
-		_gnutls_free_datum(p);
-		return ret;
-	}
-
-
-	/* G */
-	ret = _gnutls_mpi_dprint_lz(key->params.params[2], g);
-	if (ret < 0) {
-		gnutls_assert();
-		_gnutls_free_datum(p);
-		_gnutls_free_datum(q);
-		return ret;
-	}
-
-
-	/* Y */
-	ret = _gnutls_mpi_dprint_lz(key->params.params[3], y);
-	if (ret < 0) {
-		gnutls_assert();
-		_gnutls_free_datum(p);
-		_gnutls_free_datum(g);
-		_gnutls_free_datum(q);
-		return ret;
-	}
-
-	/* X */
-	ret = _gnutls_mpi_dprint_lz(key->params.params[4], x);
-	if (ret < 0) {
-		gnutls_assert();
-		_gnutls_free_datum(y);
-		_gnutls_free_datum(p);
-		_gnutls_free_datum(g);
-		_gnutls_free_datum(q);
-		return ret;
-	}
-
-	return 0;
+	return _gnutls_params_get_dsa_raw(&key->params, p, q, g, y, x);
 }
 
+#ifdef ENABLE_FIPS140
+static int pct_test(gnutls_pk_algorithm_t algo, const gnutls_pk_params_st* params)
+{
+int ret;
+gnutls_datum_t sig = {NULL, 0};
+const char const_data[20] = "onetwothreefourfive";
+gnutls_datum_t ddata, tmp = {NULL,0};
+char* gen_data = NULL;
+
+	if (algo == GNUTLS_PK_DSA || algo == GNUTLS_PK_EC) {
+		unsigned hash_len;
+
+		_gnutls_dsa_q_to_hash(algo, params, &hash_len);
+		gen_data = gnutls_malloc(hash_len);
+		gnutls_rnd(GNUTLS_RND_NONCE, gen_data, hash_len);
+
+		ddata.data = (void*)gen_data;
+		ddata.size = hash_len;
+	} else {
+		ddata.data = (void*)const_data;
+		ddata.size = sizeof(const_data);
+	}
+
+	switch (algo) {
+	case GNUTLS_PK_RSA:
+		ret = _gnutls_pk_encrypt(algo, &sig, &ddata, params);
+		if (ret < 0) {
+			ret = gnutls_assert_val(GNUTLS_E_PK_GENERATION_ERROR);
+			goto cleanup;
+		}
+
+		if (ddata.size == sig.size && memcmp(ddata.data, sig.data, sig.size) == 0) {
+			ret = gnutls_assert_val(GNUTLS_E_PK_GENERATION_ERROR);
+			gnutls_assert();
+			goto cleanup;
+		}
+
+		ret = _gnutls_pk_decrypt(algo, &tmp, &sig, params);
+		if (ret < 0) {
+			ret = gnutls_assert_val(GNUTLS_E_PK_GENERATION_ERROR);
+			gnutls_assert();
+			goto cleanup;
+		}
+
+		if (tmp.size != ddata.size || memcmp(tmp.data, ddata.data, tmp.size) != 0) {
+			ret = gnutls_assert_val(GNUTLS_E_PK_GENERATION_ERROR);
+			gnutls_assert();
+			goto cleanup;
+		}
+
+		free(sig.data);
+		sig.data = NULL;
+
+		/* Here we don't know the purpose of the key. Check both
+		 * signing and encryption.
+		 */
+	case GNUTLS_PK_EC: /* we only do keys for ECDSA */
+	case GNUTLS_PK_DSA:
+		ret = _gnutls_pk_sign(algo, &sig, &ddata, params);
+		if (ret < 0) {
+			ret = gnutls_assert_val(GNUTLS_E_PK_GENERATION_ERROR);
+			goto cleanup;
+		}
+
+		ret = _gnutls_pk_verify(algo, &ddata, &sig, params);
+		if (ret < 0) {
+			ret = gnutls_assert_val(GNUTLS_E_PK_GENERATION_ERROR);
+			gnutls_assert();
+			goto cleanup;
+		}
+		break;
+
+	default:
+		ret = gnutls_assert_val(GNUTLS_E_UNKNOWN_PK_ALGORITHM);
+		goto cleanup;
+	}
+
+	ret = 0;
+cleanup:
+	if (ret == GNUTLS_E_PK_GENERATION_ERROR) {
+		_gnutls_switch_lib_state(LIB_STATE_ERROR);
+	}
+	gnutls_free(gen_data);
+	gnutls_free(sig.data);
+	gnutls_free(tmp.data);
+	return ret;
+}
+#endif
+
 /**
  * gnutls_x509_privkey_generate:
- * @key: should contain a #gnutls_x509_privkey_t structure
+ * @key: a key
  * @algo: is one of the algorithms in #gnutls_pk_algorithm_t.
  * @bits: the size of the modulus
  * @flags: unused for now.  Must be 0.
@@ -1462,6 +1491,9 @@ gnutls_x509_privkey_export_dsa_raw(gnutls_x509_privkey_t key,
  * can be substituted in the place of the bits parameter using the
  * GNUTLS_CURVE_TO_BITS() macro.
  *
+ * For DSA keys, if the subgroup size needs to be specified check
+ * the GNUTLS_SUBGROUP_TO_BITS() macro.
+ *
  * Do not set the number of bits directly, use gnutls_sec_param_to_pk_bits().
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
@@ -1488,12 +1520,28 @@ gnutls_x509_privkey_generate(gnutls_x509_privkey_t key,
 			bits = _gnutls_ecc_bits_to_curve(bits);
 	}
 
-	ret = _gnutls_pk_generate(algo, bits, &key->params);
+	ret = _gnutls_pk_generate_params(algo, bits, &key->params);
 	if (ret < 0) {
 		gnutls_assert();
 		return ret;
 	}
 
+	ret = _gnutls_pk_generate_keys(algo, bits, &key->params);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
+#ifndef ENABLE_FIPS140
+	ret = _gnutls_pk_verify_priv_params(algo, &key->params);
+#else
+	ret = pct_test(algo, &key->params);
+#endif
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
 	ret = _gnutls_asn1_encode_privkey(algo, &key->key, &key->params);
 	if (ret < 0) {
 		gnutls_assert();
@@ -1513,7 +1561,7 @@ gnutls_x509_privkey_generate(gnutls_x509_privkey_t key,
 
 /**
  * gnutls_x509_privkey_verify_params:
- * @key: should contain a #gnutls_x509_privkey_t structure
+ * @key: a key
  *
  * This function will verify the private key parameters.
  *
@@ -1524,7 +1572,7 @@ int gnutls_x509_privkey_verify_params(gnutls_x509_privkey_t key)
 {
 	int ret;
 
-	ret = _gnutls_pk_verify_params(key->pk_algorithm, &key->params);
+	ret = _gnutls_pk_verify_priv_params(key->pk_algorithm, &key->params);
 	if (ret < 0) {
 		gnutls_assert();
 		return ret;
@@ -1535,8 +1583,8 @@ int gnutls_x509_privkey_verify_params(gnutls_x509_privkey_t key)
 
 /**
  * gnutls_x509_privkey_get_key_id:
- * @key: Holds the key
- * @flags: should be 0 for now
+ * @key: a key
+ * @flags: should be one of the flags from %gnutls_keyid_flags_t
  * @output_data: will contain the key ID
  * @output_data_size: holds the size of output_data (and will be
  *   replaced by the actual size of parameters)
@@ -1568,7 +1616,7 @@ gnutls_x509_privkey_get_key_id(gnutls_x509_privkey_t key,
 
 	ret =
 	    _gnutls_get_key_id(key->pk_algorithm, &key->params,
-			       output_data, output_data_size);
+			       output_data, output_data_size, flags);
 	if (ret < 0) {
 		gnutls_assert();
 	}
@@ -1641,7 +1689,7 @@ _gnutls_x509_privkey_sign_hash2(gnutls_x509_privkey_t signer,
 
 /**
  * gnutls_x509_privkey_sign_hash:
- * @key: Holds the key
+ * @key: a key
  * @hash: holds the data to be signed
  * @signature: will contain newly allocated signature
  *
@@ -1681,7 +1729,7 @@ gnutls_x509_privkey_sign_hash(gnutls_x509_privkey_t key,
 
 /**
  * gnutls_x509_privkey_sign_data:
- * @key: Holds the key
+ * @key: a key
  * @digest: should be MD5 or SHA1
  * @flags: should be 0 for now
  * @data: holds the data to be signed
@@ -1703,8 +1751,6 @@ gnutls_x509_privkey_sign_hash(gnutls_x509_privkey_t key,
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
- *
- * Deprecated: Use gnutls_privkey_sign_data().
  */
 int
 gnutls_x509_privkey_sign_data(gnutls_x509_privkey_t key,
@@ -1716,7 +1762,7 @@ gnutls_x509_privkey_sign_data(gnutls_x509_privkey_t key,
 	int result;
 	gnutls_datum_t sig = { NULL, 0 };
 	gnutls_datum_t hash;
-	const mac_entry_st *me = mac_to_entry(digest);
+	const mac_entry_st *me = hash_to_entry(digest);
 
 	if (key == NULL) {
 		gnutls_assert();
@@ -1757,7 +1803,7 @@ gnutls_x509_privkey_sign_data(gnutls_x509_privkey_t key,
 
 /**
  * gnutls_x509_privkey_fix:
- * @key: Holds the key
+ * @key: a key
  *
  * This function will recalculate the secondary parameters in a key.
  * In RSA keys, this can be the coefficient and exponent1,2.
@@ -1774,7 +1820,7 @@ int gnutls_x509_privkey_fix(gnutls_x509_privkey_t key)
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-	asn1_delete_structure(&key->key);
+	asn1_delete_structure2(&key->key, ASN1_DELETE_FLAG_ZEROIZE);
 
 	ret =
 	    _gnutls_asn1_encode_privkey(key->pk_algorithm, &key->key,
@@ -1786,3 +1832,26 @@ int gnutls_x509_privkey_fix(gnutls_x509_privkey_t key)
 
 	return 0;
 }
+
+/**
+ * gnutls_x509_privkey_set_pin_function:
+ * @privkey: The certificate structure
+ * @fn: the callback
+ * @userdata: data associated with the callback
+ *
+ * This function will set a callback function to be used when
+ * it is required to access a protected object. This function overrides 
+ * the global function set using gnutls_pkcs11_set_pin_function().
+ *
+ * Note that this callback is used when decrypting a key.
+ *
+ * Since: 3.4.0
+ *
+ **/
+void gnutls_x509_privkey_set_pin_function(gnutls_x509_privkey_t privkey,
+				      gnutls_pin_callback_t fn,
+				      void *userdata)
+{
+	privkey->pin.cb = fn;
+	privkey->pin.data = userdata;
+}
