@@ -66,7 +66,7 @@ static void cipher_mac_bench(int algo, int mac_algo, int size)
 	key.data = _key;
 	key.size = keysize;
 
-	printf("%16s-%s ", gnutls_cipher_get_name(algo),
+	printf("%19s-%s ", gnutls_cipher_get_name(algo),
 	       gnutls_mac_get_name(mac_algo));
 	fflush(stdout);
 
@@ -134,7 +134,7 @@ static void cipher_bench(int algo, int size, int aead)
 	key.data = _key;
 	key.size = keysize;
 
-	printf("%16s ", gnutls_cipher_get_name(algo));
+	printf("%24s ", gnutls_cipher_get_name(algo));
 	fflush(stdout);
 
 	start_benchmark(&st);
@@ -191,34 +191,31 @@ static void mac_bench(int algo, int size)
 	free(_key);
 }
 
-void benchmark_cipher(int init, int debug_level)
+void benchmark_cipher(int debug_level)
 {
+	int size = 16;
 	gnutls_global_set_log_function(tls_log_func);
 	gnutls_global_set_log_level(debug_level);
-	int size = 16;
 
-	if (init) {
-		gnutls_global_init();
-		gnutls_rnd(GNUTLS_RND_NONCE, data, sizeof(data));
-	}
+	gnutls_rnd(GNUTLS_RND_NONCE, data, sizeof(data));
 
-	printf("Checking ciphers, payload size: %u\n", size * 1024);
+	printf("Checking cipher-MAC combinations, payload size: %u\n", size * 1024);
 	cipher_mac_bench(GNUTLS_CIPHER_SALSA20_256, GNUTLS_MAC_SHA1, size);
 	cipher_mac_bench(GNUTLS_CIPHER_AES_128_CBC, GNUTLS_MAC_SHA1, size);
 	cipher_mac_bench(GNUTLS_CIPHER_AES_128_CBC, GNUTLS_MAC_SHA256,
 			 size);
 	cipher_bench(GNUTLS_CIPHER_AES_128_GCM, size, 1);
+	cipher_bench(GNUTLS_CIPHER_CHACHA20_POLY1305, size, 1);
 
+	printf("\nChecking MAC algorithms, payload size: %u\n", size * 1024);
 	mac_bench(GNUTLS_MAC_SHA1, size);
 	mac_bench(GNUTLS_MAC_SHA256, size);
 	mac_bench(GNUTLS_MAC_SHA512, size);
 
+	printf("\nChecking ciphers, payload size: %u\n", size * 1024);
 	cipher_bench(GNUTLS_CIPHER_3DES_CBC, size, 0);
-
 	cipher_bench(GNUTLS_CIPHER_AES_128_CBC, size, 0);
-
 	cipher_bench(GNUTLS_CIPHER_ARCFOUR, size, 0);
-
 	cipher_bench(GNUTLS_CIPHER_SALSA20_256, size, 0);
 
 	gnutls_global_deinit();
