@@ -2,15 +2,17 @@
 /**
  * \file putshell.c
  *
- * Time-stamp:      "2012-03-31 13:14:18 bkorb"
- *
  *  This module will interpret the options set in the tOptions
  *  structure and print them to standard out in a fashion that
  *  will allow them to be interpreted by the Bourne or Korn shells.
  *
+ * @addtogroup autoopts
+ * @{
+ */
+/*
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is Copyright (c) 1992-2012 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (C) 1992-2015 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -22,16 +24,22 @@
  *   The Modified Berkeley Software Distribution License
  *      See the file "COPYING.mbsd"
  *
- *  These files have the following md5sums:
+ *  These files have the following sha256 sums:
  *
- *  43b91e8ca915626ed3818ffb1b71248b pkg/libopts/COPYING.gplv3
- *  06a1a2e4760c90ea5e1dad8dfaac4d39 pkg/libopts/COPYING.lgplv3
- *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
+ *  8584710e9b04216a394078dc156b781d0b47e1729104d666658aecef8ee32e95  COPYING.gplv3
+ *  4379e7444a0e2ce2b12dd6f5a52a27a4d02d39d247901d3285c88cf0d37f477b  COPYING.lgplv3
+ *  13aa749a5b0a454917a944ed8fffc530b784f5ead522b1aacaf4ec8aa55a6239  COPYING.mbsd
  */
 
 /* = = = START-STATIC-FORWARD = = = */
+static size_t
+string_size(char const * scan, size_t nl_len);
+
+static char const *
+print_quoted_apostrophes(char const * str);
+
 static void
-print_quot_str(char const * pzStr);
+print_quot_str(char const * str);
 
 static void
 print_enumeration(tOptions * pOpts, tOptDesc * pOD);
@@ -43,20 +51,186 @@ static void
 print_stacked_arg(tOptions * pOpts, tOptDesc * pOD);
 
 static void
-print_reordering(tOptions * pOpts);
+print_reordering(tOptions * opts);
 /* = = = END-STATIC-FORWARD = = = */
 
-/*
- *  Make sure embedded single quotes come out okay.  The initial quote has
- *  been emitted and the closing quote will be upon return.
+/**
+ * Count the number of bytes required to represent a string as a
+ * compilable string.
+ *
+ * @param[in] scan    the text to be rewritten as a C program text string.
+ * @param[in] nl_len  the number of bytes used for each embedded newline.
+ *
+ * @returns the count, including the terminating NUL byte.
+ */
+static size_t
+string_size(char const * scan, size_t nl_len)
+{
+    /*
+     *  Start by counting the start and end quotes, plus the NUL.
+     */
+    size_t res_ln = 3;
+
+    for (;;) {
+        char ch = *(scan++);
+        if ((ch >= ' ') && (ch <= '~')) {
+
+            /*
+             * a backslash allowance for double quotes and baskslashes
+             */
+            res_ln += ((ch == '"') || (ch == '\\')) ? 2 : 1;
+        }
+
+        /*
+         *  When not a normal character, then count the characters
+         *  required to represent whatever it is.
+         */
+        else switch (ch) {
+        case NUL:
+            return res_ln;
+
+        case NL:
+            res_ln += nl_len;
+            break;
+
+        case HT:
+        case BEL:
+        case BS:
+        case FF:
+        case CR:
+        case VT:
+            res_ln += 2;
+            break;
+
+        default:
+            res_ln += 4; /* text len for \xNN */
+        }
+    }
+}
+
+/*=export_func  optionQuoteString
+ * private:
+ *
+ * what:  Print a string as quoted text suitable for a C compiler.
+ * arg:   + char const * + text  + a block of text to quote +
+ * arg:   + char const * + nl    + line splice text         +
+ *
+ * ret_type:  char const *
+ * ret_desc:  the allocated input string as a quoted string
+ *
+ * doc:
+ *  This is for internal use by autogen and autoopts.
+ *  It takes an input string and produces text the C compiler can process
+ *  to produce an exact copy of the original string.
+ *  The caller must deallocate the result.  Standard C strings and
+ *  K&R strings are distinguished by the "nl" string.
+=*/
+char const *
+optionQuoteString(char const * text, char const * nl)
+{
+    size_t   nl_len = strlen(nl);
+    size_t   out_sz = string_size(text, nl_len);
+    char *   out;
+    char *   res    = out = AGALOC(out_sz, "quot str");
+
+    *(out++) = '"';
+
+    for (;;) {
+        unsigned char ch = (unsigned char)*text;
+        if ((ch >= ' ') && (ch <= '~')) {
+            if ((ch == '"') || (ch == '\\'))
+                /*
+                 *  We must escape these characters in the output string
+                 */
+                *(out++) = '\\';
+            *(out++) = (char)ch;
+
+        } else switch (ch) {
+#       define   add_esc_ch(_ch)  { *(out++) = '\\'; *(out++) = (_ch); }
+        case BEL: add_esc_ch('a'); break;
+        case BS:  add_esc_ch('b'); break;
+        case HT:  add_esc_ch('t'); break;
+        case VT:  add_esc_ch('v'); break;
+        case FF:  add_esc_ch('f'); break;
+        case CR:  add_esc_ch('r'); break;
+
+        case LF:
+            /*
+             *  Place contiguous new-lines on a single line.
+             *  The current character is a NL, check the next one.
+             */
+            while (*++text == NL)
+                add_esc_ch('n');
+
+            /*
+             *  Insert a splice before starting next line
+             */
+            if (*text != NUL) {
+                memcpy(out, nl, nl_len);
+                out += nl_len;
+
+                continue; /* text is already at the next character */
+            }
+
+            add_esc_ch('n');
+            /* FALLTHROUGH */
+
+        case NUL:
+            /*
+             *  End of string.  Terminate the quoted output.  If necessary,
+             *  deallocate the text string.  Return the scan resumption point.
+             */
+            *(out++) = '"';
+            *(out++) = NUL;
+#ifndef NDEBUG
+            if ((out - res) > out_sz) {
+                fputs(misguess_len, stderr);
+                option_exits(EXIT_FAILURE);
+            }
+#endif
+            return res;
+
+        default:
+            /*
+             *  sprintf is safe here, because we already computed
+             *  the amount of space we will be using.  Assertion is above.
+             */
+            out += sprintf(out, MK_STR_OCT_FMT, ch);
+        }
+
+        text++;
+#       undef add_esc_ch
+    }
+}
+
+/**
+ *  Print out escaped apostorophes.
+ *
+ *  @param[in] str  the apostrophies to print
+ */
+static char const *
+print_quoted_apostrophes(char const * str)
+{
+    while (*str == APOSTROPHE) {
+        fputs(QUOT_APOS, stdout);
+        str++;
+    }
+    return str;
+}
+
+/**
+ *  Print a single quote (apostrophe quoted) string.
+ *  Other than somersaults for apostrophes, nothing else needs quoting.
+ *
+ *  @param[in] str  the string to print
  */
 static void
-print_quot_str(char const * pzStr)
+print_quot_str(char const * str)
 {
     /*
      *  Handle empty strings to make the rest of the logic simpler.
      */
-    if ((pzStr == NULL) || (*pzStr == NUL)) {
+    if ((str == NULL) || (*str == NUL)) {
         fputs(EMPTY_ARG, stdout);
         return;
     }
@@ -65,46 +239,42 @@ print_quot_str(char const * pzStr)
      *  Emit any single quotes/apostrophes at the start of the string and
      *  bail if that is all we need to do.
      */
-    while (*pzStr == '\'') {
-        fputs(QUOT_APOS, stdout);
-        pzStr++;
-    }
-    if (*pzStr == NUL)
+    str = print_quoted_apostrophes(str);
+    if (*str == NUL)
         return;
 
     /*
      *  Start the single quote string
      */
-    fputc('\'', stdout);
+    fputc(APOSTROPHE, stdout);
     for (;;) {
-        char const * pz = strchr(pzStr, '\'');
+        char const * pz = strchr(str, APOSTROPHE);
         if (pz == NULL)
             break;
 
         /*
          *  Emit the string up to the single quote (apostrophe) we just found.
          */
-        (void)fwrite(pzStr, (size_t)(pz - pzStr), (size_t)1, stdout);
-        fputc('\'', stdout);
-        pzStr = pz;
+        (void)fwrite(str, (size_t)(pz - str), (size_t)1, stdout);
 
         /*
-         *  Emit an escaped apostrophe for every one we find.
-         *  If that ends the string, do not re-open the single quotes.
+         * Close the current string, emit the apostrophes and re-open the
+         * string (IFF there is more text to print).
          */
-        while (*++pzStr == '\'')   fputs("\\'", stdout);
-        if (*pzStr == NUL)
+        fputc(APOSTROPHE, stdout);
+        str = print_quoted_apostrophes(pz);
+        if (*str == NUL)
             return;
 
-        fputc('\'', stdout);
+        fputc(APOSTROPHE, stdout);
     }
 
     /*
      *  If we broke out of the loop, we must still emit the remaining text
      *  and then close the single quote string.
      */
-    fputs(pzStr, stdout);
-    fputc('\'', stdout);
+    fputs(str, stdout);
+    fputc(APOSTROPHE, stdout);
 }
 
 static void
@@ -128,20 +298,17 @@ print_enumeration(tOptions * pOpts, tOptDesc * pOD)
 static void
 print_membership(tOptions * pOpts, tOptDesc * pOD)
 {
+    char const * svstr = pOD->optArg.argString;
     char const * pz;
     uintptr_t val = 1;
     printf(zOptNumFmt, pOpts->pzPROGNAME, pOD->pz_NAME,
            (int)(uintptr_t)(pOD->optCookie));
-    pOD->optCookie = (void*)(uintptr_t)~0UL;
+    pOD->optCookie = VOIDP(~0UL);
     (*(pOD->pOptProc))(OPTPROC_RETURN_VALNAME, pOD);
 
-    /*
-     *  We are building the typeset list.  The list returned starts with
-     *  'none + ' for use by option saving stuff.  We must ignore that.
-     */
-    pz = pOD->optArg.argString + 7;
+    pz = pOD->optArg.argString;
     while (*pz != NUL) {
-        printf("typeset -x -i %s_", pOD->pz_NAME);
+        printf("readonly %s_", pOD->pz_NAME);
         pz = SPN_PLUS_N_SPACE_CHARS(pz);
 
         for (;;) {
@@ -157,16 +324,15 @@ print_membership(tOptions * pOpts, tOptDesc * pOD)
     }
 
     AGFREE(pOD->optArg.argString);
-    pOD->optArg.argString = NULL;
-    pOD->fOptState &= ~OPTST_ALLOC_ARG;
+    pOD->optArg.argString = svstr;
 }
 
 static void
 print_stacked_arg(tOptions * pOpts, tOptDesc * pOD)
 {
-    tArgList*    pAL = (tArgList*)pOD->optCookie;
-    tCC**        ppz = pAL->apzArgs;
-    int          ct  = pAL->useCt;
+    tArgList *      pAL = (tArgList *)pOD->optCookie;
+    char const **   ppz = pAL->apzArgs;
+    int             ct  = pAL->useCt;
 
     printf(zOptCookieCt, pOpts->pzPROGNAME, pOD->pz_NAME, ct);
 
@@ -179,34 +345,24 @@ print_stacked_arg(tOptions * pOpts, tOptDesc * pOD)
     }
 }
 
+/**
+ * emit the arguments as readily parsed text.
+ * The program options are set by emitting the shell "set" command.
+ *
+ * @param[in] opts  the program options structure
+ */
 static void
-print_reordering(tOptions * pOpts)
+print_reordering(tOptions * opts)
 {
-    unsigned int optIx;
+    unsigned int ix;
 
     fputs(set_dash, stdout);
 
-    for (optIx = pOpts->curOptIdx;
-         optIx < pOpts->origArgCt;
-         optIx++) {
-
-        char* pzArg = pOpts->origArgVect[ optIx ];
-
-        if (strchr(pzArg, '\'') == NULL)
-            printf(arg_fmt, pzArg);
-
-        else {
-            fputs(" '", stdout);
-            for (;;) {
-                char ch = *(pzArg++);
-                switch (ch) {
-                case '\'':  fputs(apostrophy, stdout); break;
-                case NUL:   goto arg_done;
-                default:    fputc(ch, stdout); break;
-                }
-            } arg_done:;
-            fputc('\'', stdout);
-        }
+    for (ix = opts->curOptIdx;
+         ix < opts->origArgCt;
+         ix++) {
+        fputc(' ', stdout);
+        print_quot_str(opts->origArgVect[ ix ]);
     }
     fputs(init_optct, stdout);
 }
@@ -214,19 +370,19 @@ print_reordering(tOptions * pOpts)
 /*=export_func  optionPutShell
  * what:  write a portable shell script to parse options
  * private:
- * arg:   tOptions*, pOpts, the program options descriptor
+ * arg:   tOptions *, pOpts, the program options descriptor
  * doc:   This routine will emit portable shell script text for parsing
  *        the options described in the option definitions.
 =*/
 void
-optionPutShell(tOptions* pOpts)
+optionPutShell(tOptions * pOpts)
 {
     int  optIx = 0;
 
     printf(zOptCtFmt, pOpts->curOptIdx-1);
 
     do  {
-        tOptDesc* pOD = pOpts->pOptDesc + optIx;
+        tOptDesc * pOD = pOpts->pOptDesc + optIx;
 
         if ((pOD->fOptState & OPTST_NO_OUTPUT_MASK) != 0)
             continue;
@@ -247,7 +403,7 @@ optionPutShell(tOptions* pOpts)
          *  but copy over the set-state bits.
          */
         if (pOD->optActualIndex != optIx) {
-            tOptDesc* p   = pOpts->pOptDesc + pOD->optActualIndex;
+            tOptDesc * p  = pOpts->pOptDesc + pOD->optActualIndex;
             p->optArg     = pOD->optArg;
             p->fOptState &= OPTST_PERSISTENT_MASK;
             p->fOptState |= pOD->fOptState & ~OPTST_PERSISTENT_MASK;
@@ -352,7 +508,8 @@ optionPutShell(tOptions* pOpts)
     fflush(stdout);
 }
 
-/*
+/** @}
+ *
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
