@@ -16,11 +16,12 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include <stdio.h>
@@ -28,7 +29,7 @@
 
 #if defined(_WIN32)
 
-int main(void)
+int main()
 {
 	exit(77);
 }
@@ -43,21 +44,22 @@ int main(void)
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <errno.h>
-#include <signal.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/dtls.h>
-#include <assert.h>
 
 #include "utils.h"
 
 #define TXT1 "hello there"
-#define TXT1_SIZE (sizeof(TXT1) - 1)
+#define TXT1_SIZE (sizeof(TXT1)-1)
 
 #define TXT2 "2hello there"
-#define TXT2_SIZE (sizeof(TXT2) - 1)
+#define TXT2_SIZE (sizeof(TXT2)-1)
 
 static void terminate(void);
 
+/* This program tests the client hello verify in DTLS
+ */
+
 static void server_log_func(int level, const char *str)
 {
 	fprintf(stderr, "server|<%d>| %s", level, str);
@@ -68,11 +70,15 @@ static void client_log_func(int level, const char *str)
 	fprintf(stderr, "client|<%d>| %s", level, str);
 }
 
+/* A very basic TLS client, with anonymous authentication.
+ */
+
 #define MAX_BUF 1024
 
-static ssize_t push(gnutls_transport_ptr_t tr, const void *data, size_t len)
+static ssize_t
+push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 {
-	int fd = (long int)tr;
+	int fd = (long int) tr;
 	static int counter = 0;
 
 	if (counter % 2 == 0) {
@@ -83,7 +89,7 @@ static ssize_t push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 	return send(fd, data, len, 0);
 }
 
-static void client(int fd, const char *prio)
+static void client(int fd)
 {
 	int ret;
 	gnutls_anon_client_credentials_t anoncred;
@@ -103,10 +109,12 @@ static void client(int fd, const char *prio)
 	 */
 	gnutls_init(&session, GNUTLS_CLIENT | GNUTLS_DATAGRAM);
 	gnutls_dtls_set_mtu(session, 1500);
-	gnutls_handshake_set_timeout(session, get_timeout());
+	gnutls_handshake_set_timeout(session, 20 * 1000);
 
 	/* Use default priorities */
-	assert(gnutls_priority_set_direct(session, prio, NULL) >= 0);
+	gnutls_priority_set_direct(session,
+				   "NONE:+VERS-DTLS-ALL:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+ANON-ECDH:+CURVE-ALL",
+				   NULL);
 
 	/* put the anonymous credentials to the current session
 	 */
@@ -118,7 +126,8 @@ static void client(int fd, const char *prio)
 	 */
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 
 	if (ret < 0) {
 		fail("client: Handshake failed\n");
@@ -131,18 +140,17 @@ static void client(int fd, const char *prio)
 
 	if (debug)
 		success("client: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	gnutls_transport_set_push_function(session, push);
 	do {
 		ret = gnutls_record_send(session, TXT1, TXT1_SIZE);
 		if (ret == GNUTLS_E_AGAIN) {
-			if (debug)
-				success("discarding\n");
+			if (debug) success("discarding\n");
 			gnutls_record_discard_queued(session);
 		}
-
+			
 	} while (ret == GNUTLS_E_INTERRUPTED);
 
 	do {
@@ -160,19 +168,20 @@ static void client(int fd, const char *prio)
 	gnutls_global_deinit();
 }
 
+
 /* These are global */
 pid_t child;
 
 static void terminate(void)
 {
 	int status;
-	assert(child);
+
 	kill(child, SIGTERM);
 	wait(&status);
 	exit(1);
 }
 
-static void server(int fd, const char *prio)
+static void server(int fd)
 {
 	int ret;
 	gnutls_anon_server_credentials_t anoncred;
@@ -191,13 +200,15 @@ static void server(int fd, const char *prio)
 	gnutls_anon_allocate_server_credentials(&anoncred);
 
 	gnutls_init(&session, GNUTLS_SERVER | GNUTLS_DATAGRAM);
-	gnutls_handshake_set_timeout(session, get_timeout());
+	gnutls_handshake_set_timeout(session, 20 * 1000);
 	gnutls_dtls_set_mtu(session, 1500);
 
 	/* avoid calling all the priority functions, since the defaults
 	 * are adequate.
 	 */
-	assert(gnutls_priority_set_direct(session, prio, NULL) >= 0);
+	gnutls_priority_set_direct(session,
+				   "NONE:+VERS-DTLS1.0:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+ANON-ECDH:+CURVE-ALL",
+				   NULL);
 
 	gnutls_credentials_set(session, GNUTLS_CRD_ANON, anoncred);
 
@@ -205,7 +216,8 @@ static void server(int fd, const char *prio)
 
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 	if (ret < 0) {
 		close(fd);
 		gnutls_deinit(session);
@@ -218,8 +230,8 @@ static void server(int fd, const char *prio)
 
 	if (debug)
 		success("server: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	/* see the Getting peer's information example */
 	/* print_info(session); */
@@ -241,6 +253,7 @@ static void server(int fd, const char *prio)
 		terminate();
 	}
 
+
 	/* do not wait for the peer to close the connection.
 	 */
 	gnutls_bye(session, GNUTLS_SHUT_WR);
@@ -256,13 +269,11 @@ static void server(int fd, const char *prio)
 		success("server: finished\n");
 }
 
-static void start(const char *prio)
+void doit(void)
 {
 	int fd[2];
 	int ret;
 
-	signal(SIGPIPE, SIG_IGN);
-
 	ret = socketpair(AF_UNIX, SOCK_STREAM, 0, fd);
 	if (ret < 0) {
 		perror("socketpair");
@@ -280,20 +291,16 @@ static void start(const char *prio)
 		int status;
 		/* parent */
 
-		server(fd[0], prio);
+		server(fd[0]);
 		wait(&status);
-		check_wait_status(status);
+		if (WEXITSTATUS(status) != 0)
+			fail("Child died with status %d\n",
+			     WEXITSTATUS(status));
 	} else {
 		close(fd[0]);
-		client(fd[1], prio);
+		client(fd[1]);
 		exit(0);
 	}
 }
 
-void doit(void)
-{
-	start("NONE:+VERS-DTLS1.0:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+ANON-ECDH:+CURVE-ALL");
-	start("NONE:+VERS-DTLS1.2:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+ANON-ECDH:+CURVE-ALL");
-}
-
-#endif /* _WIN32 */
+#endif				/* _WIN32 */
