@@ -376,7 +376,9 @@ void run(unsigned do_thread)
 		close(fd[1]);
 		client(fd[0], do_thread);
 		wait(&status);
-		check_wait_status(status);
+		if (WEXITSTATUS(status) != 0)
+			fail("Child died with status %d\n",
+			     WEXITSTATUS(status));
 	} else {
 		close(fd[0]);
 		server(fd[1], 1-do_thread);
