@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2012 Free Software Foundation, Inc.
+ * Copyright (C) 2012-2013 Free Software Foundation, Inc.
+ * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -49,12 +50,12 @@ int main()
 
 #include "utils.h"
 
-#define REL_LAYER
-
 static int test_finished = 0;
 static void terminate(void);
 
-/* This program tests the rehandshake in DTLS
+/* This program tests whether messages in DTLS are received
+ * with the expected sequence. That is whether the message
+ * sequence numbers returned correspond to the received messages.
  */
 
 /*
@@ -87,10 +88,9 @@ static pid_t child;
 
 static int msg_seq[] =
     { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 10, 16, 5, 32, 11, 11, 11, 11, 12,
-10, 13, 14,
-	15, 16, 17, 19, 20, 18, 22, 24, 23, 25, 26, 27, 29, 28, 29, 29, 30,
-	    31, 32, 33, 34, 35, 37, 36, 38, 39,
-	42, 37, 40, 41, 41, -1
+	10, 13, 14, 15, 16, 17, 19, 20, 18, 22, 24, 23, 25, 26, 27, 29, 28,
+	29, 29, 30, 31, 32, 33, 34, 35, 37, 36, 38, 39, 42, 37, 40, 41, 41,
+	-1
 };
 
 static unsigned int current = 0;
@@ -99,8 +99,7 @@ static unsigned int pos = 0;
 unsigned char *stored_messages[MAX_SEQ];
 unsigned int stored_sizes[MAX_SEQ];
 
-static ssize_t
-odd_push(gnutls_transport_ptr_t tr, const void *data, size_t len)
+static ssize_t odd_push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 {
 	ssize_t ret;
 	unsigned i;
@@ -119,22 +118,21 @@ odd_push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 			if (stored_messages[msg_seq[i]] != NULL) {
 				do {
 					ret =
-					    send((long int) tr,
+					    send((long int)tr,
 						 stored_messages[msg_seq
 								 [i]],
-						 stored_sizes[msg_seq[i]],
-						 0);
+						 stored_sizes[msg_seq[i]], 0);
 				}
-				while (ret == -1 && errno == EAGAIN);
+				while (ret == -1 && (errno == EAGAIN || errno == EINTR));
 				pos++;
 			} else
 				break;
 		}
-	} else if (msg_seq[current] == (int) current) {
+	} else if (msg_seq[current] == (int)current) {
 		do {
-			ret = send((long int) tr, data, len, 0);
+			ret = send((long int)tr, data, len, 0);
 		}
-		while (ret == -1 && errno == EAGAIN);
+		while (ret == -1 && (errno == EAGAIN || errno == EINTR));
 
 		current++;
 		pos++;
@@ -143,11 +141,11 @@ odd_push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 	} else if (stored_messages[msg_seq[current]] != NULL) {
 		do {
 			ret =
-			    send((long int) tr,
+			    send((long int)tr,
 				 stored_messages[msg_seq[current]],
 				 stored_sizes[msg_seq[current]], 0);
 		}
-		while (ret == -1 && errno == EAGAIN);
+		while (ret == -1 && (errno == EAGAIN || errno == EINTR));
 		current++;
 		pos++;
 		return ret;
@@ -158,17 +156,16 @@ odd_push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 	return len;
 }
 
-static ssize_t
-n_push(gnutls_transport_ptr_t tr, const void *data, size_t len)
+static ssize_t n_push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 {
-	return send((unsigned long) tr, data, len, 0);
+	return send((unsigned long)tr, data, len, 0);
 }
 
 /* The first five messages are handshake. Thus corresponds to msg_seq+5 */
 static int recv_msg_seq[] =
     { 1, 2, 3, 4, 5, 6, 12, 28, 7, 8, 9, 10, 11, 13, 15, 16, 14, 18, 20,
-19, 21, 22,
-	23, 25, 24, 26, 27, 29, 30, 31, 33, 32, 34, 35, 38, 36, 37, -1
+	19, 21, 22, 23, 25, 24, 26, 27, 29, 30, 31, 33, 32, 34, 35, 38, 36, 37,
+	    -1
 };
 
 static void client(int fd)
@@ -178,14 +175,7 @@ static void client(int fd)
 	char buffer[MAX_BUF + 1];
 	gnutls_anon_client_credentials_t anoncred;
 	unsigned char seq[8];
-	uint64_t useq;
-	unsigned current = 0;
-#ifndef REL_LAYER
-	struct timespec ts;
-
-	ts.tv_sec = 0;
-	ts.tv_nsec = 100 * 1000 * 1000;
-#endif
+	uint32_t useq;
 
 	memset(buffer, 0, sizeof(buffer));
 
@@ -196,7 +186,7 @@ static void client(int fd)
 
 	if (debug) {
 		gnutls_global_set_log_function(client_log_func);
-		gnutls_global_set_log_level(99);
+		gnutls_global_set_log_level(2);
 	}
 
 	gnutls_anon_allocate_client_credentials(&anoncred);
@@ -204,6 +194,7 @@ static void client(int fd)
 	/* Initialize TLS session
 	 */
 	gnutls_init(&session, GNUTLS_CLIENT | GNUTLS_DATAGRAM);
+	gnutls_dtls_set_timeouts(session, 50 * 1000, 600 * 1000);
 	gnutls_heartbeat_enable(session, GNUTLS_HB_PEER_ALLOWED_TO_SEND);
 	gnutls_dtls_set_mtu(session, 1500);
 
@@ -247,29 +238,25 @@ static void client(int fd)
 
 		if (ret > 0) {
 			useq =
-			    seq[3] | (seq[2] << 8) | (seq[1] << 16) |
-			    (seq[0] << 24);
-			useq <<= 32;
-			useq |=
 			    seq[7] | (seq[6] << 8) | (seq[5] << 16) |
 			    (seq[4] << 24);
 
+			if (debug)
+				success("received %u\n", (unsigned int)useq);
+
 			if (recv_msg_seq[current] == -1) {
 				fail("received message sequence differs\n");
 				terminate();
 			}
 
-			if ((uint32_t) recv_msg_seq[current] !=
-			    (uint32_t) useq) {
-				fail("received message sequence differs\n");
+			if (((uint32_t)recv_msg_seq[current]) != useq) {
+				fail("received message sequence differs (current: %u, got: %u, expected: %u)\n",
+				     (unsigned)current, (unsigned)useq, (unsigned)recv_msg_seq[current]);
 				terminate();
 			}
 
 			current++;
 		}
-#ifndef REL_LAYER
-		nanosleep(&ts, NULL);
-#endif
 	}
 	while ((ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED
 		|| ret > 0));
@@ -285,7 +272,6 @@ static void client(int fd)
 	gnutls_global_deinit();
 }
 
-
 static void terminate(void)
 {
 	int status;
@@ -301,23 +287,18 @@ static void server(int fd)
 	gnutls_session_t session;
 	gnutls_anon_server_credentials_t anoncred;
 	char c;
-#ifndef REL_LAYER
-	struct timespec ts;
-
-	ts.tv_sec = 0;
-	ts.tv_nsec = 100 * 1000 * 1000;
-#endif
 
 	global_init();
 
 	if (debug) {
 		gnutls_global_set_log_function(server_log_func);
-		gnutls_global_set_log_level(4711);
+		gnutls_global_set_log_level(2);
 	}
 
 	gnutls_anon_allocate_server_credentials(&anoncred);
 
 	gnutls_init(&session, GNUTLS_SERVER | GNUTLS_DATAGRAM);
+	gnutls_dtls_set_timeouts(session, 50 * 1000, 600 * 1000);
 	gnutls_transport_set_push_function(session, odd_push);
 	gnutls_heartbeat_enable(session, GNUTLS_HB_PEER_ALLOWED_TO_SEND);
 	gnutls_dtls_set_mtu(session, 1500);
@@ -356,23 +337,16 @@ static void server(int fd)
 		do {
 			ret = gnutls_record_send(session, &c, 1);
 		}
-		while (ret == GNUTLS_E_AGAIN
-		       || ret == GNUTLS_E_INTERRUPTED);
+		while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
 
 		if (ret < 0) {
 			fail("send: %s\n", gnutls_strerror(ret));
 			terminate();
 		}
-#ifndef REL_LAYER
-		nanosleep(&ts, NULL);
-#endif
 	}
 	while (test_finished == 0);
 
 	gnutls_transport_set_push_function(session, n_push);
-#ifndef REL_LAYER
-	nanosleep(&ts, NULL);
-#endif
 	do {
 		ret = gnutls_bye(session, GNUTLS_SHUT_WR);
 	}
@@ -394,11 +368,7 @@ static void start(void)
 	int fd[2];
 	int ret;
 
-#ifdef REL_LAYER
 	ret = socketpair(AF_UNIX, SOCK_STREAM, 0, fd);
-#else
-	ret = socketpair(AF_UNIX, SOCK_DGRAM, 0, fd);
-#endif
 	if (ret < 0) {
 		perror("socketpair");
 		exit(1);
@@ -414,7 +384,7 @@ static void start(void)
 	if (child) {
 		int status;
 		/* parent */
-
+		close(fd[1]);
 		server(fd[0]);
 		wait(&status);
 		if (WEXITSTATUS(status) != 0)
