@@ -109,11 +109,7 @@ void doit(void)
 	gnutls_credentials_set(server, GNUTLS_CRD_CERTIFICATE,
 			       serverx509cred);
 	gnutls_priority_set_direct(server,
-#ifndef ENABLE_FIPS140
-				   "NORMAL:-CIPHER-ALL:+ARCFOUR-128",
-#else
 				   "NORMAL:-CIPHER-ALL:+AES-128-CBC",
-#endif
 				   NULL);
 	gnutls_transport_set_push_function(server, server_push);
 	gnutls_transport_set_pull_function(server, server_pull);
@@ -124,7 +120,7 @@ void doit(void)
 	gnutls_init(&client, GNUTLS_CLIENT);
 	gnutls_credentials_set(client, GNUTLS_CRD_CERTIFICATE,
 			       clientx509cred);
-	gnutls_priority_set_direct(client, "NORMAL:+ARCFOUR-128", NULL);
+	gnutls_priority_set_direct(client, "NORMAL", NULL);
 	gnutls_transport_set_push_function(client, client_push);
 	gnutls_transport_set_pull_function(client, client_pull);
 	gnutls_transport_set_ptr(client, client);
