@@ -78,28 +78,16 @@ static void dump(const char *name, uint8_t *data, unsigned data_size)
 	fprintf(stderr, "\n");
 }
 
-static void terminate(void)
-{
-	int status = 0;
-
-	if (child) {
-		kill(child, SIGTERM);
-		wait(&status);
-	}
-	exit(1);
-}
-
 static void client(int fd)
 {
 	gnutls_session_t session;
 	int ret;
+	gnutls_datum_t proto;
 	gnutls_anon_client_credentials_t anoncred;
 	gnutls_datum_t mac_key, iv, cipher_key;
 	gnutls_datum_t read_mac_key, read_iv, read_cipher_key;
-	unsigned char rseq_number[8];
-	unsigned char wseq_number[8];
+	unsigned char seq_number[8];
 	unsigned char key_material[512], *p;
-	unsigned i;
 	unsigned block_size, hash_size, key_size, iv_size;
 	const char *err;
 	/* Need to enable anonymous KX specifically. */
@@ -171,7 +159,7 @@ static void client(int fd)
 	block_size = 2*hash_size + 2*key_size + 2 *iv_size;
 
 	ret = gnutls_prf(session, 13, "key expansion", 1, 0, NULL, block_size,
-			 (void*)key_material);
+                         (void*)key_material);
 	if (ret < 0) {
 		fprintf(stderr, "error in %d\n", __LINE__);
 		gnutls_perror(ret);
@@ -179,33 +167,20 @@ static void client(int fd)
 	}
 	p = key_material;
 
-	/* check whether the key material matches our calculations */
-	ret = gnutls_record_get_state(session, 0, &mac_key, &iv, &cipher_key, wseq_number);
+	ret = gnutls_record_get_state(session, 0, &mac_key, &iv, &cipher_key, seq_number);
 	if (ret < 0) {
 		fprintf(stderr, "error in %d\n", __LINE__);
 		gnutls_perror(ret);
 		exit(1);
 	}
 
-	if (memcmp(wseq_number, "\x00\x00\x00\x00\x00\x00\x00\x01", 8) != 0) {
-		dump("wseq:", wseq_number, 8);
-		fprintf(stderr, "error in %d\n", __LINE__);
-		exit(1);
-	}
-
-	ret = gnutls_record_get_state(session, 1, &read_mac_key, &read_iv, &read_cipher_key, rseq_number);
+	ret = gnutls_record_get_state(session, 1, &read_mac_key, &read_iv, &read_cipher_key, seq_number);
 	if (ret < 0) {
 		fprintf(stderr, "error in %d\n", __LINE__);
 		gnutls_perror(ret);
 		exit(1);
 	}
 
-	if (memcmp(rseq_number, "\x00\x00\x00\x00\x00\x00\x00\x01", 8) != 0) {
-		dump("rseq:", rseq_number, 8);
-		fprintf(stderr, "error in %d\n", __LINE__);
-		exit(1);
-	}
-
 	if (hash_size != mac_key.size || memcmp(p, mac_key.data, hash_size) != 0) {
 		dump("MAC:", mac_key.data, mac_key.size);
 		dump("Block:", key_material, block_size);
@@ -245,39 +220,15 @@ static void client(int fd)
 		exit(1);
 	}
 
-	/* check sequence numbers */
-	for (i=0;i<5;i++) {
-		ret = gnutls_record_send(session, "hello", 5);
-		if (ret < 0) {
-			fail("gnutls_record_send: %s\n", gnutls_strerror(ret));
-		}
+	/* check whether the key material matches our calculations */
+	
+
+	if (debug) {
+		fprintf(stderr, "selected protocol: %.*s\n",
+			(int) proto.size, proto.data);
 	}
 
-	ret = gnutls_record_get_state(session, 0, NULL, NULL, NULL, wseq_number);
-	if (ret < 0) {
-		fprintf(stderr, "error in %d\n", __LINE__);
-		gnutls_perror(ret);
-		exit(1);
-	}
 
-	if (memcmp(wseq_number, "\x00\x00\x00\x00\x00\x00\x00\x06", 8) != 0) {
-		dump("wseq:", wseq_number, 8);
-		fprintf(stderr, "error in %d\n", __LINE__);
-		exit(1);
-	}
-
-	ret = gnutls_record_get_state(session, 1, NULL, NULL, NULL, rseq_number);
-	if (ret < 0) {
-		fprintf(stderr, "error in %d\n", __LINE__);
-		gnutls_perror(ret);
-		exit(1);
-	}
-
-	if (memcmp(rseq_number, "\x00\x00\x00\x00\x00\x00\x00\x01", 8) != 0) {
-		dump("wseq:", wseq_number, 8);
-		fprintf(stderr, "error in %d\n", __LINE__);
-		exit(1);
-	}
 	gnutls_bye(session, GNUTLS_SHUT_WR);
 
 	close(fd);
@@ -289,13 +240,21 @@ static void client(int fd)
 	gnutls_global_deinit();
 }
 
+static void terminate(void)
+{
+	int status = 0;
+
+	kill(child, SIGTERM);
+	wait(&status);
+	exit(1);
+}
+
 static void server(int fd)
 {
 	int ret;
 	gnutls_session_t session;
 	gnutls_anon_server_credentials_t anoncred;
 	gnutls_dh_params_t dh_params;
-	char buf[128];
 	const gnutls_datum_t p3 =
 	    { (unsigned char *) pkcs3, strlen(pkcs3) };
 
@@ -319,7 +278,7 @@ static void server(int fd)
 	 * are adequate.
 	 */
 	ret = gnutls_priority_set_direct(session,
-				   "NORMAL:+ANON-DH:+ANON-ECDH:-VERS-ALL:+VERS-TLS1.0", NULL);
+				   "NORMAL:+ANON-DH:+ANON-ECDH", NULL);
 	if (ret < 0) {
 		fail("server: priority set failed (%s)\n\n",
 		     gnutls_strerror(ret));
@@ -349,14 +308,6 @@ static void server(int fd)
 			gnutls_protocol_get_name
 			(gnutls_protocol_get_version(session)));
 
-	do {
-		ret = gnutls_record_recv(session, buf, sizeof(buf));
-	} while(ret > 0);
-
-	if (ret < 0) {
-		fail("error: %s\n", gnutls_strerror(ret));
-	}
-
 	/* do not wait for the peer to close the connection.
 	 */
 	gnutls_bye(session, GNUTLS_SHUT_WR);
@@ -397,7 +348,9 @@ static void start(void)
 
 		server(fd[0]);
 		wait(&status);
-		check_wait_status(status);
+		if (WEXITSTATUS(status) != 0)
+			fail("Child died with status %d\n",
+			     WEXITSTATUS(status));
 	} else {
 		close(fd[0]);
 		client(fd[1]);
