@@ -17,7 +17,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,12 +31,10 @@
 #include <gnutls/x509.h>
 #include "utils.h"
 
-#ifdef ENABLE_NON_SUITEB_CURVES
 static void tls_log_func(int level, const char *str)
 {
 	fprintf(stderr, "<%d>| %s", level, str);
 }
-#endif
 
 void doit(void)
 {
@@ -66,8 +65,9 @@ void doit(void)
 		filename = "pkcs12-decode/pkcs12_5certs.p12";
 
 	if (debug)
-		success("Reading PKCS#12 blob from `%s' using password `%s'.\n",
-			filename, password);
+		success
+		    ("Reading PKCS#12 blob from `%s' using password `%s'.\n",
+		     filename, password);
 
 	ret = gnutls_load_file(filename, &data);
 	if (ret < 0)
@@ -81,9 +81,10 @@ void doit(void)
 	if (debug)
 		success("Read file OK\n");
 
-	ret = gnutls_pkcs12_simple_parse(pkcs12, password, &pkey, &chain,
-					 &chain_size, &extras, &extras_size,
-					 NULL, 0);
+	ret =
+	    gnutls_pkcs12_simple_parse(pkcs12, password, &pkey, &chain,
+				       &chain_size, &extras, &extras_size,
+				       NULL, 0);
 	if (ret < 0)
 		fail("pkcs12_simple_parse failed %d: %s\n", ret,
 		     gnutls_strerror(ret));
@@ -107,7 +108,8 @@ void doit(void)
 		success("dn: %s\n", dn);
 
 		dn_size = sizeof(dn);
-		ret = gnutls_x509_crt_get_issuer_dn(chain[0], dn, &dn_size);
+		ret =
+		    gnutls_x509_crt_get_issuer_dn(chain[0], dn, &dn_size);
 		if (ret < 0)
 			fail("crt_get_dn failed %d: %s\n", ret,
 			     gnutls_strerror(ret));
@@ -132,8 +134,9 @@ void doit(void)
 		fail("gnutls_x509_privkey_init failed %d: %s\n", ret,
 		     gnutls_strerror(ret));
 
-	ret = gnutls_x509_privkey_import2(pkey, &data, GNUTLS_X509_FMT_DER,
-					  password, 0);
+	ret =
+	    gnutls_x509_privkey_import2(pkey, &data, GNUTLS_X509_FMT_DER,
+					password, 0);
 	if (ret < 0)
 		fail("gnutls_x509_privkey_import2 failed %d: %s\n", ret,
 		     gnutls_strerror(ret));
