@@ -27,6 +27,20 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <time.h>
+#include <unistd.h>
+#include <errno.h>
+#ifndef _WIN32
+# include <netinet/in.h>
+# include <sys/socket.h>
+#else
+#ifdef _WIN32
+# include <windows.h>		/* for Sleep */
+# include <winbase.h>  
+#endif
+
+#endif
+#include <sys/types.h>
 
 #include "utils.h"
 
@@ -54,6 +68,39 @@ void fail(const char *format, ...)
 	exit(1);
 }
 
+void fail_ignore(const char *format, ...)
+{
+	char str[1024];
+	va_list arg_ptr;
+
+	va_start(arg_ptr, format);
+	vsnprintf(str, sizeof(str), format, arg_ptr);
+	va_end(arg_ptr);
+	fputs(str, stderr);
+	error_count++;
+	exit(77);
+}
+
+void sec_sleep(int sec)
+{
+	int ret;
+#ifdef HAVE_NANOSLEEP
+	struct timespec ts;
+
+	ts.tv_sec = sec;
+	ts.tv_nsec = 0;
+	do {
+		ret = nanosleep(&ts, NULL);
+	} while (ret == -1 && errno == EINTR);
+	if (ret == -1)
+		abort();
+#else
+	do {
+		ret = sleep(sec);
+	} while (ret == -1 && errno == EINTR);
+#endif
+}
+
 void success(const char *format, ...)
 {
 	char str[1024];
@@ -151,3 +198,4 @@ int main(int argc, char *argv[])
 
 	return error_count ? 1 : 0;
 }
+
