@@ -189,10 +189,14 @@ static time_t mytime(time_t * t)
 void doit(void)
 {
 	int ret;
+	const char *path;
 	gnutls_datum_t data;
-	gnutls_x509_crt_t server_crt, ca_crt, ca_crt2;
+	gnutls_x509_crt_t server_crt, ca_crt2;
 	gnutls_x509_trust_list_t tl;
 	unsigned int status;
+	gnutls_typed_vdata_st vdata;
+	gnutls_digest_algorithm_t hash;
+	unsigned int mand;
 
 	/* this must be called once in the program
 	 */
@@ -205,39 +209,46 @@ void doit(void)
 
 	/* test for gnutls_certificate_get_issuer() */
 	gnutls_x509_trust_list_init(&tl, 0);
+
 	gnutls_x509_crt_init(&server_crt);
-	gnutls_x509_crt_init(&ca_crt);
 	gnutls_x509_crt_init(&ca_crt2);
 
+	path = getenv("X509CERTDIR");
+	if (!path)
+		path = "./x509cert-dir";
+
 	ret =
 	    gnutls_x509_crt_import(server_crt, &cert, GNUTLS_X509_FMT_PEM);
 	if (ret < 0)
 		fail("gnutls_x509_crt_import");
 
-	ret = gnutls_x509_crt_import(ca_crt, &ca, GNUTLS_X509_FMT_PEM);
-	if (ret < 0)
-		fail("gnutls_x509_crt_import");
+	ret = gnutls_x509_crt_get_signature_algorithm(server_crt);
+	if (ret != GNUTLS_SIGN_RSA_SHA1) {
+		fail("detected wrong algorithm: %s\n", gnutls_sign_get_name(ret));
+		exit(1);
+	}
+
+	ret = gnutls_x509_crt_get_preferred_hash_algorithm(server_crt, &hash, &mand);
+	if (ret < 0) {
+		fail("error in gnutls_x509_crt_get_preferred_hash_algorithm: %s\n", gnutls_strerror(ret));
+		exit(1);
+	}
+
+	if (mand != 0 || hash != GNUTLS_DIG_SHA256) {
+		fail("gnutls_x509_crt_get_preferred_hash_algorithm returned: %s/%d\n", gnutls_digest_get_name(hash), mand);
+		exit(1);
+	}
 
 	ret = gnutls_x509_crt_import(ca_crt2, &ca, GNUTLS_X509_FMT_PEM);
 	if (ret < 0)
 		fail("gnutls_x509_crt_import");
 
-	ret = gnutls_x509_trust_list_add_cas(tl, &ca_crt, 1, 0);
-	if (ret < 0)
-		fail("gnutls_x509_trust_list_add_cas");
-
 	ret =
 	    gnutls_x509_trust_list_add_named_crt(tl, server_crt, NAME,
 						 NAME_SIZE, 0);
 	if (ret < 0)
 		fail("gnutls_x509_trust_list_add_named_crt");
 
-	ret =
-	    gnutls_x509_trust_list_verify_crt(tl, &server_crt, 1, 0,
-					      &status, NULL);
-	if (ret < 0 || status != 0)
-		fail("gnutls_x509_trust_list_verify_crt\n");
-
 	ret =
 	    gnutls_x509_trust_list_verify_named_crt(tl, server_crt, NAME,
 						    NAME_SIZE, 0, &status,
@@ -262,6 +273,39 @@ void doit(void)
 		fail("gnutls_x509_trust_list_verify_named_crt: %d\n",
 		     __LINE__);
 
+	/* check whether the name-only verification works */
+	vdata.type = GNUTLS_DT_DNS_HOSTNAME;
+	vdata.data = (void*)NAME;
+	vdata.size = NAME_SIZE;
+	ret =
+	    gnutls_x509_trust_list_verify_crt2(tl, &server_crt, 1, &vdata, 1,
+					       0, &status, NULL);
+	if (ret < 0 || status != 0)
+		fail("gnutls_x509_trust_list_verify_crt2 - 1: status: %x\n", status);
+
+	vdata.type = GNUTLS_DT_DNS_HOSTNAME;
+	vdata.data = (void*)NAME;
+	vdata.size = NAME_SIZE-2;
+	ret =
+	    gnutls_x509_trust_list_verify_crt2(tl, &server_crt, 1, &vdata, 1,
+					       0, &status, NULL);
+	if (ret < 0 || status == 0)
+		fail("gnutls_x509_trust_list_verify_crt2 - 2: status: %x\n", status);
+
+
+	/* check whether the key verification works */
+	ret = gnutls_x509_trust_list_add_trust_dir(tl, path, NULL, GNUTLS_X509_FMT_PEM, 0, 0);
+	if (ret != 1)
+		fail("gnutls_x509_trust_list_add_trust_dir: %d\n", ret);
+
+	ret =
+	    gnutls_x509_trust_list_verify_crt(tl, &server_crt, 1, 0,
+					      &status, NULL);
+	if (ret < 0 || status != 0)
+		fail("gnutls_x509_trust_list_verify_crt\n");
+
+
+
 	/* test convenience functions in verify-high2.c */
 	data.data = cert_pem;
 	data.size = strlen((char *) cert_pem);
