@@ -124,6 +124,7 @@ static void client(int sd)
 	gnutls_session_t session;
 	char buffer[MAX_BUF + 1];
 	gnutls_certificate_credentials_t xcred;
+	gnutls_certificate_credentials_t tst_cred;
 
 	global_init();
 
@@ -145,7 +146,7 @@ static void client(int sd)
 	gnutls_init(&session, GNUTLS_CLIENT);
 
 	/* Use default priorities */
-	gnutls_set_default_priority(session);
+	gnutls_priority_set_direct(session, "NORMAL", NULL);
 
 	/* put the x509 credentials to the current session
 	 */
@@ -174,6 +175,14 @@ static void client(int sd)
 	if (debug)
 		print_info(session);
 
+	ret = gnutls_credentials_get(session, GNUTLS_CRD_CERTIFICATE, (void**)&tst_cred);
+	if (ret < 0) {
+		fail("client: gnutls_credentials_get failed: %s\n", gnutls_strerror(ret));
+	}
+	if (tst_cred != xcred) {
+		fail("client: gnutls_credentials_get returned invalid value\n");
+	}
+
 	ret = gnutls_record_send(session, MSG, strlen(MSG));
 
 	if (ret == strlen(MSG)) {
@@ -253,7 +262,7 @@ static gnutls_session_t initialize_tls_session(void)
 	/* avoid calling all the priority functions, since the defaults
 	 * are adequate.
 	 */
-	gnutls_set_default_priority(session);
+	gnutls_priority_set_direct(session, "NORMAL", NULL);
 
 	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, x509_cred);
 
